/* header file for autores_bucket1d.c */

typedef struct {
  double minRangeX;
  double maxRangeX;
  int xCells;
  int *startLinkIndexs;    /* [xCells] */
  int startLinkIndexOutOfRange;
  int startLinkIndexOfFreePool;
  int maxLinks;
  int nLinks;
  int *nextLinkIndexs;  /* [maxLinks] */
  int *linkItemIds;     /* [maxLinks] */
} AutoRes_Bucket1D;



/* Bucket1D */

void AutoRes_Bucket1D_initialize
(AutoRes_Bucket1D *self,
 double minRangeX, 
 double maxRangeX,
 int xCells, 
 int nAverageItemsPerCell);

void AutoRes_Bucket1D_terminate
(AutoRes_Bucket1D *self);

void AutoRes_Bucket1D_getMinRange 
(AutoRes_Bucket1D *self,
 double *minRangeX_OUT);

void AutoRes_Bucket1D_getMaxRange 
(AutoRes_Bucket1D *self,
 double *maxRangeX_OUT);

void AutoRes_Bucket1D_getNCells 
(AutoRes_Bucket1D *self,
 int *xCells_OUT);

void AutoRes_Bucket1D_getIndexWithin 
(AutoRes_Bucket1D *self,
 double minRangeX,
 double maxRangeX,
 int *cx0_OUT, 
 int *cx1_OUT);

void AutoRes_Bucket1D_getIndexAt 
(AutoRes_Bucket1D *self,
 double x,
 int *cx_OUT);

#ifdef NDEBUG

#define AutoRes_Bucket1D_isOutOfRange(self,cx) \
  (cx) < 0 || (self)->xCells <= (cx) 

#define AutoRes_Bucket1D_startLinkIndexIn(self,cx) \
 (self)->startLinkIndexs[cx] 

#define AutoRes_Bucket1D_startLinkIndexOutOfRange(self) \
  (self)->startLinkIndexOutOfRange 

#define AutoRes_Bucket1D_nextLinkIndex(self,linkIndex) \
  (self)->nextLinkIndexs[(linkIndex)]

#define AutoRes_Bucket1D_itemId(self,linkIndex) \
  (self)->linkItemIds[(linkIndex)]

#else

int AutoRes_Bucket1D_isOutOfRange 
(AutoRes_Bucket1D *self,
 int cx);

int AutoRes_Bucket1D_startLinkIndexIn 
(AutoRes_Bucket1D *self,
 int cx);

int AutoRes_Bucket1D_startLinkIndexOutOfRange 
(AutoRes_Bucket1D *self);

int AutoRes_Bucket1D_nextLinkIndex 
(AutoRes_Bucket1D *self,
 int linkIndex);

int AutoRes_Bucket1D_itemId 
(AutoRes_Bucket1D *self,
 int linkIndex);

#endif /* NDEBUG */

void AutoRes_Bucket1D_addItemWithin 
(AutoRes_Bucket1D *self,
 double minRangeX,
 double maxRangeX,
 int itemId);

void AutoRes_Bucket1D_addItemAt 
(AutoRes_Bucket1D *self,
 double x,
 int itemId);

void AutoRes_Bucket1D_removeItemWithin 
(AutoRes_Bucket1D *self,
 double minRangeX,
 double maxRangeX,
 int itemId);

void AutoRes_Bucket1D_removeItemAt 
(AutoRes_Bucket1D *self,
 double x,
 int itemId);
