/* header file for autores_bucket2d.c */

typedef struct {
  double minRangeX;
  double minRangeY;
  double maxRangeX;
  double maxRangeY;
  int xCells;
  int yCells;
  int *startLinkIndexs;    /* [xCells * yCells] */
  int startLinkIndexOutOfRange;
  int startLinkIndexOfFreePool;
  int maxLinks;
  int nLinks;
  int *nextLinkIndexs;  /* [maxLinks] */
  int *linkItemIds;     /* [maxLinks] */
} AutoRes_Bucket2D;



/* Bucket2D */

void AutoRes_Bucket2D_initialize
(AutoRes_Bucket2D *self,
 double minRangeX, double minRangeY, 
 double maxRangeX, double maxRangeY,
 int xCells, int yCells, 
 int nAverageItemsPerCell);

void AutoRes_Bucket2D_terminate
(AutoRes_Bucket2D *self);

void AutoRes_Bucket2D_getMinRange 
(AutoRes_Bucket2D *self,
 double *minRangeX_OUT, double *minRangeY_OUT);

void AutoRes_Bucket2D_getMaxRange 
(AutoRes_Bucket2D *self,
 double *maxRangeX_OUT, double *maxRangeY_OUT);

void AutoRes_Bucket2D_getNCells 
(AutoRes_Bucket2D *self,
 int *xCells_OUT, int *yCells_OUT);

void AutoRes_Bucket2D_getIndexWithin 
(AutoRes_Bucket2D *self,
 double minRangeX, double minRangeY,
 double maxRangeX, double maxRangeY,
 int *cx0_OUT, int *cy0_OUT, 
 int *cx1_OUT, int *cy1_OUT);

void AutoRes_Bucket2D_getIndexAt 
(AutoRes_Bucket2D *self,
 double x, double y,
 int *cx_OUT, int *cy_OUT);

#ifdef NDEBUG

#define AutoRes_Bucket2D_isOutOfRange(self,cx,cy) \
  (cx) < 0 || (self)->xCells <= (cx) \
    || (cy) < 0 || (self)->yCells <= (cy) 

#define AutoRes_Bucket2D_startLinkIndexIn(self,cx,cy) \
 (self)->startLinkIndexs[(cx) * (self)->yCells + (cy)] 

#define AutoRes_Bucket2D_startLinkIndexOutOfRange(self) \
  (self)->startLinkIndexOutOfRange 

#define AutoRes_Bucket2D_nextLinkIndex(self,linkIndex) \
  (self)->nextLinkIndexs[(linkIndex)]

#define AutoRes_Bucket2D_itemId(self,linkIndex) \
  (self)->linkItemIds[(linkIndex)]

#else

int AutoRes_Bucket2D_isOutOfRange 
(AutoRes_Bucket2D *self,
 int cx, int cy);

int AutoRes_Bucket2D_startLinkIndexIn 
(AutoRes_Bucket2D *self,
 int cx, int cy);

int AutoRes_Bucket2D_startLinkIndexOutOfRange 
(AutoRes_Bucket2D *self);

int AutoRes_Bucket2D_nextLinkIndex 
(AutoRes_Bucket2D *self,
 int linkIndex);

int AutoRes_Bucket2D_itemId 
(AutoRes_Bucket2D *self,
 int linkIndex);

#endif /* NDEBUG */

void AutoRes_Bucket2D_addItemWithin 
(AutoRes_Bucket2D *self,
 double minRangeX, double minRangeY,
 double maxRangeX, double maxRangeY,
 int itemId);

void AutoRes_Bucket2D_addItemAt 
(AutoRes_Bucket2D *self,
 double x, double y,
 int itemId);

void AutoRes_Bucket2D_removeItemWithin 
(AutoRes_Bucket2D *self,
 double minRangeX, double minRangeY,
 double maxRangeX, double maxRangeY,
 int itemId);

void AutoRes_Bucket2D_removeItemAt 
(AutoRes_Bucket2D *self,
 double x, double y,
 int itemId);
