/* header file for autores_bucket3d.c */

typedef struct {
  double minRangeX;
  double minRangeY;
  double minRangeZ;
  double maxRangeX;
  double maxRangeY;
  double maxRangeZ;
  int xCells;
  int yCells;
  int zCells;
  int *startLinkIndexs;    /* [xCells * yCells] */
  int startLinkIndexOutOfRange;
  int startLinkIndexOfFreePool;
  int maxLinks;
  int nLinks;
  int *nextLinkIndexs;  /* [maxLinks] */
  int *linkItemIds;     /* [maxLinks] */
} AutoRes_Bucket3D;



/* Bucket3D */

void AutoRes_Bucket3D_initialize
(AutoRes_Bucket3D *self,
 double minRangeX, double minRangeY, double minRangeZ, 
 double maxRangeX, double maxRangeY, double maxRangeZ,
 int xCells, int yCells, int zCells, 
 int nAverageItemsPerCell);

void AutoRes_Bucket3D_terminate
(AutoRes_Bucket3D *self);

void AutoRes_Bucket3D_getMinRange 
(AutoRes_Bucket3D *self,
 double *minRangeX_OUT, double *minRangeY_OUT, double *minRangeZ_OUT);

void AutoRes_Bucket3D_getMaxRange 
(AutoRes_Bucket3D *self,
 double *maxRangeX_OUT, double *maxRangeY_OUT, double *maxRangeZ_OUT);

void AutoRes_Bucket3D_getNCells 
(AutoRes_Bucket3D *self,
 int *xCells_OUT, int *yCells_OUT, int *zCells_OUT);

void AutoRes_Bucket3D_getIndexWithin 
(AutoRes_Bucket3D *self,
 double minRangeX, double minRangeY, double minRangeZ,
 double maxRangeX, double maxRangeY, double maxRangeZ,
 int *cx0_OUT, int *cy0_OUT, int *cz0_OUT, 
 int *cx1_OUT, int *cy1_OUT, int *cz1_OUT);

void AutoRes_Bucket3D_getIndexAt 
(AutoRes_Bucket3D *self,
 double x, double y, double z,
 int *cx_OUT, int *cy_OUT, int *cz_OUT);

#ifdef NDEBUG

#define AutoRes_Bucket3D_isOutOfRange(self,cx,cy,cz) \
  (cx) < 0 || (self)->xCells <= (cx) \
    || (cy) < 0 || (self)->yCells <= (cy) \
    || (cz) < 0 || (self)->zCells <= (cz) 

#define AutoRes_Bucket3D_startLinkIndexIn(self,cx,cy,cz) \
 (self)->startLinkIndexs[(cx) * (self)->yCells * (self)->zCells \
                       + (cy) * (self)->zCells + (cz)] 

#define AutoRes_Bucket3D_startLinkIndexOutOfRange(self) \
  (self)->startLinkIndexOutOfRange 

#define AutoRes_Bucket3D_nextLinkIndex(self,linkIndex) \
  (self)->nextLinkIndexs[(linkIndex)]

#define AutoRes_Bucket3D_itemId(self,linkIndex) \
  (self)->linkItemIds[(linkIndex)]

#else

int AutoRes_Bucket3D_isOutOfRange 
(AutoRes_Bucket3D *self,
 int cx, int cy, int cz);

int AutoRes_Bucket3D_startLinkIndexIn 
(AutoRes_Bucket3D *self,
 int cx, int cy, int cz);

int AutoRes_Bucket3D_startLinkIndexOutOfRange 
(AutoRes_Bucket3D *self);

int AutoRes_Bucket3D_nextLinkIndex 
(AutoRes_Bucket3D *self,
 int linkIndex);

int AutoRes_Bucket3D_itemId 
(AutoRes_Bucket3D *self,
 int linkIndex);

#endif /* NDEBUG */

void AutoRes_Bucket3D_addItemWithin 
(AutoRes_Bucket3D *self,
 double minRangeX, double minRangeY, double minRangeZ,
 double maxRangeX, double maxRangeY, double maxRangeZ,
 int itemId);

void AutoRes_Bucket3D_addItemAt 
(AutoRes_Bucket3D *self,
 double x, double y, double z,
 int itemId);

void AutoRes_Bucket3D_removeItemWithin 
(AutoRes_Bucket3D *self,
 double minRangeX, double minRangeY, double minRangeZ,
 double maxRangeX, double maxRangeY, double maxRangeZ,
 int itemId);

void AutoRes_Bucket3D_removeItemAt 
(AutoRes_Bucket3D *self,
 double x, double y, double z,
 int itemId);
