/* extract_edge.c */

/* extract all the boundary edges from a given 2-D triangle mesh */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores.h"



#define MAX_ELEMENTS 10000
int Element_nElements;
int Element_nodeIds[MAX_ELEMENTS][3];

#define MAX_EDGES 10000
int Edge_nEdges;
int Edge_elementIds[MAX_EDGES];
int Edge_elementEdgeIds[MAX_EDGES];



void ReadMesh (void)
{
  FILE *fp;
  int iElement;

  fp = fopen ("boundary_mesh.dat", "r");
  assert(fp);

  fscanf (fp, "%d", 
	  &Element_nElements);

  for (iElement = 0; iElement < Element_nElements; iElement++) {
    fscanf (fp, "%d %d %d", 
	    &Element_nodeIds[iElement][0],
	    &Element_nodeIds[iElement][1],
	    &Element_nodeIds[iElement][2]);
  }

  fclose (fp);
}

void WriteEdges (void)
{
  FILE *fp;
  int iEdge;

  fp = fopen ("boundary_edge.dat", "w");
  assert(fp);

  fprintf (fp, "%d \n", Edge_nEdges);

  for (iEdge = 0; iEdge < Edge_nEdges; iEdge++) {
    int elementId = Edge_elementIds[iEdge];
    int elementEdgeId = Edge_elementEdgeIds[iEdge];
    int edgeNode0, edgeNode1;
      
    AutoRes_GetEdgeNodesOfTriangleMesh2D
      (elementId, elementEdgeId,
       &edgeNode0, &edgeNode1);
      
    fprintf (fp, "%d %d %d %d \n", 
	     elementId, elementEdgeId,
	     edgeNode0, edgeNode1);
  }

  fclose (fp);
}

int main (int argc, char* argv[])
{
  ReadMesh ();

  AutoRes_SetTriangleMesh2D 
    (Element_nElements, 3, (int*)Element_nodeIds);
  AutoRes_ExtractEdgesOfTriangleMesh2D 
    (MAX_EDGES,
     &Edge_nEdges, Edge_elementIds, Edge_elementEdgeIds);
  
  WriteEdges ();

  return 0;
}
