/* find_element_at.c */

/* find any one element which contains a given point from a 2-D triangle mesh */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores.h"



#define MAX_ELEMENTS 10000
int Element_nElements;
int Element_nodeIds[MAX_ELEMENTS][3];

#define MAX_NODES 10000
int Node_nNodes;
double Node_XVec[MAX_NODES][2];



void ReadMesh (void)
{
  FILE *fp;
  int iElement;
  int iNode;

  fp = fopen ("find_mesh.dat", "r");
  assert(fp);

  fscanf (fp, "%d", 
	  &Element_nElements);

  for (iElement = 0; iElement < Element_nElements; iElement++) {
    fscanf (fp, "%d %d %d", 
	    &Element_nodeIds[iElement][0],
	    &Element_nodeIds[iElement][1],
	    &Element_nodeIds[iElement][2]);
  }

  fscanf (fp, "%d", 
	  &Node_nNodes);

  for (iNode = 0; iNode < Node_nNodes; iNode++) {
    fscanf (fp, "%lf %lf", 
	    &Node_XVec[iNode][0],
	    &Node_XVec[iNode][1]);
  }

  fclose (fp);
}

void FindElementAt 
(double x, double y)
{
  int elementId;
  double L0, L1, L2;
  
  printf (" find (%5.2f %5.2f) \n", x, y);
  
  elementId = AutoRes_FindTriangleElementAt2D 
    (x, y, 
     &L0, &L1, &L2);
  if (elementId == -1) {
    printf (" out of range \n");
  } else {
    printf (" Element ID %d ", elementId);

    {
      int nodeId0 = Element_nodeIds[elementId][0];
      int nodeId1 = Element_nodeIds[elementId][1];
      int nodeId2 = Element_nodeIds[elementId][2];
      double x0 = Node_XVec[nodeId0][0];
      double y0 = Node_XVec[nodeId0][1];
      double x1 = Node_XVec[nodeId1][0];
      double y1 = Node_XVec[nodeId1][1];
      double x2 = Node_XVec[nodeId2][0];
      double y2 = Node_XVec[nodeId2][1];
      double x = x0 * L0 + x1 * L1 + x2 * L2;
      double y = y0 * L0 + y1 * L1 + y2 * L2;

      printf (" (X = %5.2f Y = %5.2f) ", x, y);
    }

    printf (" (L0 = %5.2f L1 = %5.2f L2 = %5.2f) ", L0, L1, L2);
    printf ("\n");
  }
}

int main (int argc, char* argv[])
{
  ReadMesh ();

  AutoRes_SetNodalCoordinate2D 
    (Node_nNodes, (double*)Node_XVec);
  AutoRes_SetTriangleMesh2D 
    (Element_nElements, 3, (int*)Element_nodeIds);

  AutoRes_CreateTriangleElementBucket2D ();

  FindElementAt (0.0, 0.0);
  FindElementAt (0.1, 0.1);
  FindElementAt (0.1, 2.2);
  FindElementAt (5.0, 5.0);
  FindElementAt (-1.0, -1.0);
  FindElementAt (1.0, 1.0);
  FindElementAt (1.5, 1.4);
  FindElementAt (3.0, 3.0);

  AutoRes_DeleteTriangleElementBucket2D ();

  return 0;
}
