/* mapped_mesh.c */

/* make 4-node mesh from a set of blocks */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores.h"



#define MAX_ELEMENTS 10000
int Element_nElements;
int Element_nodeIds[MAX_ELEMENTS][4];

#define MAX_NODES 10000
int Node_nNodes;
double Node_XVec[MAX_NODES][2];



int TryToAddNode (double x, double y)
{
  int result = AutoRes_FindNodeAt2D (x, y);

  if (result == -1) {
    result = Node_nNodes;
    
    assert(Node_nNodes < MAX_NODES);
    
    Node_XVec[Node_nNodes][0] = x;
    Node_XVec[Node_nNodes][1] = y;
    Node_nNodes++;
    
    AutoRes_AddLastNode2D ();
  }

  return result;
}

void AddQuadrangle 
(double x0, double y0,
 double x1, double y1,
 double x2, double y2,
 double x3, double y3)
{
  int nodeId0 = TryToAddNode (x0, y0);
  int nodeId1 = TryToAddNode (x1, y1);
  int nodeId2 = TryToAddNode (x2, y2);
  int nodeId3 = TryToAddNode (x3, y3);

  assert(Element_nElements < MAX_ELEMENTS);
  
  Element_nodeIds[Element_nElements][0] = nodeId0;
  Element_nodeIds[Element_nElements][1] = nodeId1;
  Element_nodeIds[Element_nElements][2] = nodeId2;
  Element_nodeIds[Element_nElements][3] = nodeId3;
  Element_nElements++;
}

void MakeMesh (void)
{
  double nodalDistance = 0.5;
  double minRangeX = 0.0;
  double minRangeY = 0.0;
  double maxRangeX = 5.0;
  double maxRangeY = 5.0;

  AutoRes_SetNodalCoordinate2D 
    (Node_nNodes, (double*)Node_XVec);
  AutoRes_CreateEmptyNodeBucket2D 
    (nodalDistance, 
     minRangeX, minRangeY,
     maxRangeX, maxRangeY);

  AddQuadrangle (0, 0,  1, 0,  1, 1,  0, 1);
  AddQuadrangle (1, 0,  2, 0,  2, 1,  1, 1);
  AddQuadrangle (0, 1,  1, 1,  1, 2,  0, 2);
  AddQuadrangle (1, 1,  2, 1,  2, 2,  1, 2);

  AutoRes_DeleteNodeBucket2D ();
}

void WriteMesh (void)
{
  FILE *fp;
  int iElement;
  int iNode;

  fp = fopen ("mapped_mesh.dat", "w");
  assert(fp);

  fprintf (fp, "%d \n", Element_nElements);

  for (iElement = 0; iElement < Element_nElements; iElement++) {
    fprintf (fp, "%d %d %d %d \n", 
	     Element_nodeIds[iElement][0],
	     Element_nodeIds[iElement][1],
	     Element_nodeIds[iElement][2],
	     Element_nodeIds[iElement][3]);
  }

  fprintf (fp, "%d \n", Node_nNodes);

  for (iNode = 0; iNode < Node_nNodes; iNode++) {
    fprintf (fp, "%f %f \n", 
	     Node_XVec[iNode][0],
	     Node_XVec[iNode][1]);
  }

  fclose (fp);
}

int main (int argc, char* argv[])
{
  MakeMesh ();
  WriteMesh ();

  return 0;
}
