/* bucket1d_point_sample.c */

/* sample of 1-dimensional bucket */
/* add / remove point-like objects into a bucket */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores.h"



int main (int argc, char* argv[])
{
  int nAverageLinksPerCell = 10;
  AutoRes_Bucket1D bucket;
  int cx;
  int linkIndex;

  AutoRes_Bucket1D_initialize 
    (&bucket,
     0.0, 
     10.0,
     10, 
     nAverageLinksPerCell);
  
  
  
  printf (" <add> \n");
  
  AutoRes_Bucket1D_addItemAt 
    (&bucket,
     0.5, 
     101);
  AutoRes_Bucket1D_addItemAt 
    (&bucket,
     0.5, 
     102);
  AutoRes_Bucket1D_addItemAt 
    (&bucket,
     1.5, 
     103);
  AutoRes_Bucket1D_addItemAt 
    (&bucket,
     0.5, 
     104);
  AutoRes_Bucket1D_addItemAt 
    (&bucket,
     1.0, 
     105);
  
  for (cx = 0; cx < 3; cx++) {
    printf (" cell %d \t", cx);
    
    for (linkIndex = AutoRes_Bucket1D_startLinkIndexIn 
	   (&bucket, 
	    cx);
	 linkIndex != -1;
	 linkIndex = AutoRes_Bucket1D_nextLinkIndex 
	   (&bucket, 
	    linkIndex)) {
      int itemId = AutoRes_Bucket1D_itemId 
	(&bucket, 
	 linkIndex);
      
      printf (" %d ", itemId);
    }
    
    printf ("\n");
  }
  
  printf (" find 103 at (1.5) from ");
  AutoRes_Bucket1D_getIndexAt 
    (&bucket, 
     1.5,
     &cx);
  
  for (linkIndex = AutoRes_Bucket1D_startLinkIndexIn 
	 (&bucket, 
	  cx);
       linkIndex != -1;
       linkIndex = AutoRes_Bucket1D_nextLinkIndex 
	 (&bucket, 
	  linkIndex)) {
    int itemId = AutoRes_Bucket1D_itemId 
      (&bucket, 
       linkIndex);
    
    printf (" %d ", itemId);
  }
  printf ("\n");

  printf (" find 105 at (1.0) from ");
  AutoRes_Bucket1D_getIndexAt 
    (&bucket, 
     1.0,
     &cx);

  for (linkIndex = AutoRes_Bucket1D_startLinkIndexIn 
	 (&bucket, 
	  cx);
       linkIndex != -1;
       linkIndex = AutoRes_Bucket1D_nextLinkIndex 
	 (&bucket, 
	  linkIndex)) {
    int itemId = AutoRes_Bucket1D_itemId 
      (&bucket, 
       linkIndex);
    
    printf (" %d ", itemId);
  }
  printf ("\n");



  printf (" <remove> \n");

  AutoRes_Bucket1D_removeItemAt 
    (&bucket,
     0.5, 
     101);
  AutoRes_Bucket1D_removeItemAt 
    (&bucket,
     0.5, 
     104);
  AutoRes_Bucket1D_removeItemAt 
    (&bucket,
     1.0, 
     105);

  for (cx = 0; cx < 3; cx++) {
    printf (" cell %d \t", cx);
    
    for (linkIndex = AutoRes_Bucket1D_startLinkIndexIn 
	   (&bucket, 
	    cx);
	 linkIndex != -1;
	 linkIndex = AutoRes_Bucket1D_nextLinkIndex 
	   (&bucket, 
	    linkIndex)) {
      int itemId = AutoRes_Bucket1D_itemId 
	(&bucket, 
	 linkIndex);
      
      printf (" %d ", itemId);
    }
    
    printf ("\n");
  }



  AutoRes_Bucket1D_terminate 
    (&bucket);

  return 0;
}
