/* bucket1d_range_sample.c */

/* sample of 2-dimensional bucket */
/* add / remove volumetric objects into a bucket */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores.h"



int main (int argc, char* argv[])
{
  int nAverageLinksPerCell = 10;
  AutoRes_Bucket1D bucket;
  int cx0;
  int cx1;
  int outOfRangeExists;
  int cx;
  int linkIndex;
  
  AutoRes_Bucket1D_initialize 
    (&bucket,
     0.0, 
     10.0,
     10, 
     nAverageLinksPerCell);



  printf (" <add> \n");

  AutoRes_Bucket1D_addItemWithin 
    (&bucket,
     0.5, 
     1.5, 
     101);
  AutoRes_Bucket1D_addItemWithin 
    (&bucket,
     0.5, 
     1.5, 
     102);
  AutoRes_Bucket1D_addItemWithin 
    (&bucket,
     -0.5, 
     2.5, 
     103);
  AutoRes_Bucket1D_addItemWithin 
    (&bucket,
     0.5, 
     0.5, 
     104);
  AutoRes_Bucket1D_addItemWithin 
    (&bucket,
     1.0, 
     1.0, 
     105);
  AutoRes_Bucket1D_addItemWithin 
    (&bucket,
     10.5, 
     12.5, 
     106);

  AutoRes_Bucket1D_getIndexWithin 
    (&bucket, 
     0.0,
     2.0,
     &cx0, 
     &cx1);

  outOfRangeExists = 0;
  for (cx = cx0; cx <= cx1; cx++) {
    if (AutoRes_Bucket1D_isOutOfRange 
	(&bucket, 
	 cx)) {
      outOfRangeExists = 1;
    } else {
      printf (" cell %d \t", cx);
      
      for (linkIndex = AutoRes_Bucket1D_startLinkIndexIn 
	     (&bucket, 
	      cx);
	   linkIndex != -1;
	   linkIndex = AutoRes_Bucket1D_nextLinkIndex 
	     (&bucket, 
	      linkIndex)) {
	int itemId = AutoRes_Bucket1D_itemId 
	  (&bucket, 
	   linkIndex);
	
	printf (" %d ", itemId);
      }
      
      printf ("\n");
    }
  }
  if (outOfRangeExists) {
    printf (" cell out of range \t");
    
    for (linkIndex = AutoRes_Bucket1D_startLinkIndexOutOfRange 
	   (&bucket);
	 linkIndex != -1;
	 linkIndex = AutoRes_Bucket1D_nextLinkIndex 
	   (&bucket, 
	    linkIndex)) {
      int itemId = AutoRes_Bucket1D_itemId 
	(&bucket, 
	 linkIndex);
      
      printf (" %d ", itemId);
    }
    
    printf ("\n");
  }



  printf (" <remove> \n");

  AutoRes_Bucket1D_removeItemWithin 
    (&bucket,
     -0.5, 
     2.5, 
     103);
  AutoRes_Bucket1D_removeItemWithin 
    (&bucket,
     0.5, 
     0.5, 
     104);
  AutoRes_Bucket1D_removeItemWithin 
    (&bucket,
     10.5, 
     12.5, 
     106);

  AutoRes_Bucket1D_getIndexWithin 
    (&bucket, 
     0.0,
     2.0,
     &cx0, 
     &cx1);

  outOfRangeExists = 0;
  for (cx = cx0; cx <= cx1; cx++) {
    if (AutoRes_Bucket1D_isOutOfRange 
	(&bucket, 
	 cx)) {
      outOfRangeExists = 1;
    } else {
      printf (" cell %d \t", cx);

      for (linkIndex = AutoRes_Bucket1D_startLinkIndexIn 
	     (&bucket, 
	      cx);
	   linkIndex != -1;
	   linkIndex = AutoRes_Bucket1D_nextLinkIndex 
	     (&bucket, 
	      linkIndex)) {
	int itemId = AutoRes_Bucket1D_itemId 
	  (&bucket, 
	   linkIndex);
	
	printf (" %d ", itemId);
      }
      
      printf ("\n");
    }
  }
  if (outOfRangeExists) {
    printf (" cell out of range \t");
    
    for (linkIndex = AutoRes_Bucket1D_startLinkIndexOutOfRange 
	   (&bucket);
	 linkIndex != -1;
	 linkIndex = AutoRes_Bucket1D_nextLinkIndex 
	   (&bucket, 
	    linkIndex)) {
      int itemId = AutoRes_Bucket1D_itemId 
	(&bucket, 
	 linkIndex);
      
      printf (" %d ", itemId);
    }

    printf ("\n");
  }



  AutoRes_Bucket1D_terminate 
    (&bucket);

  return 0;
}
