/* bucket2d_point_sample.c */

/* sample of 2-dimensional bucket */
/* add / remove point-like objects into a bucket */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores.h"



int main (int argc, char* argv[])
{
  int nAverageLinksPerCell = 10;
  AutoRes_Bucket2D bucket;
  int cx, cy;
  int linkIndex;

  AutoRes_Bucket2D_initialize 
    (&bucket,
     0.0, 0.0, 
     10.0, 10.0,
     10, 10, 
     nAverageLinksPerCell);



  printf (" <add> \n");

  AutoRes_Bucket2D_addItemAt 
    (&bucket,
     0.5, 1.0, 
     101);
  AutoRes_Bucket2D_addItemAt 
    (&bucket,
     0.5, 2.5, 
     102);
  AutoRes_Bucket2D_addItemAt 
    (&bucket,
     1.5, 1.5, 
     103);
  AutoRes_Bucket2D_addItemAt 
    (&bucket,
     0.5, 1.5, 
     104);
  AutoRes_Bucket2D_addItemAt 
    (&bucket,
     1.0, 1.0, 
     105);

  for (cx = 0; cx < 3; cx++) {
    for (cy = 0; cy < 3; cy++) {
      printf (" cell %d %d \t", cx, cy);

      for (linkIndex = AutoRes_Bucket2D_startLinkIndexIn 
	     (&bucket, 
	      cx, cy);
	   linkIndex != -1;
	   linkIndex = AutoRes_Bucket2D_nextLinkIndex 
	     (&bucket, 
	      linkIndex)) {
	int itemId = AutoRes_Bucket2D_itemId 
	  (&bucket, 
	   linkIndex);
	
	printf (" %d ", itemId);
      }

      printf ("\n");
    }
  }

  printf (" find 103 at (1.5, 1.5) from ");
  AutoRes_Bucket2D_getIndexAt 
    (&bucket, 
     1.5, 1.5,
     &cx, &cy);

  for (linkIndex = AutoRes_Bucket2D_startLinkIndexIn 
	 (&bucket, 
	  cx, cy);
       linkIndex != -1;
       linkIndex = AutoRes_Bucket2D_nextLinkIndex 
	 (&bucket, 
	  linkIndex)) {
    int itemId = AutoRes_Bucket2D_itemId 
      (&bucket, 
       linkIndex);
    
    printf (" %d ", itemId);
  }
  printf ("\n");

  printf (" find 105 at (1.0, 1.0) from ");
  AutoRes_Bucket2D_getIndexAt 
    (&bucket, 
     1.0, 1.0,
     &cx, &cy);

  for (linkIndex = AutoRes_Bucket2D_startLinkIndexIn 
	 (&bucket, 
	  cx, cy);
       linkIndex != -1;
       linkIndex = AutoRes_Bucket2D_nextLinkIndex 
	 (&bucket, 
	  linkIndex)) {
    int itemId = AutoRes_Bucket2D_itemId 
      (&bucket, 
       linkIndex);
    
    printf (" %d ", itemId);
  }
  printf ("\n");



  printf (" <remove> \n");

  AutoRes_Bucket2D_removeItemAt 
    (&bucket,
     0.5, 1.0, 
     101);
  AutoRes_Bucket2D_removeItemAt 
    (&bucket,
     0.5, 1.5, 
     104);
  AutoRes_Bucket2D_removeItemAt 
    (&bucket,
     1.0, 1.0, 
     105);

  for (cx = 0; cx < 3; cx++) {
    for (cy = 0; cy < 3; cy++) {
      printf (" cell %d %d \t", cx, cy);

      for (linkIndex = AutoRes_Bucket2D_startLinkIndexIn 
	     (&bucket, 
	      cx, cy);
	   linkIndex != -1;
	   linkIndex = AutoRes_Bucket2D_nextLinkIndex 
	     (&bucket, 
	      linkIndex)) {
	int itemId = AutoRes_Bucket2D_itemId 
	  (&bucket, 
	   linkIndex);
	
	printf (" %d ", itemId);
      }

      printf ("\n");
    }
  }



  AutoRes_Bucket2D_terminate 
    (&bucket);

  return 0;
}
