/* bucket2d_range_sample.c */

/* sample of 2-dimensional bucket */
/* add / remove volumetric objects into a bucket */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores.h"



int main (int argc, char* argv[])
{
  int nAverageLinksPerCell = 10;
  AutoRes_Bucket2D bucket;
  int cx0, cy0;
  int cx1, cy1;
  int outOfRangeExists;
  int cx, cy;
  int linkIndex;
  
  AutoRes_Bucket2D_initialize 
    (&bucket,
     0.0, 0.0, 
     10.0, 10.0,
     10, 10, 
     nAverageLinksPerCell);



  printf (" <add> \n");

  AutoRes_Bucket2D_addItemWithin 
    (&bucket,
     0.5, 1.5, 
     1.5, 2.5, 
     101);
  AutoRes_Bucket2D_addItemWithin 
    (&bucket,
     0.5, 0.5, 
     1.5, 0.5, 
     102);
  AutoRes_Bucket2D_addItemWithin 
    (&bucket,
     -0.5, -0.5, 
     2.5, 2.5, 
     103);
  AutoRes_Bucket2D_addItemWithin 
    (&bucket,
     0.5, 1.5, 
     0.5, 1.5, 
     104);
  AutoRes_Bucket2D_addItemWithin 
    (&bucket,
     1.0, 1.0, 
     1.0, 1.0, 
     105);
  AutoRes_Bucket2D_addItemWithin 
    (&bucket,
     10.5, 10.5, 
     12.5, 12.5, 
     106);

  AutoRes_Bucket2D_getIndexWithin 
    (&bucket, 
     0.0, 0.0,
     2.0, 2.0,
     &cx0, &cy0, 
     &cx1, &cy1);

  outOfRangeExists = 0;
  for (cx = cx0; cx <= cx1; cx++) {
    for (cy = cy0; cy <= cy1; cy++) {
      if (AutoRes_Bucket2D_isOutOfRange 
	  (&bucket, 
	   cx, cy)) {
	outOfRangeExists = 1;
      } else {
	printf (" cell %d %d \t", cx, cy);

	for (linkIndex = AutoRes_Bucket2D_startLinkIndexIn 
	       (&bucket, 
		cx, cy);
	     linkIndex != -1;
	     linkIndex = AutoRes_Bucket2D_nextLinkIndex 
	       (&bucket, 
		linkIndex)) {
	  int itemId = AutoRes_Bucket2D_itemId 
	    (&bucket, 
	     linkIndex);

	  printf (" %d ", itemId);
	}

	printf ("\n");
      }
    }
  }
  if (outOfRangeExists) {
    printf (" cell out of range \t");

    for (linkIndex = AutoRes_Bucket2D_startLinkIndexOutOfRange 
	   (&bucket);
	 linkIndex != -1;
	 linkIndex = AutoRes_Bucket2D_nextLinkIndex 
	   (&bucket, 
	    linkIndex)) {
      int itemId = AutoRes_Bucket2D_itemId 
	(&bucket, 
	 linkIndex);
      
      printf (" %d ", itemId);
    }

    printf ("\n");
  }



  printf (" <remove> \n");

  AutoRes_Bucket2D_removeItemWithin 
    (&bucket,
     -0.5, -0.5, 
     2.5, 2.5, 
     103);
  AutoRes_Bucket2D_removeItemWithin 
    (&bucket,
     0.5, 1.5, 
     0.5, 1.5, 
     104);
  AutoRes_Bucket2D_removeItemWithin 
    (&bucket,
     10.5, 10.5, 
     12.5, 12.5, 
     106);

  AutoRes_Bucket2D_getIndexWithin 
    (&bucket, 
     0.0, 0.0,
     2.0, 2.0,
     &cx0, &cy0, 
     &cx1, &cy1);

  outOfRangeExists = 0;
  for (cx = cx0; cx <= cx1; cx++) {
    for (cy = cy0; cy <= cy1; cy++) {
      if (AutoRes_Bucket2D_isOutOfRange 
	  (&bucket, 
	   cx, cy)) {
	outOfRangeExists = 1;
      } else {
	printf (" cell %d %d \t", cx, cy);

	for (linkIndex = AutoRes_Bucket2D_startLinkIndexIn 
	       (&bucket, 
		cx, cy);
	     linkIndex != -1;
	     linkIndex = AutoRes_Bucket2D_nextLinkIndex 
	       (&bucket, 
		linkIndex)) {
	  int itemId = AutoRes_Bucket2D_itemId 
	    (&bucket, 
	     linkIndex);

	  printf (" %d ", itemId);
	}

	printf ("\n");
      }
    }
  }
  if (outOfRangeExists) {
    printf (" cell out of range \t");

    for (linkIndex = AutoRes_Bucket2D_startLinkIndexOutOfRange 
	   (&bucket);
	 linkIndex != -1;
	 linkIndex = AutoRes_Bucket2D_nextLinkIndex 
	   (&bucket, 
	    linkIndex)) {
      int itemId = AutoRes_Bucket2D_itemId 
	(&bucket, 
	 linkIndex);
      
      printf (" %d ", itemId);
    }

    printf ("\n");
  }



  AutoRes_Bucket2D_terminate 
    (&bucket);

  return 0;
}
