/* bucket3d_range_sample.c */

/* sample of 3-dimensional bucket */
/* add / remove volumetric objects into a bucket */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores.h"



int main (int argc, char* argv[])
{
  int nAverageLinksPerCell = 10;
  AutoRes_Bucket3D bucket;
  int cx0, cy0, cz0;
  int cx1, cy1, cz1;
  int outOfRangeExists;
  int cx, cy, cz;
  int linkIndex;
  
  AutoRes_Bucket3D_initialize 
    (&bucket,
     0.0, 0.0, 0.0, 
     10.0, 10.0, 10.0,
     10, 10, 10, 
     nAverageLinksPerCell);



  printf (" <add> \n");

  AutoRes_Bucket3D_addItemWithin 
    (&bucket,
     0.5, 1.5, 0.5, 
     1.5, 2.5, 0.5, 
     101);
  AutoRes_Bucket3D_addItemWithin 
    (&bucket,
     0.5, 0.5, 1.5, 
     1.5, 0.5, 1.5, 
     102);
  AutoRes_Bucket3D_addItemWithin 
    (&bucket,
     -0.5, -0.5, -0.5, 
     2.5, 2.5, 2.5, 
     103);
  AutoRes_Bucket3D_addItemWithin 
    (&bucket,
     0.5, 1.5, 0.5,
     0.5, 1.5, 0.5, 
     104);
  AutoRes_Bucket3D_addItemWithin 
    (&bucket,
     1.0, 1.0, 1.0,
     1.0, 1.0, 1.0,
     105);
  AutoRes_Bucket3D_addItemWithin 
    (&bucket,
     10.5, 10.5, 10.5,
     12.5, 12.5, 12.5, 
     106);

  AutoRes_Bucket3D_getIndexWithin 
    (&bucket, 
     0.0, 0.0, 0.0,
     2.0, 2.0, 2.0, 
     &cx0, &cy0, &cz0, 
     &cx1, &cy1, &cz1);

  outOfRangeExists = 0;
  for (cx = cx0; cx <= cx1; cx++) {
    for (cy = cy0; cy <= cy1; cy++) {
      for (cz = cz0; cz <= cz1; cz++) {
	if (AutoRes_Bucket3D_isOutOfRange 
	    (&bucket, 
	     cx, cy, cz)) {
	  outOfRangeExists = 1;
	} else {
	  printf (" cell %d %d %d \t", cx, cy, cz);
	  
	  for (linkIndex = AutoRes_Bucket3D_startLinkIndexIn 
		 (&bucket, 
		  cx, cy, cz);
	       linkIndex != -1;
	       linkIndex = AutoRes_Bucket3D_nextLinkIndex 
		 (&bucket, 
		  linkIndex)) {
	    int itemId = AutoRes_Bucket3D_itemId 
	      (&bucket, 
	       linkIndex);
	    
	    printf (" %d ", itemId);
	  }
	  
	  printf ("\n");
	}
      }
    }
  }
  if (outOfRangeExists) {
    printf (" cell out of range \t");

    for (linkIndex = AutoRes_Bucket3D_startLinkIndexOutOfRange 
	   (&bucket);
	 linkIndex != -1;
	 linkIndex = AutoRes_Bucket3D_nextLinkIndex 
	   (&bucket, 
	    linkIndex)) {
      int itemId = AutoRes_Bucket3D_itemId 
	(&bucket, 
	 linkIndex);
      
      printf (" %d ", itemId);
    }

    printf ("\n");
  }



  printf (" <remove> \n");

  AutoRes_Bucket3D_removeItemWithin 
    (&bucket,
     -0.5, -0.5, -0.5, 
     2.5, 2.5, 2.5, 
     103);
  AutoRes_Bucket3D_removeItemWithin 
    (&bucket,
     0.5, 1.5, 0.5, 
     0.5, 1.5, 0.5, 
     104);
  AutoRes_Bucket3D_removeItemWithin 
    (&bucket,
     10.5, 10.5, 10.5,
     12.5, 12.5, 12.5, 
     106);

  AutoRes_Bucket3D_getIndexWithin 
    (&bucket, 
     0.0, 0.0, 0.0,
     2.0, 2.0, 2.0, 
     &cx0, &cy0, &cz0, 
     &cx1, &cy1, &cz1);

  outOfRangeExists = 0;
  for (cx = cx0; cx <= cx1; cx++) {
    for (cy = cy0; cy <= cy1; cy++) {
      for (cz = cz0; cz <= cz1; cz++) {
	if (AutoRes_Bucket3D_isOutOfRange 
	    (&bucket, 
	     cx, cy, cz)) {
	  outOfRangeExists = 1;
	} else {
	  printf (" cell %d %d %d \t", cx, cy, cz);

	  for (linkIndex = AutoRes_Bucket3D_startLinkIndexIn 
		 (&bucket, 
		  cx, cy, cz);
	       linkIndex != -1;
	       linkIndex = AutoRes_Bucket3D_nextLinkIndex 
		 (&bucket, 
		  linkIndex)) {
	    int itemId = AutoRes_Bucket3D_itemId 
	      (&bucket, 
	       linkIndex);
	    
	    printf (" %d ", itemId);
	  }
	  
	  printf ("\n");
	}
      }
    }
  }
  if (outOfRangeExists) {
    printf (" cell out of range \t");

    for (linkIndex = AutoRes_Bucket3D_startLinkIndexOutOfRange 
	   (&bucket);
	 linkIndex != -1;
	 linkIndex = AutoRes_Bucket3D_nextLinkIndex 
	   (&bucket, 
	    linkIndex)) {
      int itemId = AutoRes_Bucket3D_itemId 
	(&bucket, 
	 linkIndex);
      
      printf (" %d ", itemId);
    }

    printf ("\n");
  }



  AutoRes_Bucket3D_terminate 
    (&bucket);

  return 0;
}
