/* autores_array.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_memory.h"
#include "autores_column.h"
#include "autores_integer_column.h"
#include "autores_real_column.h"
#include "autores_array.h"



/* Array 2D */

AUTORES_IMPLEMENT_HANDLE(AutoRes_Array2D);

AutoRes_Array2D *AutoRes_Array2D_New 
(AutoRes_Table *table,
 int size0, 
 int size1)
{
  AutoRes_Array2D *array;

  assert(0 < size0);
  assert(0 < size1);

  array = AutoRes_Array2D_NewImplementation ();

  array->table = table;
  array->size0 = size0;
  array->size1 = size1;

  return array;
}

void AutoRes_Array2D_delete 
(AutoRes_Array2D *self)
{
  AutoRes_ReleaseHandle (self->handle);
  self->handle = AUTORES_NULL_HANDLE;
}

AutoRes_Table *AutoRes_Array2D_table 
(AutoRes_Array2D *self)
{
  return self->table;
}

int AutoRes_Array2D_size0 
(AutoRes_Array2D *self)
{
  return self->size0;
}

int AutoRes_Array2D_size1 
(AutoRes_Array2D *self)
{
  return self->size1;
}

int AutoRes_Array2D_recordId 
(AutoRes_Array2D *self,
 int index0,
 int index1)
{
  assert(self->size0 * self->size1 == AutoRes_Table_nRecords (self->table));
  assert(0 <= index0);
  assert(index0 < self->size0);
  assert(0 <= index1);
  assert(index1 < self->size1);

  return self->size1 * index0 + index1;
}



/* Array 3D */

AUTORES_IMPLEMENT_HANDLE(AutoRes_Array3D);

AutoRes_Array3D *AutoRes_Array3D_New 
(AutoRes_Table *table,
 int size0, 
 int size1, 
 int size2)
{
  AutoRes_Array3D *array;

  assert(0 < size0);
  assert(0 < size1);
  assert(0 < size2);

  array = AutoRes_Array3D_NewImplementation ();

  array->table = table;
  array->size0 = size0;
  array->size1 = size1;
  array->size2 = size2;

  return array;
}

void AutoRes_Array3D_delete 
(AutoRes_Array3D *self)
{
  AutoRes_ReleaseHandle (self->handle);
  self->handle = AUTORES_NULL_HANDLE;
}

AutoRes_Table *AutoRes_Array3D_table 
(AutoRes_Array3D *self)
{
  return self->table;
}

int AutoRes_Array3D_size0 
(AutoRes_Array3D *self)
{
  return self->size0;
}

int AutoRes_Array3D_size1 
(AutoRes_Array3D *self)
{
  return self->size1;
}

int AutoRes_Array3D_size2 
(AutoRes_Array3D *self)
{
  return self->size2;
}

int AutoRes_Array3D_recordId 
(AutoRes_Array3D *self,
 int index0,
 int index1,
 int index2)
{
  assert(self->size0 * self->size1 * self->size2 
	 == AutoRes_Table_nRecords (self->table));
  assert(0 <= index0);
  assert(index0 < self->size0);
  assert(0 <= index1);
  assert(index1 < self->size1);
  assert(0 <= index2);
  assert(index2 < self->size2);

  return self->size1 * self->size2 * index0 
    + self->size2 * index1 
    + index2;
}
