/* autores_base.c : Base Module */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"



/* Utility */

void AutoRes_Error (const char *msg)
{
  fprintf (stderr, " AutoRes ERROR : %s\n", msg);
  abort ();
}

void AutoRes_Warning (const char *msg)
{
  fprintf (stderr, " AutoRes WARNING : %s\n", msg);
}



/* Tolerance management */

static double Tolerance = 0.0000001;

void AutoRes_SetTolerance (double tolerance)
{
  Tolerance = tolerance;
}

double AutoRes_GetTolerance (void)
{
  return Tolerance;
}



/* Handle Type */

/* handleTypeId starting from 1 */

static int NHandleTypes;

int /* boolean */
AutoRes_HandleTypeIsValid 
(int handleTypeId)
{
  return 0 < handleTypeId
    && handleTypeId <= NHandleTypes;
}

int AutoRes_NewHandleType (void)
{
  int result = 0;
  
  assert(NHandleTypes < AUTORES_MAX_HANDLE_TYPES);
  
  if (AUTORES_MAX_HANDLE_TYPES <= NHandleTypes) {
    AutoRes_Error (" too many handle types ");
  }

  NHandleTypes++;
  result = NHandleTypes;

  assert(AutoRes_HandleTypeIsValid (result));

  return result;
}



/* Handle */

static char HandleIsUsed
[AUTORES_MAX_HANDLE_TYPES + 1][AUTORES_MAX_HANDLE_INSTANCES];
/* (100 + 1) * 1000 = 0.101MB used for handle management */

int AutoRes_HandleTypeId 
(AutoRes_Handle handle)
{
  assert(0 < handle);

  return handle / AUTORES_MAX_HANDLE_INSTANCES;
}

int AutoRes_HandleInstanceId 
(AutoRes_Handle handle)
{
  assert(0 < handle);

  return handle % AUTORES_MAX_HANDLE_INSTANCES;
}

AutoRes_Handle AutoRes_HandleFrom 
(int handleTypeId, 
 int handleInstanceId)
{
  assert(AutoRes_HandleTypeIsValid (handleTypeId));
  assert(0 <= handleInstanceId);
  assert(handleInstanceId < AUTORES_MAX_HANDLE_INSTANCES);
  
  return handleTypeId * AUTORES_MAX_HANDLE_INSTANCES + handleInstanceId;
}

int /* boolean */
AutoRes_HandleIsValid 
(AutoRes_Handle handle)
{
  if (0 < handle) {
    int handleTypeId = AutoRes_HandleTypeId (handle);
    int handleInstanceId = AutoRes_HandleInstanceId (handle);
    
    if (AutoRes_HandleTypeIsValid (handleTypeId)
	&& 0 <= handleInstanceId
	&& handleInstanceId < AUTORES_MAX_HANDLE_INSTANCES) {
      return (HandleIsUsed[handleTypeId][handleInstanceId]);
    }
  }

  return 0;
}

AutoRes_Handle AutoRes_NewHandle 
(int handleTypeId)
{
  int iInstance;

  /* without <AutoRes_NewHandleType> call, */
  /* this call is trapped because <handleTypeId == 0> */

  assert(AutoRes_HandleTypeIsValid (handleTypeId));

  for (iInstance = 0; 
       iInstance < AUTORES_MAX_HANDLE_INSTANCES; 
       iInstance++) {
    if (!HandleIsUsed[handleTypeId][iInstance]) {
      HandleIsUsed[handleTypeId][iInstance] = 1;

      return AutoRes_HandleFrom (handleTypeId, iInstance);
    }
  }

  assert(0);

  {
    char buf[256];

    sprintf (buf, " too many handles of type %d ",
	     handleTypeId);
    AutoRes_Error (buf);
  }

  return AUTORES_NULL_HANDLE;
}

void AutoRes_ReleaseHandle 
(AutoRes_Handle handle)
{
  int handleTypeId;
  int handleInstanceId;

  assert(AutoRes_HandleIsValid (handle));
  
  handleTypeId = AutoRes_HandleTypeId (handle);
  handleInstanceId = AutoRes_HandleInstanceId (handle);
  HandleIsUsed[handleTypeId][handleInstanceId] = 0;
}


