/* header file for autores_base.c */

#ifndef __AUTORES_BASE__
#define __AUTORES_BASE__

/* Base Module */



/* Utility */

void AutoRes_Error (const char *msg);

void AutoRes_Warning (const char *msg);



/* Tolerance management */

void AutoRes_SetTolerance (double tolerance);

double AutoRes_GetTolerance (void);



/* Handle Type */

#define AUTORES_MAX_HANDLE_TYPES 100

int /* boolean */
AutoRes_HandleTypeIsValid 
(int handleTypeId);

int AutoRes_NewHandleType (void);



/* Handle */

#define AUTORES_MAX_HANDLE_INSTANCES 1000

typedef int AutoRes_Handle;
#define AUTORES_NULL_HANDLE -1

int AutoRes_HandleTypeId 
(AutoRes_Handle handle);

int AutoRes_HandleInstanceId 
(AutoRes_Handle handle);

AutoRes_Handle AutoRes_HandleFrom 
(int handleTypeId, 
 int handleInstanceId);

int /* boolean */
AutoRes_HandleIsValid 
(AutoRes_Handle handle);

AutoRes_Handle AutoRes_NewHandle 
(int handleTypeId);

void AutoRes_ReleaseHandle 
(AutoRes_Handle handle);



#define AUTORES_DECLARE_HANDLE(className) \
void className##_Initialize (void); \
\
int \
className##_HandleIsValid \
(AutoRes_Handle handle); \
\
className *className##_Implementation \
(AutoRes_Handle handle); \
\
className *className##_NewImplementation (void); 



#define AUTORES_IMPLEMENT_HANDLE(className) \
static int className##TypeId; \
static className *className##s[AUTORES_MAX_HANDLE_INSTANCES]; \
\
void className##_Initialize (void) \
{ \
  className##TypeId = AutoRes_NewHandleType (); \
} \
\
int \
className##_HandleIsValid \
(AutoRes_Handle handle) \
{ \
  if (AutoRes_HandleIsValid (handle)) { \
    return AutoRes_HandleTypeId (handle) == className##TypeId; \
  } \
  return 0; \
} \
\
className *className##_Implementation \
(AutoRes_Handle handle) \
{ \
  int handleInstanceId; \
\
  assert(0 < className##TypeId); \
  assert(className##_HandleIsValid (handle)); \
\
  handleInstanceId = AutoRes_HandleInstanceId (handle); \
  if (className##s[handleInstanceId] == NULL) { \
    className##s[handleInstanceId] = (className *)malloc (sizeof (className)); \
    assert(className##s[handleInstanceId] != NULL); \
  } \
  memset (className##s[handleInstanceId], 0, sizeof (className)); \
  return className##s[handleInstanceId]; \
} \
\
className *className##_NewImplementation (void) \
{ \
  AutoRes_Handle handle; \
  className *instance; \
\
  handle = AutoRes_NewHandle (className##TypeId); \
  assert(handle != AUTORES_NULL_HANDLE); \
  instance = className##_Implementation (handle); \
  instance->handle = handle; \
  return instance; \
} 



#endif  /* __AUTORES_BASE__ */






