/* header file for autores_bucket2d.c */

#ifndef __AUTORES_BUCKET2D__
#define __AUTORES_BUCKET2D__



/* Bucket2D */

typedef struct _AutoRes_Bucket2D {
  AutoRes_Handle handle;

  double minRangeX;
  double minRangeY;
  double maxRangeX;
  double maxRangeY;
  int xCells;
  int yCells;

  AutoRes_ListTable *listTable;  /* OWN */
} AutoRes_Bucket2D;

AUTORES_DECLARE_HANDLE(AutoRes_Bucket2D);

AutoRes_Bucket2D *AutoRes_Bucket2D_New 
(double minRangeX, double minRangeY, 
 double maxRangeX, double maxRangeY);

void AutoRes_Bucket2D_delete 
(AutoRes_Bucket2D *self);

int /* boolean */
AutoRes_Bucket2D_reserve 
(AutoRes_Bucket2D *self,
 int xCells, int yCells, 
 int nAverageLinksPerList);

void AutoRes_Bucket2D_getMinRange 
(AutoRes_Bucket2D *self,
 double *minRangeX_OUT, double *minRangeY_OUT);

void AutoRes_Bucket2D_getMaxRange 
(AutoRes_Bucket2D *self,
 double *maxRangeX_OUT, double *maxRangeY_OUT);

void AutoRes_Bucket2D_getNCells 
(AutoRes_Bucket2D *self,
 int *xCells_OUT, int *yCells_OUT);

void AutoRes_Bucket2D_getCellIdWithin 
(AutoRes_Bucket2D *self,
 double minRangeX, double minRangeY,
 double maxRangeX, double maxRangeY,
 int *cx0_OUT, int *cy0_OUT, 
 int *cx1_OUT, int *cy1_OUT);

void AutoRes_Bucket2D_getCellIdAt 
(AutoRes_Bucket2D *self,
 double x, double y,
 int *cx_OUT, int *cy_OUT);

int /* boolean */   
AutoRes_Bucket2D_isOutOfRange 
(AutoRes_Bucket2D *self,
 int cx, int cy);

int AutoRes_Bucket2D_startLinkIdIn 
(AutoRes_Bucket2D *self,
 int cx, int cy);

int AutoRes_Bucket2D_startLinkIdOutOfRange 
(AutoRes_Bucket2D *self);

int AutoRes_Bucket2D_nextLinkId 
(AutoRes_Bucket2D *self,
 int linkId);

int AutoRes_Bucket2D_objectId 
(AutoRes_Bucket2D *self,
 int linkId);

int /* boolean */   
AutoRes_Bucket2D_addObjectWithin 
(AutoRes_Bucket2D *self,
 double minRangeX, double minRangeY,
 double maxRangeX, double maxRangeY,
 int objectId);

int /* boolean */   
AutoRes_Bucket2D_addObjectAt 
(AutoRes_Bucket2D *self,
 double x, double y,
 int objectId);

void AutoRes_Bucket2D_removeObjectWithin 
(AutoRes_Bucket2D *self,
 double minRangeX, double minRangeY,
 double maxRangeX, double maxRangeY,
 int objectId);

void AutoRes_Bucket2D_removeObjectAt 
(AutoRes_Bucket2D *self,
 double x, double y,
 int objectId);



#endif  /* __AUTORES_BUCKET2D__ */



