/* header file for autores_bucket3d.c */

#ifndef __AUTORES_BUCKET3D__
#define __AUTORES_BUCKET3D__



/* Bucket3D */

typedef struct _AutoRes_Bucket3D {
  AutoRes_Handle handle;

  double minRangeX;
  double minRangeY;
  double minRangeZ;
  double maxRangeX;
  double maxRangeY;
  double maxRangeZ;
  int xCells;
  int yCells;
  int zCells;

  AutoRes_ListTable *listTable;  /* OWN */
} AutoRes_Bucket3D;

AUTORES_DECLARE_HANDLE(AutoRes_Bucket3D);

AutoRes_Bucket3D *AutoRes_Bucket3D_New 
(double minRangeX, double minRangeY, double minRangeZ, 
 double maxRangeX, double maxRangeY, double maxRangeZ);

void AutoRes_Bucket3D_delete 
(AutoRes_Bucket3D *self);

int /* boolean */
AutoRes_Bucket3D_reserve 
(AutoRes_Bucket3D *self,
 int xCells, int yCells, int zCells, 
 int nAverageLinksPerList);

void AutoRes_Bucket3D_getMinRange 
(AutoRes_Bucket3D *self,
 double *minRangeX_OUT, double *minRangeY_OUT, double *minRangeZ_OUT);

void AutoRes_Bucket3D_getMaxRange 
(AutoRes_Bucket3D *self,
 double *maxRangeX_OUT, double *maxRangeY_OUT, double *maxRangeZ_OUT);

void AutoRes_Bucket3D_getNCells 
(AutoRes_Bucket3D *self,
 int *xCells_OUT, int *yCells_OUT, int *zCells_OUT);

void AutoRes_Bucket3D_getCellIdWithin 
(AutoRes_Bucket3D *self,
 double minRangeX, double minRangeY, double minRangeZ,
 double maxRangeX, double maxRangeY, double maxRangeZ,
 int *cx0_OUT, int *cy0_OUT, int *cz0_OUT, 
 int *cx1_OUT, int *cy1_OUT, int *cz1_OUT);

void AutoRes_Bucket3D_getCellIdAt 
(AutoRes_Bucket3D *self,
 double x, double y, double z,
 int *cx_OUT, int *cy_OUT, int *cz_OUT);

int /* boolean */    
AutoRes_Bucket3D_isOutOfRange 
(AutoRes_Bucket3D *self,
 int cx, int cy, int cz);

int AutoRes_Bucket3D_startLinkIdIn 
(AutoRes_Bucket3D *self,
 int cx, int cy, int cz);

int AutoRes_Bucket3D_startLinkIdOutOfRange 
(AutoRes_Bucket3D *self);

int AutoRes_Bucket3D_nextLinkId 
(AutoRes_Bucket3D *self,
 int linkId);

int AutoRes_Bucket3D_objectId 
(AutoRes_Bucket3D *self,
 int linkId);

int /* boolean */    
AutoRes_Bucket3D_addObjectWithin 
(AutoRes_Bucket3D *self,
 double minRangeX, double minRangeY, double minRangeZ,
 double maxRangeX, double maxRangeY, double maxRangeZ,
 int objectId);

int /* boolean */   
AutoRes_Bucket3D_addObjectAt 
(AutoRes_Bucket3D *self,
 double x, double y, double z,
 int objectId);

void AutoRes_Bucket3D_removeObjectWithin 
(AutoRes_Bucket3D *self,
 double minRangeX, double minRangeY, double minRangeZ,
 double maxRangeX, double maxRangeY, double maxRangeZ,
 int objectId);

void AutoRes_Bucket3D_removeObjectAt 
(AutoRes_Bucket3D *self,
 double x, double y, double z,
 int objectId);



#endif  /* __AUTORES_BUCKET3D__ */



