/* autores_column.c : Table-based Memory Manager : Column */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_memory.h"
#include "autores_column.h"
#include "autores_integer_column.h"
#include "autores_real_column.h"



/* Table-based Memory Manager : Column */



/* Column */

void AutoRes_Column_Initialize (void)
{
  AutoRes_IntegerColumn_Initialize ();
  AutoRes_Integer1DColumn_Initialize ();
  AutoRes_RealColumn_Initialize ();
  AutoRes_Real1DColumn_Initialize ();
}

int /* boolean */
AutoRes_Column_HandleIsValid 
(AutoRes_Handle handle, 
 AutoRes_ColumnType columnType)
{
  switch (columnType) {
  case AUTORES_COLUMN_INTEGER:
    return AutoRes_IntegerColumn_HandleIsValid (handle);
    break;
  case AUTORES_COLUMN_INTEGER1D:
    return AutoRes_Integer1DColumn_HandleIsValid (handle);
    break;
  case AUTORES_COLUMN_REAL:
    return AutoRes_RealColumn_HandleIsValid (handle);
    break;
  case AUTORES_COLUMN_REAL1D:
    return AutoRes_Real1DColumn_HandleIsValid (handle);
    break;
  default:
    assert(0);
    break;
  }
  return 0;
}

void AutoRes_Column_delete 
(void *self, 
 AutoRes_ColumnType columnType)
{
  switch (columnType) {
  case AUTORES_COLUMN_INTEGER:
    AutoRes_IntegerColumn_delete 
      ((AutoRes_IntegerColumn *)self);
    break;
  case AUTORES_COLUMN_INTEGER1D:
    AutoRes_Integer1DColumn_delete 
      ((AutoRes_Integer1DColumn *)self);
    break;
  case AUTORES_COLUMN_REAL:
    AutoRes_RealColumn_delete 
      ((AutoRes_RealColumn *)self);
    break;
  case AUTORES_COLUMN_REAL1D:
    AutoRes_Real1DColumn_delete 
      ((AutoRes_Real1DColumn *)self);
    break;
  default:
    assert(0);
    break;
  }
}

int /* boolean */
AutoRes_Column_allocate 
(void *self, 
 AutoRes_ColumnType columnType)
{
  switch (columnType) {
  case AUTORES_COLUMN_INTEGER:
    return AutoRes_IntegerColumn_allocate 
      ((AutoRes_IntegerColumn *)self);
    break;
  case AUTORES_COLUMN_INTEGER1D:
    return AutoRes_Integer1DColumn_allocate 
      ((AutoRes_Integer1DColumn *)self);
    break;
  case AUTORES_COLUMN_REAL:
    return AutoRes_RealColumn_allocate 
      ((AutoRes_RealColumn *)self);
    break;
  case AUTORES_COLUMN_REAL1D:
    return AutoRes_Real1DColumn_allocate 
      ((AutoRes_Real1DColumn *)self);
    break;
  default:
    assert(0);
    break;
  }
  return 0;
}

void AutoRes_Column_setLockFlag 
(void *self, 
 AutoRes_ColumnType columnType,
 int flag)
{
  switch (columnType) {
  case AUTORES_COLUMN_INTEGER:
    AutoRes_IntegerColumn_setLockFlag 
      ((AutoRes_IntegerColumn *)self, flag);
    break;
  case AUTORES_COLUMN_INTEGER1D:
    AutoRes_Integer1DColumn_setLockFlag 
      ((AutoRes_Integer1DColumn *)self, flag);
    break;
  case AUTORES_COLUMN_REAL:
    AutoRes_RealColumn_setLockFlag 
      ((AutoRes_RealColumn *)self, flag);
    break;
  case AUTORES_COLUMN_REAL1D:
    AutoRes_Real1DColumn_setLockFlag 
      ((AutoRes_Real1DColumn *)self, flag);
    break;
  default:
    assert(0);
    break;
  }
}

struct _AutoRes_Table *AutoRes_Column_table 
(void *self, 
 AutoRes_ColumnType columnType)
{
  switch (columnType) {
  case AUTORES_COLUMN_INTEGER:
    return AutoRes_IntegerColumn_table 
      ((AutoRes_IntegerColumn *)self);
    break;
  case AUTORES_COLUMN_INTEGER1D:
    return AutoRes_Integer1DColumn_table 
      ((AutoRes_Integer1DColumn *)self);
    break;
  case AUTORES_COLUMN_REAL:
    return AutoRes_RealColumn_table 
      ((AutoRes_RealColumn *)self);
    break;
  case AUTORES_COLUMN_REAL1D:
    return AutoRes_Real1DColumn_table 
      ((AutoRes_Real1DColumn *)self);
    break;
  default:
    assert(0);
    break;
  }
  return NULL;
}

void AutoRes_Column_getName 
(void *self, 
 AutoRes_ColumnType columnType,
 char *name_OUT)
{
  switch (columnType) {
  case AUTORES_COLUMN_INTEGER:
    AutoRes_IntegerColumn_getName 
      ((AutoRes_IntegerColumn *)self, 
       name_OUT);
    break;
  case AUTORES_COLUMN_INTEGER1D:
    AutoRes_Integer1DColumn_getName 
      ((AutoRes_Integer1DColumn *)self, 
       name_OUT);
    break;
  case AUTORES_COLUMN_REAL:
    AutoRes_RealColumn_getName 
      ((AutoRes_RealColumn *)self, 
       name_OUT);
    break;
  case AUTORES_COLUMN_REAL1D:
    AutoRes_Real1DColumn_getName 
      ((AutoRes_Real1DColumn *)self, 
       name_OUT);
    break;
  default:
    assert(0);
    break;
  }
}



/* Table */

AUTORES_IMPLEMENT_HANDLE(AutoRes_Table);

AutoRes_Table *AutoRes_Table_New 
(const char *name)
{
  AutoRes_Table *table;

  assert(0 < strlen (name));
  assert(strlen (name) < AUTORES_MAX_TABLE_NAME_LENGTH);

  table = AutoRes_Table_NewImplementation ();

  table->name = (char *)malloc (strlen (name) + 1);
  assert(table->name != NULL);
  strcpy (table->name, name);

  table->nColumns = 0;
  table->nRecords = 0;
  table->nReservedRecords = 0;
  
  return table;
}

void AutoRes_Table_delete 
(AutoRes_Table *self)
{
  int iColumn;

  free (self->name);
  self->name = NULL;

  for (iColumn = 0; iColumn < self->nColumns; iColumn++) {
    AutoRes_Column_delete 
      (self->columns[iColumn], self->columnTypes[iColumn]);
    self->columns[iColumn] = NULL;
  }

  AutoRes_ReleaseHandle (self->handle);
  self->handle = AUTORES_NULL_HANDLE;
}

int /* boolean */
AutoRes_Table_addColumn 
(AutoRes_Table *self,
 void *column,
 AutoRes_ColumnType columnType)
{
  int flag;

  assert(self->nColumns < AUTORES_MAX_TABLE_COLUMNS);

  self->columns[self->nColumns] = column;
  self->columnTypes[self->nColumns] = columnType;
  self->nColumns++;

  flag = AutoRes_Column_allocate (column, columnType);
  if (!flag) return 0;

  return 1;
}

int /* boolean */
AutoRes_Table_addIntegerColumn 
(AutoRes_Table *self,
 void *column)
{
  return AutoRes_Table_addColumn 
    (self, column, AUTORES_COLUMN_INTEGER);
}

int /* boolean */
AutoRes_Table_addInteger1DColumn 
(AutoRes_Table *self,
 void *column)
{
  return AutoRes_Table_addColumn 
    (self, column, AUTORES_COLUMN_INTEGER1D);
}

int /* boolean */
AutoRes_Table_addRealColumn 
(AutoRes_Table *self,
 void *column)
{
  return AutoRes_Table_addColumn 
    (self, column, AUTORES_COLUMN_REAL);
}

int /* boolean */
AutoRes_Table_addReal1DColumn 
(AutoRes_Table *self,
 void *column)
{
  return AutoRes_Table_addColumn 
    (self, column, AUTORES_COLUMN_REAL1D);
}

void AutoRes_Table_removeColumn 
(AutoRes_Table *self,
 int columnId)
{
  int iColumn;

  assert(0 <= columnId);
  assert(columnId < self->nColumns);

  AutoRes_Column_delete 
    (self->columns[columnId], self->columnTypes[columnId]);

  for (iColumn = columnId; 
       iColumn < self->nColumns - 1; 
       iColumn++) {
    self->columns[iColumn] = self->columns[iColumn + 1];
    self->columnTypes[iColumn] = self->columnTypes[iColumn + 1];
  }
  self->nColumns--;
}

int /* boolean */
AutoRes_Table_allocateRecords 
(AutoRes_Table *self,
 int nRecords)
{
  int iColumn;
  int flag;

  assert(0 <= nRecords);

  self->nRecords = nRecords;
  self->nReservedRecords = nRecords;

  for (iColumn = 0; iColumn < self->nColumns; iColumn++) {
    flag = AutoRes_Column_allocate 
      (self->columns[iColumn], self->columnTypes[iColumn]);
    if (!flag) return 0;
  }

  return 1;
}

int /* boolean */
AutoRes_Table_reserveRecords 
(AutoRes_Table *self,
 int nRecords)
{
  int iColumn;
  int flag;

  assert(0 <= nRecords);

  self->nReservedRecords = nRecords;
  self->nRecords = 0;

  for (iColumn = 0; iColumn < self->nColumns; iColumn++) {
    flag = AutoRes_Column_allocate 
      (self->columns[iColumn], self->columnTypes[iColumn]);
    if (!flag) return 0;
  }

  return 1;
}

int AutoRes_Table_addRecord 
(AutoRes_Table *self)
{
  int iColumn;
  int newRecordId;
  int flag;

  assert(0 < self->nReservedRecords);

  if (self->nReservedRecords == self->nRecords) {
    self->nReservedRecords *= 2;

    for (iColumn = 0; iColumn < self->nColumns; iColumn++) {
      flag = AutoRes_Column_allocate 
	(self->columns[iColumn], self->columnTypes[iColumn]);
      if (!flag) return -1;
    }
  }

  newRecordId = self->nRecords;
  self->nRecords++;

  return newRecordId;
}

void AutoRes_Table_packNRecords 
(AutoRes_Table *self,
 int nRecords)
{
  assert(0 <= nRecords);
  assert(nRecords <= self->nReservedRecords);

  self->nRecords = nRecords;
}

void AutoRes_Table_getName 
(AutoRes_Table *self,
 char *name_OUT)
{
  strcpy (name_OUT, self->name);
}

int AutoRes_Table_nColumns 
(AutoRes_Table *self)
{
  return self->nColumns;
}

void *AutoRes_Table_column 
(AutoRes_Table *self,
 int columnId)
{
  assert(0 <= columnId);
  assert(columnId < self->nColumns);

  return self->columns[columnId];
}

AutoRes_ColumnType AutoRes_Table_columnType 
(AutoRes_Table *self,
 int columnId)
{
  assert(0 <= columnId);
  assert(columnId < self->nColumns);

  return self->columnTypes[columnId];
}

int AutoRes_Table_columnIdOfName 
(AutoRes_Table *self,
 const char *columnName)
{
  int iColumn;
  
  assert(0 < strlen (columnName));
  
  for (iColumn = 0; iColumn < self->nColumns; iColumn++) {
    char name[AUTORES_MAX_COLUMN_NAME_LENGTH];
    
    AutoRes_Column_getName 
      (self->columns[iColumn], self->columnTypes[iColumn],
       name);
    if (strcmp (columnName, name) == 0) {
      return iColumn;
    }
  }

  return -1;
}

int AutoRes_Table_columnIdOfColumn 
(AutoRes_Table *self,
 void *column)
{
  int iColumn;
  
  for (iColumn = 0; iColumn < self->nColumns; iColumn++) {
    if (self->columns[iColumn] == column) {
      return iColumn;
    }
  }
  assert(0);

  return -1;
}

void *AutoRes_Table_columnOfName 
(AutoRes_Table *self,
 const char *columnName)
{
  int columnId = AutoRes_Table_columnIdOfName (self, columnName);

  if (columnId != -1) {
    return AutoRes_Table_column (self, columnId);
  }
  return NULL;
}

int AutoRes_Table_nRecords 
(AutoRes_Table *self)
{
  return self->nRecords;
}

int AutoRes_Table_nReservedRecords 
(AutoRes_Table *self)
{
  return self->nReservedRecords;
}

int /* boolean */
AutoRes_Table_recordIdIsValid 
(AutoRes_Table *self,
 int recordId)
{
  return 0 <= recordId
    && recordId < self->nRecords;
}



