/* header file for autores_column.c */

#ifndef __AUTORES_COLUMN__
#define __AUTORES_COLUMN__

/* Table-based Memory Management System : Column */

struct _AutoRes_Table;



/* Column */

#define AUTORES_MAX_COLUMN_NAME_LENGTH 80

typedef enum {
  AUTORES_COLUMN_INTEGER,
  AUTORES_COLUMN_INTEGER1D, 
  AUTORES_COLUMN_REAL,
  AUTORES_COLUMN_REAL1D
} AutoRes_ColumnType;

void AutoRes_Column_Initialize (void);

int /* boolean */
AutoRes_Column_HandleIsValid 
(AutoRes_Handle handle, 
 AutoRes_ColumnType columnType);

void AutoRes_Column_delete 
(void *self, 
 AutoRes_ColumnType columnType);

int /* boolean */
AutoRes_Column_allocate 
(void *self, 
 AutoRes_ColumnType columnType);

void AutoRes_Column_setLockFlag 
(void *self, 
 AutoRes_ColumnType columnType,
 int flag);

struct _AutoRes_Table *AutoRes_Column_table 
(void *self, 
 AutoRes_ColumnType columnType);

void AutoRes_Column_getName 
(void *self, 
 AutoRes_ColumnType columnType,
 char *name_OUT);



/* Table */

#define AUTORES_MAX_TABLE_NAME_LENGTH 80
#define AUTORES_MAX_TABLE_COLUMNS 100

typedef struct _AutoRes_Table {
  AutoRes_Handle handle;

  char *name;  /* OWN */

  int nColumns;
  AutoRes_ColumnType columnTypes[AUTORES_MAX_TABLE_COLUMNS]; 
  void *columns[AUTORES_MAX_TABLE_COLUMNS];  /* OWN */

  int nRecords;
  int nReservedRecords;
} AutoRes_Table;

AUTORES_DECLARE_HANDLE(AutoRes_Table);

AutoRes_Table *AutoRes_Table_New 
(const char *name);

void AutoRes_Table_delete 
(AutoRes_Table *self);

int /* boolean */
AutoRes_Table_addColumn 
(AutoRes_Table *self,
 void *column,
 AutoRes_ColumnType columnType);

int /* boolean */
AutoRes_Table_addIntegerColumn 
(AutoRes_Table *self,
 void *column);

int /* boolean */
AutoRes_Table_addInteger1DColumn 
(AutoRes_Table *self,
 void *column);

int /* boolean */
AutoRes_Table_addRealColumn 
(AutoRes_Table *self,
 void *column);

int /* boolean */
AutoRes_Table_addReal1DColumn 
(AutoRes_Table *self,
 void *column);

void AutoRes_Table_removeColumn 
(AutoRes_Table *self,
 int columnId);

int /* boolean */
AutoRes_Table_allocateRecords 
(AutoRes_Table *self,
 int nRecords);

int /* boolean */
AutoRes_Table_reserveRecords 
(AutoRes_Table *self,
 int nRecords);

int AutoRes_Table_addRecord 
(AutoRes_Table *self);

void AutoRes_Table_packNRecords 
(AutoRes_Table *self,
 int nRecords);

void AutoRes_Table_getName 
(AutoRes_Table *self,
 char *name_OUT);

int AutoRes_Table_nColumns 
(AutoRes_Table *self);

void *AutoRes_Table_column 
(AutoRes_Table *self,
 int columnId);

AutoRes_ColumnType AutoRes_Table_columnType 
(AutoRes_Table *self,
 int columnId);

int AutoRes_Table_columnIdOfName 
(AutoRes_Table *self,
 const char *columnName);

int AutoRes_Table_columnIdOfColumn 
(AutoRes_Table *self,
 void *column);

void *AutoRes_Table_columnOfName 
(AutoRes_Table *self,
 const char *columnName);

int AutoRes_Table_nRecords 
(AutoRes_Table *self);

int AutoRes_Table_nReservedRecords 
(AutoRes_Table *self);

int /* boolean */
AutoRes_Table_recordIdIsValid 
(AutoRes_Table *self,
 int recordId);



#endif  /* __AUTORES_COLUMN__ */



