/* autores_file.c : File I/O */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_file.h"



/* File I/O */

#if 1
#define FULL_DIGIT
#endif



/* File */

AUTORES_IMPLEMENT_HANDLE(AutoRes_File);

AutoRes_File *AutoRes_File_New (void)
{
  AutoRes_File *file;

  file = AutoRes_File_NewImplementation ();

  file->writeFlag = 0;
  file->binaryFlag = 0;
  file->fp = NULL;

  return file;
}

void AutoRes_File_delete 
(AutoRes_File *self)
{
  assert(self->fp == NULL);

  AutoRes_ReleaseHandle (self->handle);
  self->handle = AUTORES_NULL_HANDLE;
}

int /* boolean */
AutoRes_File_open 
(AutoRes_File *self,
 const char *fileName,
 int writeFlag,
 int binaryFlag)
{
  assert(self->fp == NULL);
  assert(0 < strlen (fileName));
  
  self->writeFlag = writeFlag;
  self->binaryFlag = binaryFlag;
  
  if (writeFlag) {
    if (binaryFlag) {
      self->fp = fopen (fileName, "wb");
    } else {
      self->fp = fopen (fileName, "w");
    }
  } else {
    if (binaryFlag) {
      self->fp = fopen (fileName, "rb");
    } else {
      self->fp = fopen (fileName, "r");
    }
  }
  assert(self->fp != NULL 
	 || self->fp == NULL);
  
  if (self->fp == NULL) return 0;
  
  return 1;
}

int /* boolean */
AutoRes_File_close 
(AutoRes_File *self)
{
  assert(self->fp != NULL);

  fclose (self->fp);
  self->fp = NULL;

  return 1;
}

int /* boolean */
AutoRes_File_integer 
(AutoRes_File *self,
 int *value_IO)
{
  if (self->writeFlag) {
    if (self->binaryFlag) {
      int count;

      count = fwrite (value_IO, sizeof (int), 1, self->fp);
      if (count != 1) return 0;
    } else {
      int count;

      count = fprintf (self->fp, "%d ", 
		       *value_IO);
      if (count == 0) return 0;
    }
  } else {
    if (self->binaryFlag) {
      int count;

      count = fread (value_IO, sizeof (int), 1, self->fp);
      if (count != 1) return 0;
    } else {
      int count;

      count = fscanf (self->fp, "%d", 
		      value_IO);
      if (count != 1) return 0;
    }
  }

  return 1;
}

int /* boolean */
AutoRes_File_real 
(AutoRes_File *self,
 double *value_IO)
{
  if (self->writeFlag) {
    if (self->binaryFlag) {
      int count;

      count = fwrite (value_IO, sizeof (double), 1, self->fp);
      if (count != 1) return 0;
    } else {
      int count;

#ifdef FULL_DIGIT
      count = fprintf (self->fp, "%22.15e ", 
		       *value_IO);
#else
      count = fprintf (self->fp, "%f ", 
		       *value_IO);
#endif
      if (count == 0) return 0;
    }
  } else {
    if (self->binaryFlag) {
      int count;

      count = fread (value_IO, sizeof (double), 1, self->fp);
      if (count != 1) return 0;
    } else {
      int count;

      count = fscanf (self->fp, "%lf", 
		      value_IO);
      if (count != 1) return 0;
    }
  }

  return 1;
}

int /* boolean */
AutoRes_File_string 
(AutoRes_File *self,
 int length,
 char *value_IO)
{
  /* NEED TO REDEFINE */
  /* how about Fortran space-filled string ? */

  if (self->writeFlag) {
    if (self->binaryFlag) {
      int count;

      count = fwrite (value_IO, length, 1, self->fp);
      if (count != 1) return 0;
    } else {
      int count;

      count = fprintf (self->fp, "%s ", 
		       value_IO);
      if (count == 0) return 0;
    }
  } else {
    if (self->binaryFlag) {
      int count;

      count = fread (value_IO, length, 1, self->fp);
      if (count != 1) return 0;
    } else {
      int count;

      /* NEED TO REDEFINE */
      /* how to check string length ? */
      count = fscanf (self->fp, "%s", 
		      value_IO);
      if (count != 1) return 0;
    }
  }

  return 1;
}

int /* boolean */
AutoRes_File_nextLine 
(AutoRes_File *self)
{
  if (self->writeFlag) {
    if (!self->binaryFlag) {
      fprintf (self->fp, "\n");
    }
  }

  return 1;
}

int /* boolean */
AutoRes_File_writeFlag
(AutoRes_File *self)
{
  return self->writeFlag;
}

int /* boolean */
AutoRes_File_binaryFlag
(AutoRes_File *self)
{
  return self->binaryFlag;
}

