/* autores_initialize.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_file.h"
#include "autores_field.h"
#include "autores_io.h"
#include "autores_memory.h"
#include "autores_column.h"
#include "autores_integer_column.h"
#include "autores_real_column.h"
#include "autores_table_io.h"
#include "autores_collection.h"
#include "autores_use_flag.h"
#include "autores_free_list.h"
#include "autores_ordered.h"
#include "autores_array.h"

#include "autores_sorter.h"
#include "autores_selection.h"
#include "autores_list.h"
#include "autores_list_table.h"
#include "autores_bucket1d.h"
#include "autores_bucket2d.h"
#include "autores_bucket3d.h"

#include "autores_initialize.h"



#define AUTORES_DEFAULT_MEMORY_SIZE 100000000
/* 100 MB */

int /* boolean */
AutoRes_Initialize 
(int memorySize)
{
  int result = 1;

  assert(0 < memorySize
	 || memorySize == 0);

  if (memorySize == 0) {
    memorySize = AUTORES_DEFAULT_MEMORY_SIZE;
  }

  result = AutoRes_Memory_Initialize (memorySize);
  if (!result) {
    fprintf (stderr, 
	     " AutoRes : failed to allocate memory (%d bytes). \n",
	     memorySize);
    return result;
  }

  AutoRes_File_Initialize ();

  AutoRes_Field_Initialize ();

  AutoRes_InOut_Initialize ();

  AutoRes_Column_Initialize ();
  AutoRes_Table_Initialize ();

  AutoRes_TableInOut_Initialize ();

  AutoRes_UseFlagCollection_Initialize ();
  AutoRes_FreeListCollection_Initialize ();
  AutoRes_OrderedCollection_Initialize ();
  AutoRes_Array2D_Initialize ();
  AutoRes_Array3D_Initialize ();

  AutoRes_IntegerSorter_Initialize ();
  AutoRes_RealSorter_Initialize ();
  AutoRes_Selection_Initialize ();
  AutoRes_List_Initialize ();
  AutoRes_ListTable_Initialize ();
  AutoRes_Bucket1D_Initialize ();
  AutoRes_Bucket2D_Initialize ();
  AutoRes_Bucket3D_Initialize ();

  return result;
}
