/* header file for autores_integer_column.c */

#ifndef __AUTORES_INTEGER_COLUMN__
#define __AUTORES_INTEGER_COLUMN__

/* Table-based Memory Management System : Integer Column */



/* IntegerColumn */

typedef struct _AutoRes_IntegerColumn {
  AutoRes_Handle handle;

  AutoRes_Table *table;
  char *name;  /* OWN */
  AutoRes_Block *block;  /* OWN */
} AutoRes_IntegerColumn;

AUTORES_DECLARE_HANDLE(AutoRes_IntegerColumn);

AutoRes_IntegerColumn *AutoRes_IntegerColumn_New 
(AutoRes_Table *table,
 const char *name);

void AutoRes_IntegerColumn_delete 
(AutoRes_IntegerColumn *self);

int /* boolean */
AutoRes_IntegerColumn_allocate 
(AutoRes_IntegerColumn *self);

void AutoRes_IntegerColumn_setLockFlag 
(AutoRes_IntegerColumn *self,
 int flag);

void AutoRes_IntegerColumn_setValue 
(AutoRes_IntegerColumn *self,
 int recordId,
 int value);

AutoRes_Table *AutoRes_IntegerColumn_table 
(AutoRes_IntegerColumn *self);

void AutoRes_IntegerColumn_getName 
(AutoRes_IntegerColumn *self,
 char *name_OUT);

int AutoRes_IntegerColumn_value 
(AutoRes_IntegerColumn *self,
 int recordId);

/* WARNING : test stage */
int *AutoRes_IntegerColumn_dataArea 
(AutoRes_IntegerColumn *self);

#define AUTORES_I_SET(self, recordId, value) \
  AutoRes_IntegerColumn_setValue ((self), (recordId), (value))

#define AUTORES_I(self, recordId) \
  AutoRes_IntegerColumn_value ((self), (recordId))



/* Integer1DColumn */

typedef struct _AutoRes_Integer1DColumn {
  AutoRes_Handle handle;

  AutoRes_Table *table;
  char *name;  /* OWN */
  int size;
  AutoRes_Block *block;  /* OWN */
} AutoRes_Integer1DColumn;

AUTORES_DECLARE_HANDLE(AutoRes_Integer1DColumn);

AutoRes_Integer1DColumn *AutoRes_Integer1DColumn_New 
(AutoRes_Table *table,
 const char *name,
 int size);

void AutoRes_Integer1DColumn_delete 
(AutoRes_Integer1DColumn *self);

int /* boolean */
AutoRes_Integer1DColumn_allocate 
(AutoRes_Integer1DColumn *self);

void AutoRes_Integer1DColumn_setLockFlag 
(AutoRes_Integer1DColumn *self,
 int flag);

void AutoRes_Integer1DColumn_setValue 
(AutoRes_Integer1DColumn *self,
 int recordId, 
 int index,
 int value);

AutoRes_Table *AutoRes_Integer1DColumn_table 
(AutoRes_Integer1DColumn *self);

void AutoRes_Integer1DColumn_getName 
(AutoRes_Integer1DColumn *self,
 char *name_OUT);

int AutoRes_Integer1DColumn_size 
(AutoRes_Integer1DColumn *self);

int AutoRes_Integer1DColumn_value 
(AutoRes_Integer1DColumn *self,
 int recordId,
 int index);

/* WARNING : test stage */
int* AutoRes_Integer1DColumn_dataArea 
(AutoRes_Integer1DColumn *self);

#define AUTORES_I1D_SET(self, recordId, index, value) \
  AutoRes_Integer1DColumn_setValue ((self), (recordId), (index), (value))

#define AUTORES_I1D(self, recordId, index) \
  AutoRes_Integer1DColumn_value ((self), (recordId), (index))



#endif  /* __AUTORES_INTEGER_COLUMN__ */



