/* autores_integer_field.c : Integer Field */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_field.h"
#include "autores_integer_field.h"



/* Integer Field */



/* IntegerField */

AUTORES_IMPLEMENT_HANDLE(AutoRes_IntegerField);

AutoRes_IntegerField *AutoRes_IntegerField_New 
(const char *name)
{
  AutoRes_IntegerField *field;

  assert(0 < strlen (name));
  assert(strlen (name) < AUTORES_MAX_FIELD_NAME_LENGTH);

  field = AutoRes_IntegerField_NewImplementation ();

  field->name = (char *)malloc (strlen (name) + 1);
  assert(field->name != NULL);
  strcpy (field->name, name);

  field->value = 0;
  field->valuePtr = NULL;

  return field;
}

void AutoRes_IntegerField_delete 
(AutoRes_IntegerField *self)
{
  free (self->name);
  self->name = NULL;

  AutoRes_ReleaseHandle (self->handle);
  self->handle = AUTORES_NULL_HANDLE;
}

void AutoRes_IntegerField_associate 
(AutoRes_IntegerField *self,
 int *valuePtr)
{
  assert(valuePtr != NULL
	 || valuePtr == NULL);

  self->valuePtr = valuePtr;
}

void AutoRes_IntegerField_setValue 
(AutoRes_IntegerField *self,
 int value)
{
  if (self->valuePtr != NULL) {
    *self->valuePtr = value;
  } else {
    self->value = value;
  }
}

void AutoRes_IntegerField_getName 
(AutoRes_IntegerField *self,
 char *name_OUT)
{
  strcpy (name_OUT, self->name);
}

int AutoRes_IntegerField_value 
(AutoRes_IntegerField *self)
{
  if (self->valuePtr != NULL) {
    return *self->valuePtr;
  } else {
    return self->value;
  }
}



/* Integer1DField */

AUTORES_IMPLEMENT_HANDLE(AutoRes_Integer1DField);

AutoRes_Integer1DField *AutoRes_Integer1DField_New 
(const char *name,
 int size)
{
  AutoRes_Integer1DField *field;
  int index;

  assert(0 < strlen (name));
  assert(strlen (name) < AUTORES_MAX_FIELD_NAME_LENGTH);
  assert(0 < size);

  field = AutoRes_Integer1DField_NewImplementation ();

  field->name = (char *)malloc (strlen (name) + 1);
  assert(field->name != NULL);
  strcpy (field->name, name);

  field->size = size;
  field->valuesPtr = NULL; 

  field->values = (int *)malloc (sizeof (int) * size);
  assert(field->values != NULL);
  for (index = 0; index < size; index++) {
    field->values[index] = 0;
  }

  return field;
}

void AutoRes_Integer1DField_delete 
(AutoRes_Integer1DField *self)
{
  free (self->name);
  self->name = NULL;

  free (self->values);
  self->values = NULL;

  AutoRes_ReleaseHandle (self->handle);
  self->handle = AUTORES_NULL_HANDLE;
}

void AutoRes_Integer1DField_associate 
(AutoRes_Integer1DField *self,
 int *valuesPtr)
{
  assert(valuesPtr != NULL
	 || valuesPtr == NULL);

  self->valuesPtr = valuesPtr;
}

void AutoRes_Integer1DField_setValue 
(AutoRes_Integer1DField *self,
 int index,
 int value)
{
  assert(0 <= index);
  assert(index < self->size);
  
  if (self->valuesPtr != NULL) {
    self->valuesPtr[index] = value;
  } else {
    self->values[index] = value;
  }
}

void AutoRes_Integer1DField_getName 
(AutoRes_Integer1DField *self,
 char *name_OUT)
{
  strcpy (name_OUT, self->name);
}

int AutoRes_Integer1DField_size 
(AutoRes_Integer1DField *self)
{
  return self->size;
}

int AutoRes_Integer1DField_value 
(AutoRes_Integer1DField *self,
 int index)
{
  assert(0 <= index);
  assert(index < self->size);

  if (self->valuesPtr != NULL) {
    return self->valuesPtr[index];
  } else {
    return self->values[index];
  }
}



