/* header file for autores_io.c */

#ifndef __AUTORES_IO__
#define __AUTORES_IO__

/* Input and Output */



/* InOut */

#define AUTORES_MAX_IN_OUT_FIELDS 100

typedef struct _AutoRes_InOut {
  AutoRes_Handle handle;

  int nHeaderFields;
  AutoRes_FieldType headerFieldTypes[AUTORES_MAX_IN_OUT_FIELDS]; 
  void *headerFields[AUTORES_MAX_IN_OUT_FIELDS];  /* OWN */

  int nRecordFields;
  AutoRes_FieldType recordFieldTypes[AUTORES_MAX_IN_OUT_FIELDS]; 
  void *recordFields[AUTORES_MAX_IN_OUT_FIELDS];  /* OWN */

  struct _AutoRes_File *file;
  int nRecords;
  int *nRecordsPtr;
  int recordIdFlag;
} AutoRes_InOut;

AUTORES_DECLARE_HANDLE(AutoRes_InOut);

AutoRes_InOut *AutoRes_InOut_New (void);

void AutoRes_InOut_delete 
(AutoRes_InOut *self);

void AutoRes_InOut_addFieldIntoHeader
(AutoRes_InOut *self,
 void *field,
 AutoRes_FieldType fieldType);

void AutoRes_InOut_addIntegerFieldIntoHeader
(AutoRes_InOut *self,
 void *field);

void AutoRes_InOut_addInteger1DFieldIntoHeader
(AutoRes_InOut *self,
 void *field);

void AutoRes_InOut_addRealFieldIntoHeader
(AutoRes_InOut *self,
 void *field);

void AutoRes_InOut_addReal1DFieldIntoHeader
(AutoRes_InOut *self,
 void *field);

void AutoRes_InOut_addFieldIntoRecord
(AutoRes_InOut *self,
 void *field,
 AutoRes_FieldType fieldType);

void AutoRes_InOut_addIntegerFieldIntoRecord
(AutoRes_InOut *self,
 void *field);

void AutoRes_InOut_addInteger1DFieldIntoRecord
(AutoRes_InOut *self,
 void *field);

void AutoRes_InOut_addRealFieldIntoRecord
(AutoRes_InOut *self,
 void *field);

void AutoRes_InOut_addReal1DFieldIntoRecord
(AutoRes_InOut *self,
 void *field);

void AutoRes_InOut_setNRecords
(AutoRes_InOut *self,
 int nRecords);

void AutoRes_InOut_setRecordIdFlag
(AutoRes_InOut *self,
 int flag);

int /* boolean */
AutoRes_InOut_readMetaData
(AutoRes_InOut *self,
 const char *fileName);

int /* boolean */
AutoRes_InOut_writeMetaData
(AutoRes_InOut *self,
 const char *fileName);

void AutoRes_InOut_associateNRecords
(AutoRes_InOut *self,
 int *nRecordsPtr);

int /* boolean */
AutoRes_InOut_open
(AutoRes_InOut *self,
 const char *fileName,
 int writeFlag,
 int binaryFlag);

int /* boolean */
AutoRes_InOut_close 
(AutoRes_InOut *self);

int /* boolean */
AutoRes_InOut_doHeaderSection  
(AutoRes_InOut *self);

int /* boolean */
AutoRes_InOut_doEachOfRecordSection 
(AutoRes_InOut *self,
 int recordId);

int AutoRes_InOut_nHeaderFields 
(AutoRes_InOut *self);

void *AutoRes_InOut_headerField 
(AutoRes_InOut *self,
 int fieldId);

AutoRes_FieldType AutoRes_InOut_headerFieldType 
(AutoRes_InOut *self,
 int fieldId);

int AutoRes_InOut_headerFieldIdOfName
(AutoRes_InOut *self,
 const char* fieldName);

void *AutoRes_InOut_headerFieldOfName 
(AutoRes_InOut *self,
 const char* fieldName);

int AutoRes_InOut_nRecordFields 
(AutoRes_InOut *self);

void *AutoRes_InOut_recordField 
(AutoRes_InOut *self,
 int fieldId);

AutoRes_FieldType AutoRes_InOut_recordFieldType 
(AutoRes_InOut *self,
 int fieldId);

int AutoRes_InOut_recordFieldIdOfName
(AutoRes_InOut *self,
 const char* fieldName);

void *AutoRes_InOut_recordFieldOfName 
(AutoRes_InOut *self,
 const char* fieldName);

struct _AutoRes_File *AutoRes_InOut_file 
(AutoRes_InOut *self);

int AutoRes_InOut_nRecords 
(AutoRes_InOut *self);

int /* boolean */ 
AutoRes_InOut_recordIdFlag 
(AutoRes_InOut *self);



#endif  /* __AUTORES_IO__ */



