/* autores_memory.c : Memory Management System */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_memory.h"



#if 0
#define MEMORY_LOG
#endif



/* Memory Manager */

#define AUTORES_MAX_BLOCKS 10000
static AutoRes_Block Blocks[AUTORES_MAX_BLOCKS];

int /* boolean */
AutoRes_Memory_Initialize 
(int memorySize)
{
  int iBlock;

  assert(0 < memorySize);

  /* ignore <memorySize> */

  for (iBlock = 0; iBlock < AUTORES_MAX_BLOCKS; iBlock++) {
    Blocks[iBlock].dataArea = NULL;
    Blocks[iBlock].blockId = iBlock;
    Blocks[iBlock].useFlag = 0;
    Blocks[iBlock].memorySize = 0;
    Blocks[iBlock].lockFlag = 0;
  }

  return 1;
}

AutoRes_Block *AutoRes_Memory_AllocateBlock 
(int memorySize)
{
  AutoRes_Block *block;
  int iBlock;
  char *dataArea;
  
  assert(0 <= memorySize);

  block = NULL;
  for (iBlock = 0; iBlock < AUTORES_MAX_BLOCKS; iBlock++) {
    if (!Blocks[iBlock].useFlag) {
      block = &Blocks[iBlock];
      break;
    }
  }
  if (block == NULL) {
    AutoRes_Error (" too many memory blocks ");
    
    return NULL;
  }
  
  dataArea = NULL;
  if (0 < memorySize) { 
#ifdef MEMORY_LOG
    fprintf 
      (stderr, 
       " AutoRes_Memory_AllocateBlock : malloc block %d size %d \n",
       block->blockId, 
       memorySize);
#endif

    dataArea = (char *)malloc (memorySize);
    if (dataArea == NULL) {
      char buf[256];
      
      sprintf (buf, " short of memory, size %d ",
	       memorySize);
      AutoRes_Error (buf);

      return NULL;
    }
  } else {
#ifdef MEMORY_LOG
    fprintf 
      (stderr, 
       " AutoRes_Memory_AllocateBlock : clear block %d \n",
       block->blockId);
#endif
  }

  block->useFlag = 1;
  block->dataArea = dataArea;
  block->memorySize = memorySize;
  block->lockFlag = 0;

  assert(block->useFlag);
  assert(!block->lockFlag);

  return block;
}

void AutoRes_Memory_FreeBlock 
(AutoRes_Block *block)
{
  assert(block->useFlag);
  
  if (block->dataArea != NULL) {
#ifdef MEMORY_LOG
    fprintf 
      (stderr, 
       " AutoRes_Memory_FreeBlock : free block %d size %d \n",
       block->blockId, 
       block->memorySize);
#endif
    
    free (block->dataArea);
  }
  
  block->useFlag = 0;
  block->dataArea = NULL;
  block->memorySize = 0;
  block->lockFlag = 0;
}

int /* boolean */ 
AutoRes_Memory_ReallocateBlock 
(AutoRes_Block *block,
 int memorySize)
{
  char *dataArea;
  
  assert(block->useFlag);
  assert(!block->lockFlag);
  assert(0 <= memorySize);

  if (memorySize == block->memorySize) {
    return 1;
  }

  dataArea = NULL;
  if (0 < memorySize) { 
#ifdef MEMORY_LOG
    fprintf 
      (stderr, 
       " AutoRes_Memory_ReallocateBlock : malloc block %d size %d \n",
       block->blockId, 
       memorySize);
#endif

    dataArea = (char *)malloc (memorySize);
    if (dataArea == NULL) {
      char buf[256];
      
      sprintf (buf, " short of memory, size %d ",
	       memorySize);
      AutoRes_Error (buf);
      
      return 0;
    }
  } else {
#ifdef MEMORY_LOG
    fprintf 
      (stderr, 
       " AutoRes_Memory_ReallocateBlock : clear block %d \n",
       block->blockId);
#endif
  }

  if (dataArea != NULL 
      && block->dataArea != NULL) {
    int size = (block->memorySize < memorySize) 
      ? block->memorySize : memorySize;
    
    memcpy (dataArea, block->dataArea, size);
  }

  if (block->dataArea != NULL) {
#ifdef MEMORY_LOG
    fprintf 
      (stderr, 
       " AutoRes_Memory_ReallocateBlock : free block %d size %d \n",
       block->blockId, 
       block->memorySize);
#endif
    
    free (block->dataArea);
  }

  block->dataArea = dataArea;
  block->memorySize = memorySize;
  
  return 1;
}

void AutoRes_Memory_SetLockFlagOnBlock 
(AutoRes_Block *block,
 int flag)
{
  block->lockFlag = flag;
}

