/* header file for autores_real_column.c */

#ifndef __AUTORES_REAL_COLUMN__
#define __AUTORES_REAL_COLUMN__

/* Table-based Memory Management System : Real Column */



/* RealColumn */

typedef struct _AutoRes_RealColumn {
  AutoRes_Handle handle;

  AutoRes_Table *table;
  char *name;  /* OWN */
  AutoRes_Block *block;  /* OWN */
} AutoRes_RealColumn;

AUTORES_DECLARE_HANDLE(AutoRes_RealColumn);

AutoRes_RealColumn *AutoRes_RealColumn_New 
(AutoRes_Table *table,
 const char *name);

void AutoRes_RealColumn_delete 
(AutoRes_RealColumn *self);

int /* boolean */
AutoRes_RealColumn_allocate 
(AutoRes_RealColumn *self);

void AutoRes_RealColumn_setLockFlag 
(AutoRes_RealColumn *self,
 int flag);

void AutoRes_RealColumn_setValue 
(AutoRes_RealColumn *self,
 int recordId,
 double value);

AutoRes_Table *AutoRes_RealColumn_table 
(AutoRes_RealColumn *self);

void AutoRes_RealColumn_getName 
(AutoRes_RealColumn *self,
 char *name_OUT);

double AutoRes_RealColumn_value 
(AutoRes_RealColumn *self,
 int recordId);

/* WARNING : test stage */
double *AutoRes_RealColumn_dataArea 
(AutoRes_RealColumn *self);

#define AUTORES_R_SET(self, recordId, value) \
  AutoRes_RealColumn_setValue ((self), (recordId), (value))

#define AUTORES_R(self, recordId) \
  AutoRes_RealColumn_value ((self), (recordId))



/* Real1DColumn */

typedef struct _AutoRes_Real1DColumn {
  AutoRes_Handle handle;

  AutoRes_Table *table;
  char *name;  /* OWN */
  int size;
  AutoRes_Block *block;  /* OWN */
} AutoRes_Real1DColumn;

AUTORES_DECLARE_HANDLE(AutoRes_Real1DColumn);

AutoRes_Real1DColumn *AutoRes_Real1DColumn_New 
(AutoRes_Table *table,
 const char *name,
 int size);

void AutoRes_Real1DColumn_delete 
(AutoRes_Real1DColumn *self);

int /* boolean */
AutoRes_Real1DColumn_allocate 
(AutoRes_Real1DColumn *self);

void AutoRes_Real1DColumn_setLockFlag 
(AutoRes_Real1DColumn *self,
 int flag);

void AutoRes_Real1DColumn_setValue 
(AutoRes_Real1DColumn *self,
 int recordId, 
 int index,
 double value);

AutoRes_Table *AutoRes_Real1DColumn_table 
(AutoRes_Real1DColumn *self);

void AutoRes_Real1DColumn_getName 
(AutoRes_Real1DColumn *self,
 char *name_OUT);

int AutoRes_Real1DColumn_size 
(AutoRes_Real1DColumn *self);

double AutoRes_Real1DColumn_value 
(AutoRes_Real1DColumn *self,
 int recordId,
 int index);

/* WARNING : test stage */
double *AutoRes_Real1DColumn_dataArea 
(AutoRes_Real1DColumn *self);

#define AUTORES_R1D_SET(self, recordId, index, value) \
  AutoRes_Real1DColumn_setValue ((self), (recordId), (index), (value))

#define AUTORES_R1D(self, recordId, index) \
  AutoRes_Real1DColumn_value ((self), (recordId), (index))



#endif  /* __AUTORES_REAL_COLUMN__ */



