/* autores_real_field.c : Real Field */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_field.h"
#include "autores_real_field.h"



/* Real Field */



/* RealField */

AUTORES_IMPLEMENT_HANDLE(AutoRes_RealField);

AutoRes_RealField *AutoRes_RealField_New 
(const char *name)
{
  AutoRes_RealField *field;

  assert(0 < strlen (name));
  assert(strlen (name) < AUTORES_MAX_FIELD_NAME_LENGTH);

  field = AutoRes_RealField_NewImplementation ();

  field->name = (char *)malloc (strlen (name) + 1);
  assert(field->name != NULL);
  strcpy (field->name, name);

  field->value = 0.0;
  field->valuePtr = NULL;

  return field;
}

void AutoRes_RealField_delete 
(AutoRes_RealField *self)
{
  free (self->name);
  self->name = NULL;

  AutoRes_ReleaseHandle (self->handle);
  self->handle = AUTORES_NULL_HANDLE;
}

void AutoRes_RealField_associate 
(AutoRes_RealField *self,
 double *valuePtr)
{
  assert(valuePtr != NULL
	 || valuePtr == NULL);

  self->valuePtr = valuePtr;
}

void AutoRes_RealField_setValue 
(AutoRes_RealField *self,
 double value)
{
  if (self->valuePtr != NULL) {
    *self->valuePtr = value;
  } else {
    self->value = value;
  }
}

void AutoRes_RealField_getName 
(AutoRes_RealField *self,
 char *name_OUT)
{
  strcpy (name_OUT, self->name);
}

double AutoRes_RealField_value 
(AutoRes_RealField *self)
{
  if (self->valuePtr != NULL) {
    return *self->valuePtr;
  } else {
    return self->value;
  }
}



/* Real1DField */

AUTORES_IMPLEMENT_HANDLE(AutoRes_Real1DField);

AutoRes_Real1DField *AutoRes_Real1DField_New 
(const char *name,
 int size)
{
  AutoRes_Real1DField *field;
  int index;

  assert(0 < strlen (name));
  assert(strlen (name) < AUTORES_MAX_FIELD_NAME_LENGTH);
  assert(0 < size);

  field = AutoRes_Real1DField_NewImplementation ();

  field->name = (char *)malloc (strlen (name) + 1);
  assert(field->name != NULL);
  strcpy (field->name, name);

  field->size = size;
  field->valuesPtr = NULL; 

  field->values = (double *)malloc (sizeof (double) * size);
  assert(field->values != NULL);
  for (index = 0; index < size; index++) {
    field->values[index] = 0.0;
  }

  return field;
}

void AutoRes_Real1DField_delete 
(AutoRes_Real1DField *self)
{
  free (self->name);
  self->name = NULL;

  free (self->values);
  self->values = NULL;

  AutoRes_ReleaseHandle (self->handle);
  self->handle = AUTORES_NULL_HANDLE;
}

void AutoRes_Real1DField_associate 
(AutoRes_Real1DField *self,
 double *valuesPtr)
{
  assert(valuesPtr != NULL
	 || valuesPtr == NULL);

  self->valuesPtr = valuesPtr;
}

void AutoRes_Real1DField_setValue 
(AutoRes_Real1DField *self,
 int index,
 double value)
{
  assert(0 <= index);
  assert(index < self->size);
  
  if (self->valuesPtr != NULL) {
    self->valuesPtr[index] = value;
  } else {
    self->values[index] = value;
  }
}

void AutoRes_Real1DField_getName 
(AutoRes_Real1DField *self,
 char *name_OUT)
{
  strcpy (name_OUT, self->name);
}

int AutoRes_Real1DField_size 
(AutoRes_Real1DField *self)
{
  return self->size;
}

double AutoRes_Real1DField_value 
(AutoRes_Real1DField *self,
 int index)
{
  assert(0 <= index);
  assert(index < self->size);

  if (self->valuesPtr != NULL) {
    return self->valuesPtr[index];
  } else {
    return self->values[index];
  }
}



