/* autores_selection.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_memory.h"
#include "autores_column.h"
#include "autores_integer_column.h"
#include "autores_real_column.h"
#include "autores_selection.h"



#define TABLE_NAME_ENTRYS "AutoRes:entrys"
#define COLUMN_NAME_OBJECT_ID "AutoRes:objectId"



/* Integer Selection */

AUTORES_IMPLEMENT_HANDLE(AutoRes_Selection);

AutoRes_Selection *AutoRes_Selection_New (void)
{
  AutoRes_Selection *selection;
  int flag;

  selection = AutoRes_Selection_NewImplementation ();

  selection->entrys = AutoRes_Table_New (TABLE_NAME_ENTRYS);
  assert(selection->entrys != NULL);

  selection->entrys_objectId = AutoRes_IntegerColumn_New 
    (selection->entrys, COLUMN_NAME_OBJECT_ID);
  assert(selection->entrys_objectId != NULL);

  flag = AutoRes_Table_addIntegerColumn 
    (selection->entrys, selection->entrys_objectId);
  assert(flag);

  return selection;
}

void AutoRes_Selection_delete 
(AutoRes_Selection *self)
{
  AutoRes_Table_delete (self->entrys);
  self->entrys = NULL;
  self->entrys_objectId = NULL;

  AutoRes_ReleaseHandle (self->handle);
  self->handle = AUTORES_NULL_HANDLE;
}

int /* boolean */
AutoRes_Selection_reserve 
(AutoRes_Selection *self,
 int nInitialEntrys)
{
  int flag;

  assert(self->entrys != NULL);
  assert(0 <= nInitialEntrys);
  
  flag = AutoRes_Table_reserveRecords (self->entrys, nInitialEntrys);
  if (!flag) return 0;

  return 1;
}

int AutoRes_Selection_addObject 
(AutoRes_Selection *self,
 int objectId)
{
  int entryId;

  assert(0 <= objectId);
  assert(0 < AutoRes_Table_nReservedRecords (self->entrys));

  entryId = AutoRes_Table_addRecord (self->entrys);
  if (entryId == -1) return -1;

  AUTORES_I_SET(self->entrys_objectId, entryId, objectId);

  return entryId;
}

int /* boolean */
AutoRes_Selection_import 
(AutoRes_Selection *self,
 AutoRes_IntegerColumn *objects_flag)
{
  AutoRes_Table *objects;
  int nObjects;
  int iObject;
  int count;
  int flag;

  objects = AutoRes_IntegerColumn_table (objects_flag);
  nObjects = AutoRes_Table_nRecords (objects);

  count = 0;
  for (iObject = 0; iObject < nObjects; iObject++) {
    if (AUTORES_I(objects_flag, iObject)) {
      count++;
    }
  }

  flag = AutoRes_Selection_reserve (self, count);
  if (!flag) return 0;

  for (iObject = 0; iObject < nObjects; iObject++) {
    if (AUTORES_I(objects_flag, iObject)) {
      int entryId = AutoRes_Selection_addObject (self, iObject);

      if (entryId == -1) return 0;
    }
  }

  return 1;
}

void AutoRes_Selection_export 
(AutoRes_Selection *self,
 AutoRes_IntegerColumn *objects_flag)
{
  AutoRes_Table *objects;
  int nObjects;
  int iObject;
  int nEntrys;
  int iEntry;

  objects = AutoRes_IntegerColumn_table (objects_flag);
  nObjects = AutoRes_Table_nRecords (objects);

  for (iObject = 0; iObject < nObjects; iObject++) {
    AUTORES_I_SET(objects_flag, iObject, 0);
  }

  nEntrys = AutoRes_Selection_nEntrys (self);

  for (iEntry = 0; iEntry < nEntrys; iEntry++) {
    int objectId = AutoRes_Selection_objectId (self, iEntry);

    AUTORES_I_SET(objects_flag, objectId, 1);
  }
}

AutoRes_Table *AutoRes_Selection_entrys 
(AutoRes_Selection *self)
{
  return self->entrys;
}

int AutoRes_Selection_nEntrys 
(AutoRes_Selection *self)
{
  return AutoRes_Table_nRecords (self->entrys);
}

int AutoRes_Selection_objectId 
(AutoRes_Selection *self,
 int entryId)
{
  return AUTORES_I(self->entrys_objectId, entryId);
}



