/* autores_sorter.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_memory.h"
#include "autores_column.h"
#include "autores_integer_column.h"
#include "autores_real_column.h"
#include "autores_sorter.h"



#define TABLE_NAME_ENTRYS "AutoRes:entrys"
#define COLUMN_NAME_OBJECT_ID "AutoRes:objectId"
#define COLUMN_NAME_VALUE "AutoRes:value"



/* Integer Sorter */

AUTORES_IMPLEMENT_HANDLE(AutoRes_IntegerSorter);

AutoRes_IntegerSorter *AutoRes_IntegerSorter_New 
(int nEntrys)
{
  AutoRes_IntegerSorter *sorter;

  assert(0 < nEntrys);

  sorter = AutoRes_IntegerSorter_NewImplementation ();

  sorter->nEntrys = nEntrys;

  sorter->entrys = NULL;
  sorter->entrys_objectId = NULL;
  sorter->entrys_value = NULL;

  return sorter;
}

void AutoRes_IntegerSorter_delete 
(AutoRes_IntegerSorter *self)
{
  if (self->entrys != NULL) {
    AutoRes_Table_delete (self->entrys);
    self->entrys = NULL;
    self->entrys_objectId = NULL;
    self->entrys_value = NULL;
  }

  AutoRes_ReleaseHandle (self->handle);
  self->handle = AUTORES_NULL_HANDLE;
}

int /* boolean */
AutoRes_IntegerSorter_allocate 
(AutoRes_IntegerSorter *self)
{
  int iEntry;
  int flag;

  assert(self->entrys == NULL);

  self->entrys = AutoRes_Table_New (TABLE_NAME_ENTRYS);
  assert(self->entrys != NULL);
  AutoRes_Table_allocateRecords (self->entrys, self->nEntrys);

  self->entrys_objectId = AutoRes_IntegerColumn_New 
    (self->entrys, COLUMN_NAME_OBJECT_ID);
  assert(self->entrys_objectId != NULL);

  flag = AutoRes_Table_addIntegerColumn 
    (self->entrys, self->entrys_objectId);
  if (!flag) return 0;

  for (iEntry = 0; iEntry < self->nEntrys; iEntry++) {
    AUTORES_I_SET(self->entrys_objectId, iEntry, iEntry); 
  }

  self->entrys_value = AutoRes_IntegerColumn_New 
    (self->entrys, COLUMN_NAME_VALUE);
  assert(self->entrys_value != NULL);

  flag = AutoRes_Table_addIntegerColumn 
    (self->entrys, self->entrys_value);
  if (!flag) return 0;

  for (iEntry = 0; iEntry < self->nEntrys; iEntry++) {
    AUTORES_I_SET(self->entrys_value, iEntry, 0); 
  }

  return 1;
}

void AutoRes_IntegerSorter_setValue 
(AutoRes_IntegerSorter *self,
 int entryId,
 int value)
{
  assert(self->entrys != NULL);
  assert(0 <= entryId);
  assert(entryId < self->nEntrys);

  AUTORES_I_SET(self->entrys_value, entryId, value);
}

void AutoRes_IntegerSorter_sort 
(AutoRes_IntegerSorter *self)
{
  assert(self->entrys != NULL);

  /* NEED TO REDEFINE */
  assert(0);
}

int AutoRes_IntegerSorter_nEntrys 
(AutoRes_IntegerSorter *self)
{
  return self->nEntrys;
}

int AutoRes_IntegerSorter_objectId 
(AutoRes_IntegerSorter *self,
 int entryId)
{
  assert(self->entrys != NULL);
  assert(0 <= entryId);
  assert(entryId < self->nEntrys);

  return AUTORES_I(self->entrys_objectId, entryId);
}

int AutoRes_IntegerSorter_value 
(AutoRes_IntegerSorter *self,
 int entryId)
{
  assert(self->entrys != NULL);
  assert(0 <= entryId);
  assert(entryId < self->nEntrys);

  return AUTORES_I(self->entrys_value, entryId);
}



/* Real Sorter */

AUTORES_IMPLEMENT_HANDLE(AutoRes_RealSorter);

AutoRes_RealSorter *AutoRes_RealSorter_New 
(int nEntrys)
{
  AutoRes_RealSorter *sorter;

  assert(0 < nEntrys);

  sorter = AutoRes_RealSorter_NewImplementation ();

  sorter->nEntrys = nEntrys;

  sorter->entrys = NULL;
  sorter->entrys_objectId = NULL;
  sorter->entrys_value = NULL;

  return sorter;
}

void AutoRes_RealSorter_delete 
(AutoRes_RealSorter *self)
{
  if (self->entrys != NULL) {
    AutoRes_Table_delete (self->entrys);
    self->entrys = NULL;
    self->entrys_objectId = NULL;
    self->entrys_value = NULL;
  }

  AutoRes_ReleaseHandle (self->handle);
  self->handle = AUTORES_NULL_HANDLE;
}

int /* boolean */
AutoRes_RealSorter_allocate 
(AutoRes_RealSorter *self)
{
  int iEntry;
  int flag;

  assert(self->entrys == NULL);

  self->entrys = AutoRes_Table_New (TABLE_NAME_ENTRYS);
  assert(self->entrys != NULL);
  AutoRes_Table_allocateRecords (self->entrys, self->nEntrys);

  self->entrys_objectId = AutoRes_IntegerColumn_New 
    (self->entrys, COLUMN_NAME_OBJECT_ID);
  assert(self->entrys_objectId != NULL);

  flag = AutoRes_Table_addIntegerColumn 
    (self->entrys, self->entrys_objectId);
  if (!flag) return 0;

  for (iEntry = 0; iEntry < self->nEntrys; iEntry++) {
    AUTORES_I_SET(self->entrys_objectId, iEntry, iEntry); 
  }

  self->entrys_value = AutoRes_RealColumn_New 
    (self->entrys, COLUMN_NAME_VALUE);
  assert(self->entrys_value != NULL);

  flag = AutoRes_Table_addRealColumn 
    (self->entrys, self->entrys_value);
  if (!flag) return 0;

  for (iEntry = 0; iEntry < self->nEntrys; iEntry++) {
    AUTORES_R_SET(self->entrys_value, iEntry, 0.0); 
  }

  return 1;
}

void AutoRes_RealSorter_setValue 
(AutoRes_RealSorter *self,
 int entryId,
 double value)
{
  assert(self->entrys != NULL);
  assert(0 <= entryId);
  assert(entryId < self->nEntrys);

  AUTORES_R_SET(self->entrys_value, entryId, value);
}

void AutoRes_RealSorter_sort 
(AutoRes_RealSorter *self)
{
  assert(self->entrys != NULL);

  /* NEED TO REDEFINE */
  assert(0);
}

int AutoRes_RealSorter_nEntrys 
(AutoRes_RealSorter *self)
{
  return self->nEntrys;
}

int AutoRes_RealSorter_objectId 
(AutoRes_RealSorter *self,
 int entryId)
{
  assert(self->entrys != NULL);
  assert(0 <= entryId);
  assert(entryId < self->nEntrys);

  return AUTORES_I(self->entrys_objectId, entryId);
}

double AutoRes_RealSorter_value 
(AutoRes_RealSorter *self,
 int entryId)
{
  assert(self->entrys != NULL);
  assert(0 <= entryId);
  assert(entryId < self->nEntrys);

  return AUTORES_R(self->entrys_value, entryId);
}



