/* header file for autores_table_io.c */

#ifndef __AUTORES_TABLE_IO__
#define __AUTORES_TABLE_IO__

/* Table I/O */



/* TableInOut */

#define AUTORES_MAX_TABLE_IN_OUT_COLUMNS 100

typedef struct _AutoRes_TableInOut {
  AutoRes_Handle handle;

  AutoRes_Table *table;

  int nColumns;
  AutoRes_ColumnType columnTypes[AUTORES_MAX_TABLE_IN_OUT_COLUMNS]; 
  void *columns[AUTORES_MAX_TABLE_IN_OUT_COLUMNS];
  AutoRes_FieldType fieldTypes[AUTORES_MAX_TABLE_IN_OUT_COLUMNS]; 
  void *fields[AUTORES_MAX_TABLE_IN_OUT_COLUMNS];

  AutoRes_InOut *inOut;  /* OWN */
} AutoRes_TableInOut;

AUTORES_DECLARE_HANDLE(AutoRes_TableInOut);

AutoRes_TableInOut *AutoRes_TableInOut_New 
(AutoRes_Table *table);

void AutoRes_TableInOut_delete 
(AutoRes_TableInOut *self);

void AutoRes_TableInOut_associateColumnWithField 
(AutoRes_TableInOut *self,
 void *column,
 AutoRes_ColumnType columnType,
 void *field,
 AutoRes_FieldType fieldType);

void AutoRes_TableInOut_associateColumnOfNameWithFieldOfName 
(AutoRes_TableInOut *self,
 const char *columnName,
 const char *fieldName);

void AutoRes_TableInOut_associateIntegerColumn 
(AutoRes_TableInOut *self,
 AutoRes_IntegerColumn *column);

void AutoRes_TableInOut_associateInteger1DColumn 
(AutoRes_TableInOut *self,
 AutoRes_Integer1DColumn *column);

void AutoRes_TableInOut_associateRealColumn 
(AutoRes_TableInOut *self,
 AutoRes_RealColumn *column);

void AutoRes_TableInOut_associateReal1DColumn 
(AutoRes_TableInOut *self,
 AutoRes_Real1DColumn *column);

int /* boolean */
AutoRes_TableInOut_open
(AutoRes_TableInOut *self,
 const char *fileName,
 int writeFlag,
 int binaryFlag);

int /* boolean */
AutoRes_TableInOut_close 
(AutoRes_TableInOut *self);

int /* boolean */
AutoRes_TableInOut_doHeaderSection  
(AutoRes_TableInOut *self);

int /* boolean */
AutoRes_TableInOut_doRecordSection 
(AutoRes_TableInOut *self);

int /* boolean */
AutoRes_TableInOut_do 
(AutoRes_TableInOut *self,
 const char *fileName,
 int writeFlag,
 int binaryFlag);

AutoRes_InOut *AutoRes_TableInOut_inOut 
(AutoRes_TableInOut *self);



/* Table I/O Utility */

void AutoRes_TableInOut_SetBinaryFlag (int flag);

void AutoRes_TableInOut_SetRecordIdFlag (int flag);

void AutoRes_TableInOut_AssociateIntegerColumn 
(AutoRes_IntegerColumn *column);

void AutoRes_TableInOut_AssociateInteger1DColumn 
(AutoRes_Integer1DColumn *column);

void AutoRes_TableInOut_AssociateRealColumn 
(AutoRes_RealColumn *column);

void AutoRes_TableInOut_AssociateReal1DColumn 
(AutoRes_Real1DColumn *column);

int /* boolean */
AutoRes_TableInOut_Read (const char *fileName);

int /* boolean */
AutoRes_TableInOut_Write (const char *fileName);

int /* boolean */
AutoRes_TableInOut_WriteMetaData (const char *fileName);



#endif  /* __AUTORES_TABLE_IO__ */



