/* test_column.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_memory.h"
#include "autores_column.h"
#include "autores_integer_column.h"
#include "autores_real_column.h"
#include "autores_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static void Test_IntegerColumn (void)
{
  AutoRes_Table *table;
  AutoRes_IntegerColumn *column;



  printf (" Test integer column \n"); 



  table = AutoRes_Table_New ("table");

  column = AutoRes_IntegerColumn_New (table, "value");
  AutoRes_Table_addIntegerColumn (table, column);
  
  assert(AutoRes_Column_HandleIsValid 
	 (column->handle, AUTORES_COLUMN_INTEGER));

  assert (table == AutoRes_IntegerColumn_table (column));
  assert (table == AutoRes_Column_table (column, AUTORES_COLUMN_INTEGER));

  {
    char buf[AUTORES_MAX_COLUMN_NAME_LENGTH];

    AutoRes_IntegerColumn_getName (column,
				  buf);
    assert(strcmp (buf, "value") == 0);

    AutoRes_Column_getName (column, AUTORES_COLUMN_INTEGER,
			    buf);
    assert(strcmp (buf, "value") == 0);
  }

  AutoRes_Table_allocateRecords (table, 10);

  AUTORES_I_SET(column, 0, 10);
  assert(AUTORES_I(column, 0) == 10);

  AUTORES_I_SET(column, 9, 19);
  assert(AUTORES_I(column, 9) == 19);

  AutoRes_Table_delete (table);
}

static void Test_Integer1DColumn (void)
{
  AutoRes_Table *table;
  AutoRes_Integer1DColumn *column;
  


  printf (" Test integer1D column \n"); 



  table = AutoRes_Table_New ("table");

  column = AutoRes_Integer1DColumn_New (table, "values", 3);
  AutoRes_Table_addInteger1DColumn (table, column);

  assert(AutoRes_Column_HandleIsValid 
	 (column->handle, AUTORES_COLUMN_INTEGER1D));

  assert (table == AutoRes_Integer1DColumn_table (column));
  assert (table == AutoRes_Column_table (column, AUTORES_COLUMN_INTEGER1D));

  {
    char buf[AUTORES_MAX_COLUMN_NAME_LENGTH];
    
    AutoRes_Integer1DColumn_getName (column,
				    buf);
    assert(strcmp (buf, "values") == 0);

    AutoRes_Column_getName (column, AUTORES_COLUMN_INTEGER1D,
			   buf);
    assert(strcmp (buf, "values") == 0);
  }
  assert(AutoRes_Integer1DColumn_size (column) == 3);
  
  AutoRes_Table_allocateRecords (table, 10);

  AUTORES_I1D_SET(column, 0, 0, 10);
  AUTORES_I1D_SET(column, 0, 1, 11);
  AUTORES_I1D_SET(column, 0, 2, 12);
  assert(AUTORES_I1D(column, 0, 0) == 10);
  assert(AUTORES_I1D(column, 0, 1) == 11);
  assert(AUTORES_I1D(column, 0, 2) == 12);

  AUTORES_I1D_SET(column, 9, 0, 100);
  AUTORES_I1D_SET(column, 9, 1, 101);
  AUTORES_I1D_SET(column, 9, 2, 102);
  assert(AUTORES_I1D(column, 9, 0) == 100);
  assert(AUTORES_I1D(column, 9, 1) == 101);
  assert(AUTORES_I1D(column, 9, 2) == 102);

  AutoRes_Table_delete (table);
}

static void Test_RealColumn (void)
{
  AutoRes_Table *table;
  AutoRes_RealColumn *column;



  printf (" Test real column \n"); 



  table = AutoRes_Table_New ("table");

  column = AutoRes_RealColumn_New (table, "value");
  AutoRes_Table_addRealColumn (table, column);
  
  assert(AutoRes_Column_HandleIsValid 
	 (column->handle, AUTORES_COLUMN_REAL));

  assert (table == AutoRes_RealColumn_table (column));
  assert (table == AutoRes_Column_table (column, AUTORES_COLUMN_REAL));

  {
    char buf[AUTORES_MAX_COLUMN_NAME_LENGTH];

    AutoRes_RealColumn_getName (column,
			       buf);
    assert(strcmp (buf, "value") == 0);

    AutoRes_Column_getName (column, AUTORES_COLUMN_REAL,
			   buf);
    assert(strcmp (buf, "value") == 0);
  }

  AutoRes_Table_allocateRecords (table, 10);

  AUTORES_R_SET(column, 0, 10.0);
  assert(EQ(AUTORES_R(column, 0), 10.0));

  AUTORES_R_SET(column, 9, 19.0);
  assert(EQ(AUTORES_R(column, 9), 19.0));

  AutoRes_Table_delete (table);
}

static void Test_Real1DColumn (void)
{
  AutoRes_Table *table;
  AutoRes_Real1DColumn *column;



  printf (" Test real1D column \n"); 



  table = AutoRes_Table_New ("table");

  column = AutoRes_Real1DColumn_New (table, "values", 3);
  AutoRes_Table_addReal1DColumn (table, column);
  
  assert(AutoRes_Column_HandleIsValid 
	 (column->handle, AUTORES_COLUMN_REAL1D));

  assert (table == AutoRes_Real1DColumn_table (column));
  assert (table == AutoRes_Column_table (column, AUTORES_COLUMN_REAL1D));

  {
    char buf[AUTORES_MAX_COLUMN_NAME_LENGTH];

    AutoRes_Real1DColumn_getName (column,
				 buf);
    assert(strcmp (buf, "values") == 0);

    AutoRes_Column_getName (column, AUTORES_COLUMN_REAL1D,
			   buf);
    assert(strcmp (buf, "values") == 0);
  }
  assert(AutoRes_Real1DColumn_size (column) == 3);

  AutoRes_Table_allocateRecords (table, 10);

  AUTORES_R1D_SET(column, 0, 0, 10.0);
  AUTORES_R1D_SET(column, 0, 1, 11.0);
  AUTORES_R1D_SET(column, 0, 2, 12.0);
  assert(EQ(AUTORES_R1D(column, 0, 0), 10.0));
  assert(EQ(AUTORES_R1D(column, 0, 1), 11.0));
  assert(EQ(AUTORES_R1D(column, 0, 2), 12.0));

  AUTORES_R1D_SET(column, 9, 0, 100.0);
  AUTORES_R1D_SET(column, 9, 1, 101.0);
  AUTORES_R1D_SET(column, 9, 2, 102.0);
  assert(EQ(AUTORES_R1D(column, 9, 0), 100.0));
  assert(EQ(AUTORES_R1D(column, 9, 1), 101.0));
  assert(EQ(AUTORES_R1D(column, 9, 2), 102.0));

  AutoRes_Table_delete (table);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);

  Test_IntegerColumn ();
  Test_Integer1DColumn ();
  Test_RealColumn ();
  Test_Real1DColumn ();

  return 0;
}



