/* test_field.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_field.h"
#include "autores_integer_field.h"
#include "autores_real_field.h"
#include "autores_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static void Test_IntegerField (void)
{
  AutoRes_IntegerField *field;
  int value;



  printf (" Test integer field \n"); 



  field = AutoRes_IntegerField_New ("value");
  
  assert(AutoRes_Field_HandleIsValid 
	 (field->handle, AUTORES_FIELD_INTEGER));

  {
    char buf[AUTORES_MAX_FIELD_NAME_LENGTH];

    AutoRes_IntegerField_getName (field,
				  buf);
    assert(strcmp (buf, "value") == 0);

    AutoRes_Field_getName (field, AUTORES_FIELD_INTEGER,
			   buf);
    assert(strcmp (buf, "value") == 0);
  }

  AutoRes_IntegerField_setValue (field, 10);
  assert(AutoRes_IntegerField_value (field) == 10);

  AutoRes_IntegerField_associate (field, &value);
  value = 20;
  assert(AutoRes_IntegerField_value (field) == 20);

  AutoRes_IntegerField_setValue (field, 30);
  assert(value == 30);

  AutoRes_IntegerField_associate (field, NULL);
  assert(AutoRes_IntegerField_value (field) == 10);

  AutoRes_IntegerField_delete (field);
}

static void Test_Integer1DField (void)
{
  AutoRes_Integer1DField *field;
  int values[3];
  


  printf (" Test integer1D field \n"); 



  field = AutoRes_Integer1DField_New ("values", 3);
  
  assert(AutoRes_Field_HandleIsValid 
	 (field->handle, AUTORES_FIELD_INTEGER1D));

  {
    char buf[AUTORES_MAX_FIELD_NAME_LENGTH];
    
    AutoRes_Integer1DField_getName (field,
				    buf);
    assert(strcmp (buf, "values") == 0);

    AutoRes_Field_getName (field, AUTORES_FIELD_INTEGER1D,
			   buf);
    assert(strcmp (buf, "values") == 0);
  }
  assert(AutoRes_Integer1DField_size (field) == 3);
  
  AutoRes_Integer1DField_setValue (field, 0, 10);
  AutoRes_Integer1DField_setValue (field, 1, 11);
  AutoRes_Integer1DField_setValue (field, 2, 12);
  assert(AutoRes_Integer1DField_value (field, 0) == 10);
  assert(AutoRes_Integer1DField_value (field, 1) == 11);
  assert(AutoRes_Integer1DField_value (field, 2) == 12);
  
  AutoRes_Integer1DField_associate (field, values);
  values[0] = 20;
  values[1] = 21;
  values[2] = 22;
  assert(AutoRes_Integer1DField_value (field, 0) == 20);
  assert(AutoRes_Integer1DField_value (field, 1) == 21);
  assert(AutoRes_Integer1DField_value (field, 2) == 22);
  
  AutoRes_Integer1DField_setValue (field, 0, 30);
  AutoRes_Integer1DField_setValue (field, 1, 31);
  AutoRes_Integer1DField_setValue (field, 2, 32);
  assert(values[0] == 30);
  assert(values[1] == 31);
  assert(values[2] == 32);

  AutoRes_Integer1DField_associate (field, NULL);
  assert(AutoRes_Integer1DField_value (field, 0) == 10);
  assert(AutoRes_Integer1DField_value (field, 1) == 11);
  assert(AutoRes_Integer1DField_value (field, 2) == 12);

  AutoRes_Integer1DField_delete (field);
}

static void Test_RealField (void)
{
  AutoRes_RealField *field;
  double value;



  printf (" Test real field \n"); 



  field = AutoRes_RealField_New ("value");
  
  assert(AutoRes_Field_HandleIsValid 
	 (field->handle, AUTORES_FIELD_REAL));

  {
    char buf[AUTORES_MAX_FIELD_NAME_LENGTH];

    AutoRes_RealField_getName (field,
			       buf);
    assert(strcmp (buf, "value") == 0);

    AutoRes_Field_getName (field, AUTORES_FIELD_REAL,
			   buf);
    assert(strcmp (buf, "value") == 0);
  }

  AutoRes_RealField_setValue (field, 10.0);
  assert(EQ(AutoRes_RealField_value (field), 10.0));

  AutoRes_RealField_associate (field, &value);
  value = 20.0;
  assert(EQ(AutoRes_RealField_value (field), 20.0));

  AutoRes_RealField_setValue (field, 30.0);
  assert(EQ(value, 30));

  AutoRes_RealField_associate (field, NULL);
  assert(EQ(AutoRes_RealField_value (field), 10.0));

  AutoRes_RealField_delete (field);
}

static void Test_Real1DField (void)
{
  AutoRes_Real1DField *field;
  double values[3];



  printf (" Test real1D field \n"); 



  field = AutoRes_Real1DField_New ("values", 3);
  
  assert(AutoRes_Field_HandleIsValid 
	 (field->handle, AUTORES_FIELD_REAL1D));

  {
    char buf[AUTORES_MAX_FIELD_NAME_LENGTH];

    AutoRes_Real1DField_getName (field,
				 buf);
    assert(strcmp (buf, "values") == 0);

    AutoRes_Field_getName (field, AUTORES_FIELD_REAL1D,
			   buf);
    assert(strcmp (buf, "values") == 0);
  }
  assert(AutoRes_Real1DField_size (field) == 3);

  AutoRes_Real1DField_setValue (field, 0, 10.0);
  AutoRes_Real1DField_setValue (field, 1, 11.0);
  AutoRes_Real1DField_setValue (field, 2, 12.0);
  assert(EQ(AutoRes_Real1DField_value (field, 0), 10.0));
  assert(EQ(AutoRes_Real1DField_value (field, 1), 11.0));
  assert(EQ(AutoRes_Real1DField_value (field, 2), 12.0));
  
  AutoRes_Real1DField_associate (field, values);
  values[0] = 20.0;
  values[1] = 21.0;
  values[2] = 22.0;
  assert(EQ(AutoRes_Real1DField_value (field, 0), 20.0));
  assert(EQ(AutoRes_Real1DField_value (field, 1), 21.0));
  assert(EQ(AutoRes_Real1DField_value (field, 2), 22.0));

  AutoRes_Real1DField_setValue (field, 0, 30.0);
  AutoRes_Real1DField_setValue (field, 1, 31.0);
  AutoRes_Real1DField_setValue (field, 2, 32.0);
  assert(EQ(values[0], 30));
  assert(EQ(values[1], 31));
  assert(EQ(values[2], 32));

  AutoRes_Real1DField_associate (field, NULL);
  assert(EQ(AutoRes_Real1DField_value (field, 0), 10.0));
  assert(EQ(AutoRes_Real1DField_value (field, 1), 11.0));
  assert(EQ(AutoRes_Real1DField_value (field, 2), 12.0));

  AutoRes_Real1DField_delete (field);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);

  Test_IntegerField ();
  Test_Integer1DField ();
  Test_RealField ();
  Test_Real1DField ();

  return 0;
}



