/* test_file.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_file.h"
#include "autores_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static void Test0 (void)
{
  AutoRes_File *file;
  int writeFlag;
  int binaryFlag = 0;
  int integerValue0;
  int integerValue1;
  double realValue0;
  double realValue1;
  int flag;
  char *fileName = "test_file_test0.txt";



  printf (" Test0 \n"); 



  integerValue0 = 10;
  integerValue1 = 20;
  realValue0 = 30.0;
  realValue1 = 40.0;



  file = AutoRes_File_New ();
  assert(file != NULL);



  printf (" write text \n"); 



  writeFlag = 1;
  flag = AutoRes_File_open 
    (file, fileName, writeFlag, binaryFlag);
  assert(flag);
  
  assert(AutoRes_File_writeFlag (file));
  assert(!AutoRes_File_binaryFlag (file));

  flag = AutoRes_File_integer (file, &integerValue0);
  assert(flag);
  
  flag = AutoRes_File_integer (file, &integerValue1);
  assert(flag);

  flag = AutoRes_File_nextLine (file);
  assert(flag);

  flag = AutoRes_File_real (file, &realValue0);
  assert(flag);

  flag = AutoRes_File_real (file, &realValue1);
  assert(flag);

  flag = AutoRes_File_nextLine (file);
  assert(flag);

  flag = AutoRes_File_close (file);
  assert(flag);



  printf (" read text \n"); 



  integerValue0 = 0;
  integerValue1 = 0;
  realValue0 = 0.0;
  realValue1 = 0.0;

  writeFlag = 0;
  flag = AutoRes_File_open 
    (file, fileName, writeFlag, binaryFlag);
  assert(flag);
  
  assert(!AutoRes_File_writeFlag (file));
  assert(!AutoRes_File_binaryFlag (file));

  flag = AutoRes_File_integer (file, &integerValue0);
  assert(flag);
  assert(integerValue0 == 10);

  flag = AutoRes_File_integer (file, &integerValue1);
  assert(flag);
  assert(integerValue1 == 20);

  flag = AutoRes_File_nextLine (file);
  assert(flag);

  flag = AutoRes_File_real (file, &realValue0);
  assert(flag);
  assert(EQ(realValue0, 30.0));

  flag = AutoRes_File_real (file, &realValue1);
  assert(flag);
  assert(EQ(realValue1, 40.0));

  flag = AutoRes_File_nextLine (file);
  assert(flag);

  flag = AutoRes_File_close (file);
  assert(flag);



  AutoRes_File_delete (file);
}

static void Test1 (void)
{
  AutoRes_File *file;
  int writeFlag;
  int binaryFlag = 1;
  int integerValue0;
  int integerValue1;
  double realValue0;
  double realValue1;
  int flag;
  char *fileName = "test_file_test1.bin";



  printf (" Test1 \n"); 



  integerValue0 = 10;
  integerValue1 = 20;
  realValue0 = 30.0;
  realValue1 = 40.0;



  file = AutoRes_File_New ();
  assert(file != NULL);



  printf (" write binary \n"); 



  writeFlag = 1;
  flag = AutoRes_File_open 
    (file, fileName, writeFlag, binaryFlag);
  assert(flag);
  
  assert(AutoRes_File_writeFlag (file));
  assert(AutoRes_File_binaryFlag (file));

  flag = AutoRes_File_integer (file, &integerValue0);
  assert(flag);
  
  flag = AutoRes_File_integer (file, &integerValue1);
  assert(flag);

  flag = AutoRes_File_nextLine (file);
  assert(flag);

  flag = AutoRes_File_real (file, &realValue0);
  assert(flag);

  flag = AutoRes_File_real (file, &realValue1);
  assert(flag);

  flag = AutoRes_File_nextLine (file);
  assert(flag);

  flag = AutoRes_File_close (file);
  assert(flag);



  printf (" read binary \n"); 



  integerValue0 = 0;
  integerValue1 = 0;
  realValue0 = 0.0;
  realValue1 = 0.0;

  writeFlag = 0;
  flag = AutoRes_File_open 
    (file, fileName, writeFlag, binaryFlag);
  assert(flag);
  
  assert(!AutoRes_File_writeFlag (file));
  assert(AutoRes_File_binaryFlag (file));

  flag = AutoRes_File_integer (file, &integerValue0);
  assert(flag);
  assert(integerValue0 == 10);

  flag = AutoRes_File_integer (file, &integerValue1);
  assert(flag);
  assert(integerValue1 == 20);

  flag = AutoRes_File_nextLine (file);
  assert(flag);

  flag = AutoRes_File_real (file, &realValue0);
  assert(flag);
  assert(EQ(realValue0, 30.0));
  
  flag = AutoRes_File_real (file, &realValue1);
  assert(flag);
  assert(EQ(realValue1, 40.0));

  flag = AutoRes_File_nextLine (file);
  assert(flag);

  flag = AutoRes_File_close (file);
  assert(flag);



  AutoRes_File_delete (file);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);

  Test0 ();
  Test1 ();

  return 0;
}



