/* test_free_list.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_memory.h"
#include "autores_column.h"
#include "autores_integer_column.h"
#include "autores_real_column.h"
#include "autores_free_list.h"
#include "autores_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static void Test0 (void)
{
  AutoRes_Table *table;
  AutoRes_RealColumn *valueColumn;
  AutoRes_FreeListCollection *collection;
  int nRecords;
  int nReservedRecords;
  int i;
  int flag;



  printf (" Test0 \n"); 



  table = AutoRes_Table_New ("table");

  valueColumn = AutoRes_RealColumn_New (table, "value");
  AutoRes_Table_addRealColumn (table, valueColumn);

  collection = AutoRes_FreeListCollection_New (table);
  assert(AutoRes_FreeListCollection_table (collection) == table);

  flag = AutoRes_FreeListCollection_makeColumn (collection);
  assert(flag);
  assert(AutoRes_Table_nColumns (table) == 2); 
  


  flag = AutoRes_Table_reserveRecords (table, 3);
  assert(flag);

  for (i = 0; i < 5; i++) {
    int newRecordId = AutoRes_FreeListCollection_addRecord (collection);

    assert(newRecordId != -1);
    AUTORES_R_SET(valueColumn, newRecordId, i * 10.0);
  }
  
  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 5);
  assert(nRecords <= nReservedRecords);

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 0));
  assert(EQ(AUTORES_R(valueColumn, 0), 0.0));
  
  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 1));
  assert(EQ(AUTORES_R(valueColumn, 1), 10.0));
  
  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 2));
  assert(EQ(AUTORES_R(valueColumn, 2), 20.0));

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 3));
  assert(EQ(AUTORES_R(valueColumn, 3), 30.0));

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 4));
  assert(EQ(AUTORES_R(valueColumn, 4), 40.0));



  AutoRes_FreeListCollection_removeRecord (collection, 1);
  AutoRes_FreeListCollection_removeRecord (collection, 2);
  AutoRes_FreeListCollection_removeRecord (collection, 3);

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 5);
  assert(nRecords <= nReservedRecords);

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 0));
  assert(EQ(AUTORES_R(valueColumn, 0), 0.0));
  
  assert(!AutoRes_FreeListCollection_recordIsUsed (collection, 1));
  
  assert(!AutoRes_FreeListCollection_recordIsUsed (collection, 2));

  assert(!AutoRes_FreeListCollection_recordIsUsed (collection, 3));

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 4));
  assert(EQ(AUTORES_R(valueColumn, 4), 40.0));



  for (i = 5; i < 7; i++) {
    int newRecordId = AutoRes_FreeListCollection_addRecord (collection);

    assert(newRecordId != -1);
    AUTORES_R_SET(valueColumn, newRecordId, i * 10.0);
  }

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 5);
  assert(nRecords <= nReservedRecords);

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 0));
  assert(EQ(AUTORES_R(valueColumn, 0), 0.0));
  
  assert(!AutoRes_FreeListCollection_recordIsUsed (collection, 1));
  
  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 2));
  assert(EQ(AUTORES_R(valueColumn, 2), 60.0));

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 3));
  assert(EQ(AUTORES_R(valueColumn, 3), 50.0));

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 4));
  assert(EQ(AUTORES_R(valueColumn, 4), 40.0));



  AutoRes_FreeListCollection_pack (collection);

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 4);
  assert(nRecords <= nReservedRecords);

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 0));
  assert(EQ(AUTORES_R(valueColumn, 0), 0.0));
  
  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 1));
  assert(EQ(AUTORES_R(valueColumn, 1), 60.0));

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 2));
  assert(EQ(AUTORES_R(valueColumn, 2), 50.0));

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 3));
  assert(EQ(AUTORES_R(valueColumn, 3), 40.0));



  AutoRes_FreeListCollection_removeRecord (collection, 0);
  AutoRes_FreeListCollection_removeRecord (collection, 2);

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 4);
  assert(nRecords <= nReservedRecords);

  assert(!AutoRes_FreeListCollection_recordIsUsed (collection, 0));
  
  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 1));
  assert(EQ(AUTORES_R(valueColumn, 1), 60.0));

  assert(!AutoRes_FreeListCollection_recordIsUsed (collection, 2));

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 3));
  assert(EQ(AUTORES_R(valueColumn, 3), 40.0));



  for (i = 7; i < 12; i++) {
    int newRecordId = AutoRes_FreeListCollection_addRecord (collection);

    assert(newRecordId != -1);
    AUTORES_R_SET(valueColumn, newRecordId, i * 10.0);
  }

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 7);
  assert(nRecords <= nReservedRecords);

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 0));
  assert(EQ(AUTORES_R(valueColumn, 0), 80.0));
  
  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 1));
  assert(EQ(AUTORES_R(valueColumn, 1), 60.0));

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 2));
  assert(EQ(AUTORES_R(valueColumn, 2), 70.0));

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 3));
  assert(EQ(AUTORES_R(valueColumn, 3), 40.0));

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 4));
  assert(EQ(AUTORES_R(valueColumn, 4), 90.0));

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 5));
  assert(EQ(AUTORES_R(valueColumn, 5), 100.0));

  assert(AutoRes_FreeListCollection_recordIsUsed (collection, 6));
  assert(EQ(AUTORES_R(valueColumn, 6), 110.0));



  AutoRes_FreeListCollection_delete (collection);
  AutoRes_Table_delete (table);
}



int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);

  Test0 ();

  return 0;
}

