/* test_io.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_field.h"
#include "autores_integer_field.h"
#include "autores_real_field.h"
#include "autores_io.h"
#include "autores_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

#define OLD_META_DATA_FILE_NAME "test_io_test0.cdm"
#define NEW_META_DATA_FILE_NAME "test_io_test1.cdm"
#define OLD_FILE_NAME "test_io_test0.txt"
#define NEW_FILE_NAME "test_io_test1.txt"

static void Test0 (void)
{
  AutoRes_InOut *inOut;
  int writeFlag;
  int binaryFlag = 0;

  AutoRes_IntegerField *hvf0;
  AutoRes_RealField *hvf1;
  AutoRes_Integer1DField *hvf2;
  AutoRes_Real1DField *hvf3;
  AutoRes_IntegerField *rvf0;
  AutoRes_RealField *rvf1;
  AutoRes_Integer1DField *rvf2;
  AutoRes_Real1DField *rvf3;

  int nRecords;
  int iRecord;
  int flag;
  


  printf (" Test0 \n"); 



  inOut = AutoRes_InOut_New ();
  assert(AutoRes_InOut_file (inOut) == NULL);
  assert(AutoRes_InOut_nRecords (inOut) == 0);



  /* define header section */

  hvf0 = AutoRes_IntegerField_New ("hvf0");
  AutoRes_InOut_addIntegerFieldIntoHeader (inOut, hvf0);

  hvf1 = AutoRes_RealField_New ("hvf1");
  AutoRes_InOut_addRealFieldIntoHeader (inOut, hvf1);

  hvf2 = AutoRes_Integer1DField_New ("hvf2", 2);
  AutoRes_InOut_addInteger1DFieldIntoHeader (inOut, hvf2);
  
  hvf3 = AutoRes_Real1DField_New ("hvf3", 3);
  AutoRes_InOut_addReal1DFieldIntoHeader (inOut, hvf3);

  assert(AutoRes_InOut_nHeaderFields (inOut) == 4);

  assert(AutoRes_InOut_headerField (inOut, 0) == hvf0);
  assert(AutoRes_InOut_headerField (inOut, 1) == hvf1);
  assert(AutoRes_InOut_headerField (inOut, 2) == hvf2);
  assert(AutoRes_InOut_headerField (inOut, 3) == hvf3);

  assert(AutoRes_InOut_headerFieldType (inOut, 0) == AUTORES_FIELD_INTEGER);
  assert(AutoRes_InOut_headerFieldType (inOut, 1) == AUTORES_FIELD_REAL);
  assert(AutoRes_InOut_headerFieldType (inOut, 2) == AUTORES_FIELD_INTEGER1D);
  assert(AutoRes_InOut_headerFieldType (inOut, 3) == AUTORES_FIELD_REAL1D);

  assert(AutoRes_InOut_headerFieldIdOfName (inOut, "hvf0") == 0);
  assert(AutoRes_InOut_headerFieldOfName (inOut, "hvf3") == hvf3);



  /* define record section */

  rvf0 = AutoRes_IntegerField_New ("rvf0");
  AutoRes_InOut_addIntegerFieldIntoRecord (inOut, rvf0);

  rvf1 = AutoRes_RealField_New ("rvf1");
  AutoRes_InOut_addRealFieldIntoRecord (inOut, rvf1);

  rvf2 = AutoRes_Integer1DField_New ("rvf2", 2);
  AutoRes_InOut_addInteger1DFieldIntoRecord (inOut, rvf2);
  
  rvf3 = AutoRes_Real1DField_New ("rvf3", 3);
  AutoRes_InOut_addReal1DFieldIntoRecord (inOut, rvf3);

  assert(AutoRes_InOut_nRecordFields (inOut) == 4);

  assert(AutoRes_InOut_recordField (inOut, 0) == rvf0);
  assert(AutoRes_InOut_recordField (inOut, 1) == rvf1);
  assert(AutoRes_InOut_recordField (inOut, 2) == rvf2);
  assert(AutoRes_InOut_recordField (inOut, 3) == rvf3);

  assert(AutoRes_InOut_recordFieldType (inOut, 0) == AUTORES_FIELD_INTEGER);
  assert(AutoRes_InOut_recordFieldType (inOut, 1) == AUTORES_FIELD_REAL);
  assert(AutoRes_InOut_recordFieldType (inOut, 2) == AUTORES_FIELD_INTEGER1D);
  assert(AutoRes_InOut_recordFieldType (inOut, 3) == AUTORES_FIELD_REAL1D);

  assert(AutoRes_InOut_recordFieldIdOfName (inOut, "rvf2") == 2);
  assert(AutoRes_InOut_recordFieldOfName (inOut, "rvf1") == rvf1);



  printf (" write meta-data \n"); 



  flag = AutoRes_InOut_writeMetaData (inOut, OLD_META_DATA_FILE_NAME);
  assert(flag);

  

  printf (" write file \n"); 



  nRecords = 4;
  
  writeFlag = 1;
  flag = AutoRes_InOut_open (inOut, OLD_FILE_NAME, writeFlag, binaryFlag);
  assert(flag);
  assert(AutoRes_InOut_file (inOut) != NULL);

  AutoRes_InOut_setNRecords (inOut, nRecords);
  assert(AutoRes_InOut_nRecords (inOut) == nRecords);

  AutoRes_IntegerField_setValue (hvf0, 1);
  AutoRes_RealField_setValue (hvf1, 2.0);
  AutoRes_Integer1DField_setValue (hvf2, 0, 3);
  AutoRes_Integer1DField_setValue (hvf2, 1, 4);
  AutoRes_Real1DField_setValue (hvf3, 0, 5.0);
  AutoRes_Real1DField_setValue (hvf3, 1, 6.0);
  AutoRes_Real1DField_setValue (hvf3, 2, 7.0);

  flag = AutoRes_InOut_doHeaderSection (inOut);
  assert(flag);
  
  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;

    AutoRes_IntegerField_setValue (rvf0, base + 1);
    AutoRes_RealField_setValue (rvf1, base + 2.0);
    AutoRes_Integer1DField_setValue (rvf2, 0, base + 3);
    AutoRes_Integer1DField_setValue (rvf2, 1, base + 4);
    AutoRes_Real1DField_setValue (rvf3, 0, base + 5.0);
    AutoRes_Real1DField_setValue (rvf3, 1, base + 6.0);
    AutoRes_Real1DField_setValue (rvf3, 2, base + 7.0);
    
    flag = AutoRes_InOut_doEachOfRecordSection (inOut, iRecord);
    assert(flag);
  }

  flag = AutoRes_InOut_close (inOut);
  assert(flag);
  assert(AutoRes_InOut_file (inOut) == NULL);



  printf (" read file \n"); 



  writeFlag = 0;
  flag = AutoRes_InOut_open (inOut, OLD_FILE_NAME, writeFlag, binaryFlag);
  assert(flag);
  assert(AutoRes_InOut_file (inOut) != NULL);

  flag = AutoRes_InOut_doHeaderSection (inOut);
  assert(flag);

  nRecords = AutoRes_InOut_nRecords (inOut);
  assert(nRecords == 4);

  assert(AutoRes_IntegerField_value (hvf0) == 1);
  assert(EQ(AutoRes_RealField_value (hvf1), 2.0));
  assert(AutoRes_Integer1DField_value (hvf2, 0) == 3);
  assert(AutoRes_Integer1DField_value (hvf2, 1) == 4);
  assert(EQ(AutoRes_Real1DField_value (hvf3, 0), 5.0));
  assert(EQ(AutoRes_Real1DField_value (hvf3, 1), 6.0));
  assert(EQ(AutoRes_Real1DField_value (hvf3, 2), 7.0));
  
  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;
    
    flag = AutoRes_InOut_doEachOfRecordSection (inOut, iRecord);
    assert(flag);

    assert(AutoRes_IntegerField_value (rvf0) == base + 1);
    assert(EQ(AutoRes_RealField_value (rvf1), base + 2.0));
    assert(AutoRes_Integer1DField_value (rvf2, 0) == base + 3);
    assert(AutoRes_Integer1DField_value (rvf2, 1) == base + 4);
    assert(EQ(AutoRes_Real1DField_value (rvf3, 0), base + 5.0));
    assert(EQ(AutoRes_Real1DField_value (rvf3, 1), base + 6.0));
    assert(EQ(AutoRes_Real1DField_value (rvf3, 2), base + 7.0));
  }

  flag = AutoRes_InOut_close (inOut);
  assert(flag);
  assert(AutoRes_InOut_file (inOut) == NULL);



  AutoRes_InOut_delete (inOut);
}

static void Test1 (void)
{
  AutoRes_InOut *inOut;
  int writeFlag;
  int binaryFlag = 0;

  AutoRes_IntegerField *hvf0;
  AutoRes_RealField *hvf1;
  AutoRes_Integer1DField *hvf2;
  AutoRes_Real1DField *hvf3;
  AutoRes_IntegerField *rvf0;
  AutoRes_RealField *rvf1;
  AutoRes_Integer1DField *rvf2;
  AutoRes_Real1DField *rvf3;

  int hval0;
  double hval1;
  int hval2[2];
  double hval3[3];

  static int rval0[100];
  static double rval1[100];
  static int rval2[100][2];
  static double rval3[100][3];

  int nRecords;
  int iRecord;
  int flag;
  


  printf (" Test1 \n"); 



  inOut = AutoRes_InOut_New ();
  assert(AutoRes_InOut_file (inOut) == NULL);
  assert(AutoRes_InOut_nRecords (inOut) == 0);

  /* read meta-data file */
  flag = AutoRes_InOut_readMetaData (inOut, OLD_META_DATA_FILE_NAME);
  assert(flag);

  

  /* find header section */

  assert(AutoRes_InOut_nHeaderFields (inOut) == 4);

  assert(AutoRes_InOut_headerFieldType (inOut, 0) == AUTORES_FIELD_INTEGER);
  assert(AutoRes_InOut_headerFieldType (inOut, 1) == AUTORES_FIELD_REAL);
  assert(AutoRes_InOut_headerFieldType (inOut, 2) == AUTORES_FIELD_INTEGER1D);
  assert(AutoRes_InOut_headerFieldType (inOut, 3) == AUTORES_FIELD_REAL1D);

  hvf0 = (AutoRes_IntegerField *)
    AutoRes_InOut_headerFieldOfName (inOut, "hvf0");
  assert(hvf0 != NULL);

  hvf1 = (AutoRes_RealField *)
    AutoRes_InOut_headerFieldOfName (inOut, "hvf1");
  assert(hvf1 != NULL);

  hvf2 = (AutoRes_Integer1DField *)
    AutoRes_InOut_headerFieldOfName (inOut, "hvf2");
  assert(hvf2 != NULL);
  assert(AutoRes_Integer1DField_size (hvf2) == 2);

  hvf3 = (AutoRes_Real1DField *)
    AutoRes_InOut_headerFieldOfName (inOut, "hvf3");
  assert(hvf3 != NULL);
  assert(AutoRes_Real1DField_size (hvf3) == 3);

  assert(AutoRes_InOut_headerField (inOut, 0) == hvf0);
  assert(AutoRes_InOut_headerField (inOut, 1) == hvf1);
  assert(AutoRes_InOut_headerField (inOut, 2) == hvf2);
  assert(AutoRes_InOut_headerField (inOut, 3) == hvf3);



  /* find record section */

  assert(AutoRes_InOut_nRecordFields (inOut) == 4);

  assert(AutoRes_InOut_recordFieldType (inOut, 0) == AUTORES_FIELD_INTEGER);
  assert(AutoRes_InOut_recordFieldType (inOut, 1) == AUTORES_FIELD_REAL);
  assert(AutoRes_InOut_recordFieldType (inOut, 2) == AUTORES_FIELD_INTEGER1D);
  assert(AutoRes_InOut_recordFieldType (inOut, 3) == AUTORES_FIELD_REAL1D);

  rvf0 = (AutoRes_IntegerField *)
    AutoRes_InOut_recordFieldOfName (inOut, "rvf0");
  assert(rvf0 != NULL);

  rvf1 = (AutoRes_RealField *)
    AutoRes_InOut_recordFieldOfName (inOut, "rvf1");
  assert(rvf1 != NULL);

  rvf2 = (AutoRes_Integer1DField *)
    AutoRes_InOut_recordFieldOfName (inOut, "rvf2");
  assert(rvf2 != NULL);
  assert(AutoRes_Integer1DField_size (rvf2) == 2);

  rvf3 = (AutoRes_Real1DField *)
    AutoRes_InOut_recordFieldOfName (inOut, "rvf3");
  assert(rvf3 != NULL);
  assert(AutoRes_Real1DField_size (rvf3) == 3);

  assert(AutoRes_InOut_recordField (inOut, 0) == rvf0);
  assert(AutoRes_InOut_recordField (inOut, 1) == rvf1);
  assert(AutoRes_InOut_recordField (inOut, 2) == rvf2);
  assert(AutoRes_InOut_recordField (inOut, 3) == rvf3);



  printf (" write meta-data \n"); 



  flag = AutoRes_InOut_writeMetaData (inOut, NEW_META_DATA_FILE_NAME);
  assert(flag);



  printf (" read file \n"); 



  nRecords = 0;

  writeFlag = 0;
  AutoRes_InOut_open (inOut, OLD_FILE_NAME, writeFlag, binaryFlag);
  assert(AutoRes_InOut_file (inOut) != NULL);

  AutoRes_InOut_associateNRecords (inOut, &nRecords);

  AutoRes_IntegerField_associate (hvf0, &hval0);
  AutoRes_RealField_associate (hvf1, &hval1);
  AutoRes_Integer1DField_associate (hvf2, hval2);
  AutoRes_Real1DField_associate (hvf3, hval3);

  AutoRes_InOut_doHeaderSection (inOut);

  assert(nRecords == 4);
  assert(AutoRes_InOut_nRecords (inOut) == 4);

  assert(hval0 == 1);
  assert(EQ(hval1, 2.0));
  assert(hval2[0] == 3);
  assert(hval2[1] == 4);
  assert(EQ(hval3[0], 5.0));
  assert(EQ(hval3[1], 6.0));
  assert(EQ(hval3[2], 7.0));
  
  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;
    
    AutoRes_IntegerField_associate (rvf0, &rval0[iRecord]);
    AutoRes_RealField_associate (rvf1, &rval1[iRecord]);
    AutoRes_Integer1DField_associate (rvf2, rval2[iRecord]);
    AutoRes_Real1DField_associate (rvf3, rval3[iRecord]);

    AutoRes_InOut_doEachOfRecordSection (inOut, iRecord);

    assert(rval0[iRecord] == base + 1);
    assert(EQ(rval1[iRecord], base + 2.0));
    assert(rval2[iRecord][0] == base + 3);
    assert(rval2[iRecord][1] == base + 4);
    assert(EQ(rval3[iRecord][0], base + 5.0));
    assert(EQ(rval3[iRecord][1], base + 6.0));
    assert(EQ(rval3[iRecord][2], base + 7.0));
  }

  AutoRes_InOut_close (inOut);
  assert(AutoRes_InOut_file (inOut) == NULL);



  printf (" write file \n"); 



  writeFlag = 1;
  AutoRes_InOut_open (inOut, NEW_FILE_NAME, writeFlag, binaryFlag);
  assert(AutoRes_InOut_file (inOut) != NULL);

  AutoRes_InOut_associateNRecords (inOut, &nRecords);

  AutoRes_IntegerField_associate (hvf0, &hval0);
  AutoRes_RealField_associate (hvf1, &hval1);
  AutoRes_Integer1DField_associate (hvf2, hval2);
  AutoRes_Real1DField_associate (hvf3, hval3);

  AutoRes_InOut_doHeaderSection (inOut);
  
  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    AutoRes_IntegerField_associate (rvf0, &rval0[iRecord]);
    AutoRes_RealField_associate (rvf1, &rval1[iRecord]);
    AutoRes_Integer1DField_associate (rvf2, rval2[iRecord]);
    AutoRes_Real1DField_associate (rvf3, rval3[iRecord]);

    AutoRes_InOut_doEachOfRecordSection (inOut, iRecord);
  }

  AutoRes_InOut_close (inOut);
  assert(AutoRes_InOut_file (inOut) == NULL);



  AutoRes_InOut_delete (inOut);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);

  Test0 ();
  Test1 ();

  return 0;
}



