/* test_list_table.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_memory.h"
#include "autores_column.h"
#include "autores_integer_column.h"
#include "autores_real_column.h"
#include "autores_list_table.h"
#include "autores_initialize.h"



static int HashTableSize = 5;

static int HashFunction (int objectId)
{
  return objectId % HashTableSize;
}

static int ObjectIsFound 
(AutoRes_ListTable *listTable, int objectIdToFind)
{
  int listId = HashFunction (objectIdToFind);
  int linkId;
  
  for (linkId = AutoRes_ListTable_startLinkId (listTable, listId);
       linkId != -1;
       linkId = AutoRes_ListTable_nextLinkId (listTable, linkId)) {
    int objectId = AutoRes_ListTable_objectId (listTable, linkId);
    
    if (objectId == objectIdToFind) return 1;
  }
  return 0;
}

static void Test0 (void)
{
  AutoRes_ListTable *listTable;
  int nLists = HashTableSize;
  int nAverageLinksPerList = 1;
  int objectId;
  int flag;



  printf (" Test0 \n"); 



  listTable = AutoRes_ListTable_New ();
  
  flag = AutoRes_ListTable_reserve 
    (listTable, nLists, nAverageLinksPerList);
  assert(flag);
  assert(AutoRes_ListTable_nLists (listTable) == nLists);




  objectId = 101;
  AutoRes_ListTable_addObject 
    (listTable, HashFunction (objectId), objectId);
  objectId = 102;
  AutoRes_ListTable_addObject 
    (listTable, HashFunction (objectId), objectId);
  objectId = 103;
  AutoRes_ListTable_addObject 
    (listTable, HashFunction (objectId), objectId);
  objectId = 106;
  AutoRes_ListTable_addObject 
    (listTable, HashFunction (objectId), objectId);
  objectId = 107;
  AutoRes_ListTable_addObject 
    (listTable, HashFunction (objectId), objectId);
  
  assert(!ObjectIsFound (listTable, 100));
  assert(ObjectIsFound (listTable, 101));
  assert(ObjectIsFound (listTable, 102));
  assert(ObjectIsFound (listTable, 103));
  assert(!ObjectIsFound (listTable, 104));
  assert(!ObjectIsFound (listTable, 105));
  assert(ObjectIsFound (listTable, 106));
  assert(ObjectIsFound (listTable, 107));
  assert(!ObjectIsFound (listTable, 108));
  assert(!ObjectIsFound (listTable, 109));



  objectId = 101;
  AutoRes_ListTable_removeObject 
    (listTable, HashFunction (objectId), objectId);
  objectId = 102;
  AutoRes_ListTable_removeObject 
    (listTable, HashFunction (objectId), objectId);
  objectId = 107;
  AutoRes_ListTable_removeObject 
    (listTable, HashFunction (objectId), objectId);
  
  assert(!ObjectIsFound (listTable, 100));
  assert(!ObjectIsFound (listTable, 101));
  assert(!ObjectIsFound (listTable, 102));
  assert(ObjectIsFound (listTable, 103));
  assert(!ObjectIsFound (listTable, 104));
  assert(!ObjectIsFound (listTable, 105));
  assert(ObjectIsFound (listTable, 106));
  assert(!ObjectIsFound (listTable, 107));
  assert(!ObjectIsFound (listTable, 108));
  assert(!ObjectIsFound (listTable, 109));



  objectId = 107;
  AutoRes_ListTable_addObject 
    (listTable, HashFunction (objectId), objectId);
  objectId = 108;
  AutoRes_ListTable_addObject 
    (listTable, HashFunction (objectId), objectId);
  objectId = 109;
  AutoRes_ListTable_addObject 
    (listTable, HashFunction (objectId), objectId);
  
  assert(!ObjectIsFound (listTable, 100));
  assert(!ObjectIsFound (listTable, 101));
  assert(!ObjectIsFound (listTable, 102));
  assert(ObjectIsFound (listTable, 103));
  assert(!ObjectIsFound (listTable, 104));
  assert(!ObjectIsFound (listTable, 105));
  assert(ObjectIsFound (listTable, 106));
  assert(ObjectIsFound (listTable, 107));
  assert(ObjectIsFound (listTable, 108));
  assert(ObjectIsFound (listTable, 109));
  


  objectId = 100;
  AutoRes_ListTable_addObject 
    (listTable, HashFunction (objectId), objectId);
  objectId = 101;
  AutoRes_ListTable_addObject 
    (listTable, HashFunction (objectId), objectId);
  objectId = 102;
  AutoRes_ListTable_addObject 
    (listTable, HashFunction (objectId), objectId);
  objectId = 104;
  AutoRes_ListTable_addObject 
    (listTable, HashFunction (objectId), objectId);
  objectId = 105;
  AutoRes_ListTable_addObject 
    (listTable, HashFunction (objectId), objectId);
  
  assert(ObjectIsFound (listTable, 100));
  assert(ObjectIsFound (listTable, 101));
  assert(ObjectIsFound (listTable, 102));
  assert(ObjectIsFound (listTable, 103));
  assert(ObjectIsFound (listTable, 104));
  assert(ObjectIsFound (listTable, 105));
  assert(ObjectIsFound (listTable, 106));
  assert(ObjectIsFound (listTable, 107));
  assert(ObjectIsFound (listTable, 108));
  assert(ObjectIsFound (listTable, 109));



  AutoRes_ListTable_delete (listTable);
}



int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);

  Test0 ();

  return 0;
}

