/* test_ordered.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_memory.h"
#include "autores_column.h"
#include "autores_integer_column.h"
#include "autores_real_column.h"
#include "autores_ordered.h"
#include "autores_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static void Test0 (void)
{
  AutoRes_Table *table;
  AutoRes_RealColumn *valueColumn;
  AutoRes_OrderedCollection *collection;
  int nRecords;
  int nReservedRecords;
  int iRecord;
  int i;
  int count;
  int flag;



  printf (" Test0 \n"); 



  table = AutoRes_Table_New ("table");

  valueColumn = AutoRes_RealColumn_New (table, "value");
  AutoRes_Table_addRealColumn (table, valueColumn);

  collection = AutoRes_OrderedCollection_New (table);
  assert(AutoRes_OrderedCollection_table (collection) == table);

  flag = AutoRes_OrderedCollection_makeColumn (collection);
  assert(flag);
  assert(AutoRes_Table_nColumns (table) == 3); 
  


  flag = AutoRes_Table_reserveRecords (table, 3);
  assert(flag);

  for (i = 0; i < 5; i++) {
    int newRecordId = AutoRes_OrderedCollection_addRecord (collection);

    assert(newRecordId != -1);
    AUTORES_R_SET(valueColumn, newRecordId, i * 10.0);
  }
  
  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 5);
  assert(nRecords <= nReservedRecords);

  count = 0;
  for (iRecord = AutoRes_OrderedCollection_startRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_nextRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    switch (count) {
    case 0:  assert(iRecord == 0);  assert(EQ(value, 0.0));  break;
    case 1:  assert(iRecord == 1);  assert(EQ(value, 10.0));  break;
    case 2:  assert(iRecord == 2);  assert(EQ(value, 20.0));  break;
    case 3:  assert(iRecord == 3);  assert(EQ(value, 30.0));  break;
    case 4:  assert(iRecord == 4);  assert(EQ(value, 40.0));  break;
    default:  assert(0);  break;
    }
    count++;
  }

  for (iRecord = AutoRes_OrderedCollection_endRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_prevRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    count--;
    switch (count) {
    case 0:  assert(iRecord == 0);  assert(EQ(value, 0.0));  break;
    case 1:  assert(iRecord == 1);  assert(EQ(value, 10.0));  break;
    case 2:  assert(iRecord == 2);  assert(EQ(value, 20.0));  break;
    case 3:  assert(iRecord == 3);  assert(EQ(value, 30.0));  break;
    case 4:  assert(iRecord == 4);  assert(EQ(value, 40.0));  break;
    default:  assert(0);  break;
    }
  }



  AutoRes_OrderedCollection_removeRecord (collection, 2);
  AutoRes_OrderedCollection_removeRecord (collection, 3);

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 5);
  assert(nRecords <= nReservedRecords);

  count = 0;
  for (iRecord = AutoRes_OrderedCollection_startRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_nextRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    switch (count) {
    case 0:  assert(iRecord == 0);  assert(EQ(value, 0.0));  break;
    case 1:  assert(iRecord == 1);  assert(EQ(value, 10.0));  break;
    case 2:  assert(iRecord == 4);  assert(EQ(value, 40.0));  break;
    default:  assert(0);  break;
    }
    count++;
  }

  for (iRecord = AutoRes_OrderedCollection_endRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_prevRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    count--;
    switch (count) {
    case 0:  assert(iRecord == 0);  assert(EQ(value, 0.0));  break;
    case 1:  assert(iRecord == 1);  assert(EQ(value, 10.0));  break;
    case 2:  assert(iRecord == 4);  assert(EQ(value, 40.0));  break;
    default:  assert(0);  break;
    }
  }



  for (i = 5; i < 7; i++) {
    int newRecordId = AutoRes_OrderedCollection_addRecord (collection);

    assert(newRecordId != -1);
    AUTORES_R_SET(valueColumn, newRecordId, i * 10.0);
  }

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 5);
  assert(nRecords <= nReservedRecords);

  count = 0; 
  for (iRecord = AutoRes_OrderedCollection_startRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_nextRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    switch (count) {
    case 0:  assert(iRecord == 0);  assert(EQ(value, 0.0));  break;
    case 1:  assert(iRecord == 1);  assert(EQ(value, 10.0));  break;
    case 2:  assert(iRecord == 4);  assert(EQ(value, 40.0));  break;
    case 3:  assert(iRecord == 3);  assert(EQ(value, 50.0));  break;
    case 4:  assert(iRecord == 2);  assert(EQ(value, 60.0));  break;
    default:  assert(0);  break;
    }
    count++;
  }

  for (iRecord = AutoRes_OrderedCollection_endRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_prevRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    count--;
    switch (count) {
    case 0:  assert(iRecord == 0);  assert(EQ(value, 0.0));  break;
    case 1:  assert(iRecord == 1);  assert(EQ(value, 10.0));  break;
    case 2:  assert(iRecord == 4);  assert(EQ(value, 40.0));  break;
    case 3:  assert(iRecord == 3);  assert(EQ(value, 50.0));  break;
    case 4:  assert(iRecord == 2);  assert(EQ(value, 60.0));  break;
    default:  assert(0);  break;
    }
  }



  AutoRes_OrderedCollection_removeRecord (collection, 0);
  AutoRes_OrderedCollection_removeRecord (collection, 1);

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 5);
  assert(nRecords <= nReservedRecords);

  count = 0;
  for (iRecord = AutoRes_OrderedCollection_startRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_nextRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    switch (count) {
    case 0:  assert(iRecord == 4);  assert(EQ(value, 40.0));  break;
    case 1:  assert(iRecord == 3);  assert(EQ(value, 50.0));  break;
    case 2:  assert(iRecord == 2);  assert(EQ(value, 60.0));  break;
    default:  assert(0);  break;
    }
    count++;
  }

  for (iRecord = AutoRes_OrderedCollection_endRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_prevRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    count--;
    switch (count) {
    case 0:  assert(iRecord == 4);  assert(EQ(value, 40.0));  break;
    case 1:  assert(iRecord == 3);  assert(EQ(value, 50.0));  break;
    case 2:  assert(iRecord == 2);  assert(EQ(value, 60.0));  break;
    default:  assert(0);  break;
    }
  }



  for (iRecord = 7; iRecord < 12; iRecord++) {
    int newRecordId = AutoRes_OrderedCollection_addRecord (collection);

    assert(newRecordId != -1);
    AUTORES_R_SET(valueColumn, newRecordId, iRecord * 10.0);
  }

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 8);
  assert(nRecords <= nReservedRecords);

  count = 0;
  for (iRecord = AutoRes_OrderedCollection_startRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_nextRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    switch (count) {
    case 0:  assert(iRecord == 4);  assert(EQ(value, 40.0));  break;
    case 1:  assert(iRecord == 3);  assert(EQ(value, 50.0));  break;
    case 2:  assert(iRecord == 2);  assert(EQ(value, 60.0));  break;
    case 3:  assert(iRecord == 1);  assert(EQ(value, 70.0));  break;
    case 4:  assert(iRecord == 0);  assert(EQ(value, 80.0));  break;
    case 5:  assert(iRecord == 5);  assert(EQ(value, 90.0));  break;
    case 6:  assert(iRecord == 6);  assert(EQ(value, 100.0));  break;
    case 7:  assert(iRecord == 7);  assert(EQ(value, 110.0));  break;
    default:  assert(0);  break;
    }
    count++;
  }

  for (iRecord = AutoRes_OrderedCollection_endRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_prevRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    count--;
    switch (count) {
    case 0:  assert(iRecord == 4);  assert(EQ(value, 40.0));  break;
    case 1:  assert(iRecord == 3);  assert(EQ(value, 50.0));  break;
    case 2:  assert(iRecord == 2);  assert(EQ(value, 60.0));  break;
    case 3:  assert(iRecord == 1);  assert(EQ(value, 70.0));  break;
    case 4:  assert(iRecord == 0);  assert(EQ(value, 80.0));  break;
    case 5:  assert(iRecord == 5);  assert(EQ(value, 90.0));  break;
    case 6:  assert(iRecord == 6);  assert(EQ(value, 100.0));  break;
    case 7:  assert(iRecord == 7);  assert(EQ(value, 110.0));  break;
    default:  assert(0);  break;
    }
  }



  AutoRes_OrderedCollection_delete (collection);
  AutoRes_Table_delete (table);
}



int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);

  Test0 ();

  return 0;
}

