/* test_table.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_memory.h"
#include "autores_column.h"
#include "autores_integer_column.h"
#include "autores_real_column.h"
#include "autores_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static void Test0 (void)
{
  AutoRes_Table *table;

  AutoRes_IntegerColumn *rvf0;
  AutoRes_RealColumn *rvf1;
  AutoRes_Integer1DColumn *rvf2;
  AutoRes_Real1DColumn *rvf3;
  AutoRes_IntegerColumn *rvf4;
  AutoRes_Real1DColumn *rvf5;

  int nRecords;
  int nReservedRecords;
  int iRecord;
  int recordId;
  int nOldRecords;
  int flag;
  


  printf (" Test0 \n"); 



  table = AutoRes_Table_New ("table");
  assert(AutoRes_Table_nColumns (table) == 0);
  assert(AutoRes_Table_nRecords (table) == 0);
  assert(AutoRes_Table_nReservedRecords (table) == 0);

  {
    char buf[AUTORES_MAX_TABLE_NAME_LENGTH];
    
    AutoRes_Table_getName (table,
			   buf);
    assert(strcmp (buf, "table") == 0);
  }



  printf (" add column \n"); 



  rvf0 = AutoRes_IntegerColumn_New (table, "rvf0");
  flag = AutoRes_Table_addIntegerColumn (table, rvf0);
  assert(flag);

  rvf1 = AutoRes_RealColumn_New (table, "rvf1");
  flag = AutoRes_Table_addRealColumn (table, rvf1);
  assert(flag);

  rvf2 = AutoRes_Integer1DColumn_New (table, "rvf2", 2);
  flag = AutoRes_Table_addInteger1DColumn (table, rvf2);
  assert(flag);
  
  rvf3 = AutoRes_Real1DColumn_New (table, "rvf3", 3);
  flag = AutoRes_Table_addReal1DColumn (table, rvf3);
  assert(flag);

  assert(AutoRes_Table_nColumns (table) == 4);

  assert(AutoRes_Table_column (table, 0) == rvf0);
  assert(AutoRes_Table_column (table, 1) == rvf1);
  assert(AutoRes_Table_column (table, 2) == rvf2);
  assert(AutoRes_Table_column (table, 3) == rvf3);

  assert(AutoRes_Table_columnType (table, 0) == AUTORES_COLUMN_INTEGER);
  assert(AutoRes_Table_columnType (table, 1) == AUTORES_COLUMN_REAL);
  assert(AutoRes_Table_columnType (table, 2) == AUTORES_COLUMN_INTEGER1D);
  assert(AutoRes_Table_columnType (table, 3) == AUTORES_COLUMN_REAL1D);

  assert(AutoRes_Table_columnIdOfName (table, "rvf2") == 2);
  assert(AutoRes_Table_columnIdOfColumn (table, rvf3) == 3);
  assert(AutoRes_Table_columnOfName (table, "rvf1") == rvf1);



  printf (" allocate records \n"); 



  nRecords = 4;
  
  flag = AutoRes_Table_allocateRecords (table, nRecords);
  assert(flag);
  assert(AutoRes_Table_nRecords (table) == nRecords);
  assert(AutoRes_Table_nReservedRecords (table) == nRecords);

  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(AutoRes_Table_recordIdIsValid (table, 0));
  assert(AutoRes_Table_recordIdIsValid (table, 1));
  assert(AutoRes_Table_recordIdIsValid (table, 2));
  assert(AutoRes_Table_recordIdIsValid (table, 3));
  assert(!AutoRes_Table_recordIdIsValid (table, 4));
  assert(!AutoRes_Table_recordIdIsValid (table, 5));

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;

    assert(AutoRes_Table_recordIdIsValid (table, iRecord));

    AUTORES_I_SET(rvf0, iRecord, base + 1);
    AUTORES_R_SET(rvf1, iRecord, base + 2.0);
    AUTORES_I1D_SET(rvf2, iRecord, 0, base + 3);
    AUTORES_I1D_SET(rvf2, iRecord, 1, base + 4);
    AUTORES_R1D_SET(rvf3, iRecord, 0, base + 5.0);
    AUTORES_R1D_SET(rvf3, iRecord, 1, base + 6.0);
    AUTORES_R1D_SET(rvf3, iRecord, 2, base + 7.0);
  }

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;
    
    assert(AutoRes_Table_recordIdIsValid (table, iRecord));

    assert(AUTORES_I(rvf0, iRecord) == base + 1);
    assert(EQ(AUTORES_R(rvf1, iRecord), base + 2.0));
    assert(AUTORES_I1D(rvf2, iRecord, 0) == base + 3);
    assert(AUTORES_I1D(rvf2, iRecord, 1) == base + 4);
    assert(EQ(AUTORES_R1D(rvf3, iRecord, 0), base + 5.0));
    assert(EQ(AUTORES_R1D(rvf3, iRecord, 1), base + 6.0));
    assert(EQ(AUTORES_R1D(rvf3, iRecord, 2), base + 7.0));
  }



  printf (" remove column \n"); 



  AutoRes_Table_removeColumn 
    (table, AutoRes_Table_columnIdOfColumn (table, rvf3));
  rvf3 = NULL;

  AutoRes_Table_removeColumn 
    (table, AutoRes_Table_columnIdOfColumn (table, rvf1));
  rvf1 = NULL;

  assert(AutoRes_Table_nColumns (table) == 2);

  assert(AutoRes_Table_column (table, 0) == rvf0);
  assert(AutoRes_Table_column (table, 1) == rvf2);

  assert(AutoRes_Table_columnType (table, 0) == AUTORES_COLUMN_INTEGER);
  assert(AutoRes_Table_columnType (table, 1) == AUTORES_COLUMN_INTEGER1D);

  assert(AutoRes_Table_nRecords (table) == nRecords);
  assert(AutoRes_Table_nReservedRecords (table) == nRecords);

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;
    
    assert(AutoRes_Table_recordIdIsValid (table, iRecord));

    assert(AUTORES_I(rvf0, iRecord) == base + 1);
    assert(AUTORES_I1D(rvf2, iRecord, 0) == base + 3);
    assert(AUTORES_I1D(rvf2, iRecord, 1) == base + 4);
  }



  printf (" add column \n"); 



  rvf4 = AutoRes_IntegerColumn_New (table, "rvf4");
  flag = AutoRes_Table_addIntegerColumn (table, rvf4);
  assert(flag);
  
  rvf5 = AutoRes_Real1DColumn_New (table, "rvf5", 2);
  flag = AutoRes_Table_addReal1DColumn (table, rvf5);
  assert(flag);

  assert(AutoRes_Table_nColumns (table) == 4);

  assert(AutoRes_Table_column (table, 0) == rvf0);
  assert(AutoRes_Table_column (table, 1) == rvf2);
  assert(AutoRes_Table_column (table, 2) == rvf4);
  assert(AutoRes_Table_column (table, 3) == rvf5);

  assert(AutoRes_Table_columnType (table, 0) == AUTORES_COLUMN_INTEGER);
  assert(AutoRes_Table_columnType (table, 1) == AUTORES_COLUMN_INTEGER1D);
  assert(AutoRes_Table_columnType (table, 2) == AUTORES_COLUMN_INTEGER);
  assert(AutoRes_Table_columnType (table, 3) == AUTORES_COLUMN_REAL1D);

  assert(AutoRes_Table_nRecords (table) == nRecords);
  assert(AutoRes_Table_nReservedRecords (table) == nRecords);

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;

    assert(AutoRes_Table_recordIdIsValid (table, iRecord));

    AUTORES_I_SET(rvf4, iRecord, base + 8);
    AUTORES_R1D_SET(rvf5, iRecord, 0, base + 9.0);
    AUTORES_R1D_SET(rvf5, iRecord, 1, base + 9.1);
  }

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;
    
    assert(AutoRes_Table_recordIdIsValid (table, iRecord));

    assert(AUTORES_I(rvf0, iRecord) == base + 1);
    assert(AUTORES_I1D(rvf2, iRecord, 0) == base + 3);
    assert(AUTORES_I1D(rvf2, iRecord, 1) == base + 4);

    assert(AUTORES_I(rvf4, iRecord) == base + 8);
    assert(EQ(AUTORES_R1D(rvf5, iRecord, 0), base + 9.0));
    assert(EQ(AUTORES_R1D(rvf5, iRecord, 1), base + 9.1));
  }



  printf (" add record \n"); 



  nOldRecords = nRecords;

  recordId = AutoRes_Table_addRecord (table);
  assert(recordId == 4);
  nRecords++;
  
  recordId = AutoRes_Table_addRecord (table);
  assert(recordId == 5);
  nRecords++;
  
  assert(AutoRes_Table_nRecords (table) == nRecords);
  assert(nRecords <= AutoRes_Table_nReservedRecords (table));

  assert(AutoRes_Table_recordIdIsValid (table, 0));
  assert(AutoRes_Table_recordIdIsValid (table, 1));
  assert(AutoRes_Table_recordIdIsValid (table, 2));
  assert(AutoRes_Table_recordIdIsValid (table, 3));
  assert(AutoRes_Table_recordIdIsValid (table, 4));
  assert(AutoRes_Table_recordIdIsValid (table, 5));

  for (iRecord = nOldRecords; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;

    assert(AutoRes_Table_recordIdIsValid (table, iRecord));

    AUTORES_I_SET(rvf0, iRecord, base + 1);
    AUTORES_I1D_SET(rvf2, iRecord, 0, base + 3);
    AUTORES_I1D_SET(rvf2, iRecord, 1, base + 4);

    AUTORES_I_SET(rvf4, iRecord, base + 8);
    AUTORES_R1D_SET(rvf5, iRecord, 0, base + 9.0);
    AUTORES_R1D_SET(rvf5, iRecord, 1, base + 9.1);
  }

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;
    
    assert(AutoRes_Table_recordIdIsValid (table, iRecord));

    assert(AUTORES_I(rvf0, iRecord) == base + 1);
    assert(AUTORES_I1D(rvf2, iRecord, 0) == base + 3);
    assert(AUTORES_I1D(rvf2, iRecord, 1) == base + 4);

    assert(AUTORES_I(rvf4, iRecord) == base + 8);
    assert(EQ(AUTORES_R1D(rvf5, iRecord, 0), base + 9.0));
    assert(EQ(AUTORES_R1D(rvf5, iRecord, 1), base + 9.1));
  }



  printf (" pack \n"); 



  nReservedRecords = AutoRes_Table_nReservedRecords (table);
  nRecords = 2;
  
  AutoRes_Table_packNRecords (table, nRecords);
  assert(AutoRes_Table_nRecords (table) == nRecords);
  assert(AutoRes_Table_nReservedRecords (table) == nReservedRecords);
  
  assert(AutoRes_Table_nColumns (table) == 4);

  assert(AutoRes_Table_recordIdIsValid (table, 0));
  assert(AutoRes_Table_recordIdIsValid (table, 1));
  assert(!AutoRes_Table_recordIdIsValid (table, 2));
  assert(!AutoRes_Table_recordIdIsValid (table, 3));
  assert(!AutoRes_Table_recordIdIsValid (table, 4));
  assert(!AutoRes_Table_recordIdIsValid (table, 5));

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;
    
    assert(AutoRes_Table_recordIdIsValid (table, iRecord));

    assert(AUTORES_I(rvf0, iRecord) == base + 1);
    assert(AUTORES_I1D(rvf2, iRecord, 0) == base + 3);
    assert(AUTORES_I1D(rvf2, iRecord, 1) == base + 4);

    assert(AUTORES_I(rvf4, iRecord) == base + 8);
    assert(EQ(AUTORES_R1D(rvf5, iRecord, 0), base + 9.0));
    assert(EQ(AUTORES_R1D(rvf5, iRecord, 1), base + 9.1));
  }



  AutoRes_Table_delete (table);
}

static void Test1 (void)
{
  AutoRes_Table *table;

  AutoRes_IntegerColumn *rvf0;
  AutoRes_RealColumn *rvf1;
  AutoRes_Integer1DColumn *rvf2;
  AutoRes_Real1DColumn *rvf3;

  int nRecords;
  int nReservedRecords;
  int iRecord;
  int flag;
  


  printf (" Test1 \n"); 



  table = AutoRes_Table_New ("table");
  assert(AutoRes_Table_nColumns (table) == 0);
  assert(AutoRes_Table_nRecords (table) == 0);
  assert(AutoRes_Table_nReservedRecords (table) == 0);

  {
    char buf[AUTORES_MAX_TABLE_NAME_LENGTH];
    
    AutoRes_Table_getName (table,
			   buf);
    assert(strcmp (buf, "table") == 0);
  }



  printf (" add column \n"); 



  rvf0 = AutoRes_IntegerColumn_New (table, "rvf0");
  flag = AutoRes_Table_addIntegerColumn (table, rvf0);
  assert(flag);

  rvf1 = AutoRes_RealColumn_New (table, "rvf1");
  flag = AutoRes_Table_addRealColumn (table, rvf1);
  assert(flag);

  rvf2 = AutoRes_Integer1DColumn_New (table, "rvf2", 2);
  flag = AutoRes_Table_addInteger1DColumn (table, rvf2);
  assert(flag);
  
  rvf3 = AutoRes_Real1DColumn_New (table, "rvf3", 3);
  flag = AutoRes_Table_addReal1DColumn (table, rvf3);
  assert(flag);

  assert(AutoRes_Table_nColumns (table) == 4);

  assert(AutoRes_Table_column (table, 0) == rvf0);
  assert(AutoRes_Table_column (table, 1) == rvf1);
  assert(AutoRes_Table_column (table, 2) == rvf2);
  assert(AutoRes_Table_column (table, 3) == rvf3);

  assert(AutoRes_Table_columnType (table, 0) == AUTORES_COLUMN_INTEGER);
  assert(AutoRes_Table_columnType (table, 1) == AUTORES_COLUMN_REAL);
  assert(AutoRes_Table_columnType (table, 2) == AUTORES_COLUMN_INTEGER1D);
  assert(AutoRes_Table_columnType (table, 3) == AUTORES_COLUMN_REAL1D);

  assert(AutoRes_Table_columnIdOfName (table, "rvf2") == 2);
  assert(AutoRes_Table_columnIdOfColumn (table, rvf3) == 3);
  assert(AutoRes_Table_columnOfName (table, "rvf1") == rvf1);



  printf (" reserve records, add record \n"); 



  nRecords = 5;
  
  flag = AutoRes_Table_reserveRecords (table, 2);
  assert(flag);
  assert(AutoRes_Table_nRecords (table) == 0);
  assert(AutoRes_Table_nReservedRecords (table) == 2);

  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(!AutoRes_Table_recordIdIsValid (table, 0));
  assert(!AutoRes_Table_recordIdIsValid (table, 1));
  assert(!AutoRes_Table_recordIdIsValid (table, 2));
  assert(!AutoRes_Table_recordIdIsValid (table, 3));
  assert(!AutoRes_Table_recordIdIsValid (table, 4));
  assert(!AutoRes_Table_recordIdIsValid (table, 5));

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;
    int recordId = AutoRes_Table_addRecord (table);

    assert(recordId != -1);
    assert(AutoRes_Table_recordIdIsValid (table, iRecord));
    assert(recordId == iRecord);

    AUTORES_I_SET(rvf0, recordId, base + 1);
    AUTORES_R_SET(rvf1, recordId, base + 2.0);
    AUTORES_I1D_SET(rvf2, recordId, 0, base + 3);
    AUTORES_I1D_SET(rvf2, recordId, 1, base + 4);
    AUTORES_R1D_SET(rvf3, recordId, 0, base + 5.0);
    AUTORES_R1D_SET(rvf3, recordId, 1, base + 6.0);
    AUTORES_R1D_SET(rvf3, recordId, 2, base + 7.0);
  }

  assert(AutoRes_Table_nRecords (table) == nRecords);
  assert(nRecords <= AutoRes_Table_nReservedRecords (table));

  assert(AutoRes_Table_recordIdIsValid (table, 0));
  assert(AutoRes_Table_recordIdIsValid (table, 1));
  assert(AutoRes_Table_recordIdIsValid (table, 2));
  assert(AutoRes_Table_recordIdIsValid (table, 3));
  assert(AutoRes_Table_recordIdIsValid (table, 4));
  assert(!AutoRes_Table_recordIdIsValid (table, 5));

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;
    
    assert(AutoRes_Table_recordIdIsValid (table, iRecord));

    assert(AUTORES_I(rvf0, iRecord) == base + 1);
    assert(EQ(AUTORES_R(rvf1, iRecord), base + 2.0));
    assert(AUTORES_I1D(rvf2, iRecord, 0) == base + 3);
    assert(AUTORES_I1D(rvf2, iRecord, 1) == base + 4);
    assert(EQ(AUTORES_R1D(rvf3, iRecord, 0), base + 5.0));
    assert(EQ(AUTORES_R1D(rvf3, iRecord, 1), base + 6.0));
    assert(EQ(AUTORES_R1D(rvf3, iRecord, 2), base + 7.0));
  }



  AutoRes_Table_delete (table);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);

  Test0 ();
  Test1 ();

  return 0;
}



