/* test_table_io.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_field.h"
#include "autores_integer_field.h"
#include "autores_real_field.h"
#include "autores_io.h"
#include "autores_memory.h"
#include "autores_column.h"
#include "autores_integer_column.h"
#include "autores_real_column.h"
#include "autores_table_io.h"
#include "autores_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

#define META_DATA_FILE_NAME_0 "test_table_io_test0.cdm"
#define META_DATA_FILE_NAME_1 "test_table_io_test1.cdm"
#define FILE_NAME_0 "test_table_io_test0.txt"
#define FILE_NAME_1 "test_table_io_test1.txt"
#define FILE_NAME_2 "test_table_io_test2.txt"

static void Test0 (void)
{
  AutoRes_Table *table;

  AutoRes_IntegerColumn *column0;
  AutoRes_RealColumn *column1;
  AutoRes_Integer1DColumn *column2;
  AutoRes_Real1DColumn *column3;

  AutoRes_TableInOut *tableInOut;
  AutoRes_InOut *inOut;
  int writeFlag;
  int binaryFlag = 0;

  AutoRes_IntegerField *hvf0;
  AutoRes_RealField *hvf1;
  AutoRes_Integer1DField *hvf2;
  AutoRes_Real1DField *hvf3;
  AutoRes_IntegerField *rvf0;
  AutoRes_RealField *rvf1;
  AutoRes_Integer1DField *rvf2;
  AutoRes_Real1DField *rvf3;

  int hval0;
  double hval1;
  int hval2[2];
  double hval3[3];

  int nRecords;
  int iRecord;
  int flag;
  


  printf (" Test0 \n"); 



  printf (" make table \n"); 



  table = AutoRes_Table_New ("table");

  column0 = AutoRes_IntegerColumn_New (table, "rvf0");
  AutoRes_Table_addIntegerColumn (table, column0);

  column1 = AutoRes_RealColumn_New (table, "rvf1");
  AutoRes_Table_addRealColumn (table, column1);

  column2 = AutoRes_Integer1DColumn_New (table, "rvf2", 2);
  AutoRes_Table_addInteger1DColumn (table, column2);
  
  column3 = AutoRes_Real1DColumn_New (table, "rvf3", 3);
  AutoRes_Table_addReal1DColumn (table, column3);

  nRecords = 4;
  
  AutoRes_Table_allocateRecords (table, nRecords);

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;

    AUTORES_I_SET(column0, iRecord, base + 1);
    AUTORES_R_SET(column1, iRecord, base + 2.0);
    AUTORES_I1D_SET(column2, iRecord, 0, base + 3);
    AUTORES_I1D_SET(column2, iRecord, 1, base + 4);
    AUTORES_R1D_SET(column3, iRecord, 0, base + 5.0);
    AUTORES_R1D_SET(column3, iRecord, 1, base + 6.0);
    AUTORES_R1D_SET(column3, iRecord, 2, base + 7.0);
  }



  printf (" make header \n"); 



  hval0 = 1;
  hval1 = 2.0;
  hval2[0] = 3;
  hval2[1] = 4;
  hval3[0] = 5.0;
  hval3[1] = 6.0;
  hval3[2] = 7.0;



  printf (" make table I/O \n"); 



  tableInOut = AutoRes_TableInOut_New (table);
  inOut = AutoRes_TableInOut_inOut (tableInOut);

  /* define header section */

  hvf0 = AutoRes_IntegerField_New ("hvf0");
  AutoRes_InOut_addIntegerFieldIntoHeader (inOut, hvf0);
  AutoRes_IntegerField_associate (hvf0, &hval0);

  hvf1 = AutoRes_RealField_New ("hvf1");
  AutoRes_InOut_addRealFieldIntoHeader (inOut, hvf1);
  AutoRes_RealField_associate (hvf1, &hval1);

  hvf2 = AutoRes_Integer1DField_New ("hvf2", 2);
  AutoRes_InOut_addInteger1DFieldIntoHeader (inOut, hvf2);
  AutoRes_Integer1DField_associate (hvf2, hval2);
  
  hvf3 = AutoRes_Real1DField_New ("hvf3", 3);
  AutoRes_InOut_addReal1DFieldIntoHeader (inOut, hvf3);
  AutoRes_Real1DField_associate (hvf3, hval3);

  /* define record section */

  rvf0 = AutoRes_IntegerField_New ("rvf0");
  AutoRes_InOut_addIntegerFieldIntoRecord (inOut, rvf0);
  AutoRes_TableInOut_associateColumnWithField 
    (tableInOut, 
     column0, AUTORES_COLUMN_INTEGER, 
     rvf0, AUTORES_FIELD_INTEGER);

  rvf1 = AutoRes_RealField_New ("rvf1");
  AutoRes_InOut_addRealFieldIntoRecord (inOut, rvf1);
  AutoRes_TableInOut_associateColumnWithField 
    (tableInOut, 
     column1, AUTORES_COLUMN_REAL,
     rvf1, AUTORES_FIELD_REAL);

  rvf2 = AutoRes_Integer1DField_New ("rvf2", 2);
  AutoRes_InOut_addInteger1DFieldIntoRecord (inOut, rvf2);
  AutoRes_TableInOut_associateColumnWithField 
    (tableInOut, 
     column2, AUTORES_COLUMN_INTEGER1D, 
     rvf2, AUTORES_FIELD_INTEGER1D);
  
  rvf3 = AutoRes_Real1DField_New ("rvf3", 3);
  AutoRes_InOut_addReal1DFieldIntoRecord (inOut, rvf3);
  AutoRes_TableInOut_associateColumnWithField 
    (tableInOut, 
     column3, AUTORES_COLUMN_REAL1D,
     rvf3, AUTORES_FIELD_REAL1D);



  printf (" write meta-data \n"); 



  AutoRes_InOut_writeMetaData (inOut, META_DATA_FILE_NAME_0);



  printf (" write file \n"); 



  writeFlag = 1;
  flag = AutoRes_TableInOut_do 
    (tableInOut, FILE_NAME_0, writeFlag, binaryFlag);
  assert(flag);



  printf (" read file \n"); 



  hval0 = 0;
  hval1 = 0.0;
  hval2[0] = 0;
  hval2[1] = 0;
  hval3[0] = 0.0;
  hval3[1] = 0.0;
  hval3[2] = 0.0;

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    AUTORES_I_SET(column0, iRecord, 0);
    AUTORES_R_SET(column1, iRecord, 0.0);
    AUTORES_I1D_SET(column2, iRecord, 0, 0);
    AUTORES_I1D_SET(column2, iRecord, 1, 0);
    AUTORES_R1D_SET(column3, iRecord, 0, 0.0);
    AUTORES_R1D_SET(column3, iRecord, 1, 0.0);
    AUTORES_R1D_SET(column3, iRecord, 2, 0.0);
  }

  writeFlag = 0;
  flag = AutoRes_TableInOut_do 
    (tableInOut, FILE_NAME_0, writeFlag, binaryFlag);
  assert(flag);

  assert(hval0 == 1);
  assert(EQ(hval1, 2.0));
  assert(hval2[0] == 3);
  assert(hval2[1] == 4);
  assert(EQ(hval3[0], 5.0));
  assert(EQ(hval3[1], 6.0));
  assert(EQ(hval3[2], 7.0));
  
  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;
    
    assert(AUTORES_I(column0, iRecord) == base + 1);
    assert(EQ(AUTORES_R(column1, iRecord), base + 2.0));
    assert(AUTORES_I1D(column2, iRecord, 0) == base + 3);
    assert(AUTORES_I1D(column2, iRecord, 1) == base + 4);
    assert(EQ(AUTORES_R1D(column3, iRecord, 0), base + 5.0));
    assert(EQ(AUTORES_R1D(column3, iRecord, 1), base + 6.0));
    assert(EQ(AUTORES_R1D(column3, iRecord, 2), base + 7.0));
  }



  AutoRes_TableInOut_delete (tableInOut);
  AutoRes_Table_delete (table);
}

static void Test1 (void)
{
  AutoRes_Table *table;

  AutoRes_IntegerColumn *column0;
  AutoRes_RealColumn *column1;
  AutoRes_Integer1DColumn *column2;
  AutoRes_Real1DColumn *column3;

  AutoRes_TableInOut *tableInOut;
  AutoRes_InOut *inOut;
  int writeFlag;
  int binaryFlag = 0;

  AutoRes_IntegerField *hvf0;
  AutoRes_RealField *hvf1;
  AutoRes_Integer1DField *hvf2;
  AutoRes_Real1DField *hvf3;

  int hval0;
  double hval1;
  int hval2[2];
  double hval3[3];

  int nRecords;
  int iRecord;
  int flag;
  


  printf (" Test1 \n"); 



  printf (" make table \n"); 



  table = AutoRes_Table_New ("table");

  column0 = AutoRes_IntegerColumn_New (table, "rvf0");
  AutoRes_Table_addIntegerColumn (table, column0);

  column1 = AutoRes_RealColumn_New (table, "rvf1");
  AutoRes_Table_addRealColumn (table, column1);

  column2 = AutoRes_Integer1DColumn_New (table, "rvf2", 2);
  AutoRes_Table_addInteger1DColumn (table, column2);
  
  column3 = AutoRes_Real1DColumn_New (table, "rvf3", 3);
  AutoRes_Table_addReal1DColumn (table, column3);

  nRecords = 4;
  
  AutoRes_Table_allocateRecords (table, nRecords);

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;

    AUTORES_I_SET(column0, iRecord, base + 1);
    AUTORES_R_SET(column1, iRecord, base + 2.0);
    AUTORES_I1D_SET(column2, iRecord, 0, base + 3);
    AUTORES_I1D_SET(column2, iRecord, 1, base + 4);
    AUTORES_R1D_SET(column3, iRecord, 0, base + 5.0);
    AUTORES_R1D_SET(column3, iRecord, 1, base + 6.0);
    AUTORES_R1D_SET(column3, iRecord, 2, base + 7.0);
  }



  printf (" make header \n"); 



  hval0 = 1;
  hval1 = 2.0;
  hval2[0] = 3;
  hval2[1] = 4;
  hval3[0] = 5.0;
  hval3[1] = 6.0;
  hval3[2] = 7.0;



  printf (" make table I/O \n"); 



  tableInOut = AutoRes_TableInOut_New (table);
  inOut = AutoRes_TableInOut_inOut (tableInOut);

  /* define header section */

  hvf0 = AutoRes_IntegerField_New ("hvf0");
  AutoRes_InOut_addIntegerFieldIntoHeader (inOut, hvf0);
  AutoRes_IntegerField_associate (hvf0, &hval0);

  hvf1 = AutoRes_RealField_New ("hvf1");
  AutoRes_InOut_addRealFieldIntoHeader (inOut, hvf1);
  AutoRes_RealField_associate (hvf1, &hval1);

  hvf2 = AutoRes_Integer1DField_New ("hvf2", 2);
  AutoRes_InOut_addInteger1DFieldIntoHeader (inOut, hvf2);
  AutoRes_Integer1DField_associate (hvf2, hval2);
  
  hvf3 = AutoRes_Real1DField_New ("hvf3", 3);
  AutoRes_InOut_addReal1DFieldIntoHeader (inOut, hvf3);
  AutoRes_Real1DField_associate (hvf3, hval3);

  /* define record section */

  AutoRes_TableInOut_associateIntegerColumn (tableInOut, column0);
  AutoRes_TableInOut_associateRealColumn (tableInOut, column1);
  AutoRes_TableInOut_associateInteger1DColumn (tableInOut, column2);
  AutoRes_TableInOut_associateReal1DColumn (tableInOut, column3);



  printf (" write meta-data \n"); 



  AutoRes_InOut_writeMetaData (inOut, META_DATA_FILE_NAME_1);



  printf (" write file \n"); 



  writeFlag = 1;
  flag = AutoRes_TableInOut_do 
    (tableInOut, FILE_NAME_1, writeFlag, binaryFlag);
  assert(flag);



  printf (" read file \n"); 



  hval0 = 0;
  hval1 = 0.0;
  hval2[0] = 0;
  hval2[1] = 0;
  hval3[0] = 0.0;
  hval3[1] = 0.0;
  hval3[2] = 0.0;

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    AUTORES_I_SET(column0, iRecord, 0);
    AUTORES_R_SET(column1, iRecord, 0.0);
    AUTORES_I1D_SET(column2, iRecord, 0, 0);
    AUTORES_I1D_SET(column2, iRecord, 1, 0);
    AUTORES_R1D_SET(column3, iRecord, 0, 0.0);
    AUTORES_R1D_SET(column3, iRecord, 1, 0.0);
    AUTORES_R1D_SET(column3, iRecord, 2, 0.0);
  }

  writeFlag = 0;
  flag = AutoRes_TableInOut_do 
    (tableInOut, FILE_NAME_1, writeFlag, binaryFlag);
  assert(flag);

  assert(hval0 == 1);
  assert(EQ(hval1, 2.0));
  assert(hval2[0] == 3);
  assert(hval2[1] == 4);
  assert(EQ(hval3[0], 5.0));
  assert(EQ(hval3[1], 6.0));
  assert(EQ(hval3[2], 7.0));
  
  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;
    
    assert(AUTORES_I(column0, iRecord) == base + 1);
    assert(EQ(AUTORES_R(column1, iRecord), base + 2.0));
    assert(AUTORES_I1D(column2, iRecord, 0) == base + 3);
    assert(AUTORES_I1D(column2, iRecord, 1) == base + 4);
    assert(EQ(AUTORES_R1D(column3, iRecord, 0), base + 5.0));
    assert(EQ(AUTORES_R1D(column3, iRecord, 1), base + 6.0));
    assert(EQ(AUTORES_R1D(column3, iRecord, 2), base + 7.0));
  }



  AutoRes_TableInOut_delete (tableInOut);
  AutoRes_Table_delete (table);
}

static void Test2 (void)
{
  AutoRes_Table *table = NULL;

  AutoRes_IntegerColumn *column0 = NULL;
  AutoRes_RealColumn *column1 = NULL;
  AutoRes_Integer1DColumn *column2 = NULL;
  AutoRes_Real1DColumn *column3 = NULL;

  AutoRes_TableInOut *tableInOut = NULL;
  AutoRes_InOut *inOut = NULL;
  int writeFlag;
  int binaryFlag = 0;

  AutoRes_IntegerField *hvf0 = NULL;
  AutoRes_RealField *hvf1 = NULL;
  AutoRes_Integer1DField *hvf2 = NULL;
  AutoRes_Real1DField *hvf3 = NULL;

  int hval0;
  double hval1;
  int hval2[2];
  double hval3[3];

  int nRecords;
  int iRecord;
  int flag;
  


  printf (" Test2 \n"); 



  printf (" make table \n"); 



  table = AutoRes_Table_New ("table");

  column0 = AutoRes_IntegerColumn_New (table, "rvf0");
  AutoRes_Table_addIntegerColumn (table, column0);

  column3 = AutoRes_Real1DColumn_New (table, "rvf3", 3);
  AutoRes_Table_addReal1DColumn (table, column3);

  hval0 = 0;
  hval1 = 0.0;
  hval2[0] = 0;
  hval2[1] = 0;
  hval3[0] = 0.0;
  hval3[1] = 0.0;
  hval3[2] = 0.0;



  printf (" make table I/O \n"); 



  tableInOut = AutoRes_TableInOut_New (table);
  inOut = AutoRes_TableInOut_inOut (tableInOut);

  AutoRes_InOut_readMetaData (inOut, META_DATA_FILE_NAME_0);

  /* define header section */

  hvf1 = (AutoRes_RealField *)
    AutoRes_InOut_headerFieldOfName (inOut, "hvf1");
  assert(hvf1 != NULL);
  AutoRes_RealField_associate (hvf1, &hval1);

  hvf2 = (AutoRes_Integer1DField *)
    AutoRes_InOut_headerFieldOfName (inOut, "hvf2");
  assert(hvf2 != NULL);
  AutoRes_Integer1DField_associate (hvf2, hval2);

  /* define record section */

  AutoRes_TableInOut_associateIntegerColumn (tableInOut, column0);
  AutoRes_TableInOut_associateReal1DColumn (tableInOut, column3);



  printf (" read file \n"); 



  writeFlag = 0;
  flag = AutoRes_TableInOut_do 
    (tableInOut, FILE_NAME_0, writeFlag, binaryFlag);
  assert(flag);

  nRecords = AutoRes_Table_nRecords (table);
  
  assert(nRecords == 4);
  assert(AutoRes_Table_nReservedRecords (table) == nRecords);

  assert(hval0 == 0);
  assert(EQ(hval1, 2.0));
  assert(hval2[0] == 3);
  assert(hval2[1] == 4);
  assert(EQ(hval3[0], 0.0));
  assert(EQ(hval3[1], 0.0));
  assert(EQ(hval3[2], 0.0));
  
  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;
    
    assert(AUTORES_I(column0, iRecord) == base + 1);
    assert(EQ(AUTORES_R1D(column3, iRecord, 0), base + 5.0));
    assert(EQ(AUTORES_R1D(column3, iRecord, 1), base + 6.0));
    assert(EQ(AUTORES_R1D(column3, iRecord, 2), base + 7.0));
  }



  printf (" delete table I/O \n"); 



  AutoRes_TableInOut_delete (tableInOut);
  tableInOut = NULL;
  inOut = NULL;
  hvf1 = NULL;
  hvf2 = NULL;



  printf (" add column \n"); 



  column1 = AutoRes_RealColumn_New (table, "rvf1");
  AutoRes_Table_addRealColumn (table, column1);

  column2 = AutoRes_Integer1DColumn_New (table, "rvf2", 2);
  AutoRes_Table_addInteger1DColumn (table, column2);
  


  printf (" make table I/O, second \n"); 



  tableInOut = AutoRes_TableInOut_New (table);
  inOut = AutoRes_TableInOut_inOut (tableInOut);

  AutoRes_InOut_readMetaData (inOut, META_DATA_FILE_NAME_0);

  /* define header section */

  hvf0 = (AutoRes_IntegerField *)
    AutoRes_InOut_headerFieldOfName (inOut, "hvf0");
  assert(hvf0 != NULL);
  AutoRes_IntegerField_associate (hvf0, &hval0);

  hvf3 = (AutoRes_Real1DField *)
    AutoRes_InOut_headerFieldOfName (inOut, "hvf3");
  assert(hvf3 != NULL);
  AutoRes_Real1DField_associate (hvf3, hval3);

  /* define record section */

  AutoRes_TableInOut_associateRealColumn (tableInOut, column1);
  AutoRes_TableInOut_associateInteger1DColumn (tableInOut, column2);



  printf (" read file, second \n"); 



  writeFlag = 0;
  flag = AutoRes_TableInOut_do 
    (tableInOut, FILE_NAME_0, writeFlag, binaryFlag);
  assert(flag);

  nRecords = AutoRes_Table_nRecords (table);
  
  assert(nRecords == 4);
  assert(AutoRes_Table_nReservedRecords (table) == nRecords);

  assert(hval0 == 1);
  assert(EQ(hval1, 2.0));
  assert(hval2[0] == 3);
  assert(hval2[1] == 4);
  assert(EQ(hval3[0], 5.0));
  assert(EQ(hval3[1], 6.0));
  assert(EQ(hval3[2], 7.0));
  
  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;
    
    assert(AUTORES_I(column0, iRecord) == base + 1);
    assert(EQ(AUTORES_R(column1, iRecord), base + 2.0));
    assert(AUTORES_I1D(column2, iRecord, 0) == base + 3);
    assert(AUTORES_I1D(column2, iRecord, 1) == base + 4);
    assert(EQ(AUTORES_R1D(column3, iRecord, 0), base + 5.0));
    assert(EQ(AUTORES_R1D(column3, iRecord, 1), base + 6.0));
    assert(EQ(AUTORES_R1D(column3, iRecord, 2), base + 7.0));
  }



  printf (" delete table I/O \n"); 



  AutoRes_TableInOut_delete (tableInOut);
  tableInOut = NULL;
  inOut = NULL;
  hvf0 = NULL;
  hvf3 = NULL;



  printf (" make table I/O, third \n"); 



  tableInOut = AutoRes_TableInOut_New (table);
  inOut = AutoRes_TableInOut_inOut (tableInOut);

  AutoRes_InOut_readMetaData (inOut, META_DATA_FILE_NAME_0);

  /* define header section */

  hvf0 = (AutoRes_IntegerField *)
    AutoRes_InOut_headerFieldOfName (inOut, "hvf0");
  assert(hvf0 != NULL);
  AutoRes_IntegerField_associate (hvf0, &hval0);

  hvf1 = (AutoRes_RealField *)
    AutoRes_InOut_headerFieldOfName (inOut, "hvf1");
  assert(hvf1 != NULL);
  AutoRes_RealField_associate (hvf1, &hval1);

  hvf2 = (AutoRes_Integer1DField *)
    AutoRes_InOut_headerFieldOfName (inOut, "hvf2");
  assert(hvf2 != NULL);
  AutoRes_Integer1DField_associate (hvf2, hval2);

  hvf3 = (AutoRes_Real1DField *)
    AutoRes_InOut_headerFieldOfName (inOut, "hvf3");
  assert(hvf3 != NULL);
  AutoRes_Real1DField_associate (hvf3, hval3);

  /* define record section */

  AutoRes_TableInOut_associateIntegerColumn (tableInOut, column0);
  AutoRes_TableInOut_associateRealColumn (tableInOut, column1);
  AutoRes_TableInOut_associateInteger1DColumn (tableInOut, column2);
  AutoRes_TableInOut_associateReal1DColumn (tableInOut, column3);



  printf (" write file \n"); 



  writeFlag = 1;
  flag = AutoRes_TableInOut_do 
    (tableInOut, FILE_NAME_2, writeFlag, binaryFlag);
  assert(flag);



  AutoRes_TableInOut_delete (tableInOut);



  AutoRes_Table_delete (table);
}

static void Test3 (void)
{
  AutoRes_Table *table;

  AutoRes_IntegerColumn *column0;
  AutoRes_RealColumn *column1;
  AutoRes_Integer1DColumn *column2;
  AutoRes_Real1DColumn *column3;

  int binaryFlag = 0;
  int recordIdFlag = 0;
  int nRecords;
  int iRecord;
  int flag;
  


  printf (" Test3 \n"); 



  printf (" make table \n"); 



  table = AutoRes_Table_New ("table");

  column0 = AutoRes_IntegerColumn_New (table, "rvf0");
  AutoRes_Table_addIntegerColumn (table, column0);

  column1 = AutoRes_RealColumn_New (table, "rvf1");
  AutoRes_Table_addRealColumn (table, column1);

  column2 = AutoRes_Integer1DColumn_New (table, "rvf2", 2);
  AutoRes_Table_addInteger1DColumn (table, column2);
  
  column3 = AutoRes_Real1DColumn_New (table, "rvf3", 3);
  AutoRes_Table_addReal1DColumn (table, column3);

  nRecords = 4;
  
  AutoRes_Table_allocateRecords (table, nRecords);

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;

    AUTORES_I_SET(column0, iRecord, base + 1);
    AUTORES_R_SET(column1, iRecord, base + 2.0);
    AUTORES_I1D_SET(column2, iRecord, 0, base + 3);
    AUTORES_I1D_SET(column2, iRecord, 1, base + 4);
    AUTORES_R1D_SET(column3, iRecord, 0, base + 5.0);
    AUTORES_R1D_SET(column3, iRecord, 1, base + 6.0);
    AUTORES_R1D_SET(column3, iRecord, 2, base + 7.0);
  }

  AutoRes_TableInOut_SetBinaryFlag (binaryFlag);
  AutoRes_TableInOut_SetRecordIdFlag (recordIdFlag);



  printf (" write meta-data \n"); 



  AutoRes_TableInOut_AssociateIntegerColumn (column0);
  flag = AutoRes_TableInOut_WriteMetaData ("test_table_io_column0.cdm");
  assert(flag);

  AutoRes_TableInOut_AssociateRealColumn (column1);
  flag = AutoRes_TableInOut_WriteMetaData ("test_table_io_column1.cdm");
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (column2);
  AutoRes_TableInOut_AssociateReal1DColumn (column3);
  flag = AutoRes_TableInOut_WriteMetaData ("test_table_io_column23.cdm");
  assert(flag);



  printf (" write file \n"); 



  AutoRes_TableInOut_AssociateIntegerColumn (column0);
  flag = AutoRes_TableInOut_Write ("test_table_io_column0.txt");
  assert(flag);

  AutoRes_TableInOut_AssociateRealColumn (column1);
  flag = AutoRes_TableInOut_Write ("test_table_io_column1.txt");
  assert(flag);

  AutoRes_TableInOut_AssociateInteger1DColumn (column2);
  AutoRes_TableInOut_AssociateReal1DColumn (column3);
  flag = AutoRes_TableInOut_Write ("test_table_io_column23.txt");
  assert(flag);



  printf (" read file \n"); 


  
  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    AUTORES_I_SET(column0, iRecord, 0);
    AUTORES_R_SET(column1, iRecord, 0.0);
    AUTORES_I1D_SET(column2, iRecord, 0, 0);
    AUTORES_I1D_SET(column2, iRecord, 1, 0);
    AUTORES_R1D_SET(column3, iRecord, 0, 0.0);
    AUTORES_R1D_SET(column3, iRecord, 1, 0.0);
    AUTORES_R1D_SET(column3, iRecord, 2, 0.0);
  }

  AutoRes_TableInOut_AssociateIntegerColumn (column0);
  flag = AutoRes_TableInOut_Read ("test_table_io_column0.txt");
  assert(flag);

  AutoRes_TableInOut_AssociateRealColumn (column1);
  flag = AutoRes_TableInOut_Read ("test_table_io_column1.txt");
  assert(flag);

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;
    
    assert(AUTORES_I(column0, iRecord) == base + 1);
    assert(EQ(AUTORES_R(column1, iRecord), base + 2.0));
    assert(AUTORES_I1D(column2, iRecord, 0) == 0);
    assert(AUTORES_I1D(column2, iRecord, 1) == 0);
    assert(EQ(AUTORES_R1D(column3, iRecord, 0), 0.0));
    assert(EQ(AUTORES_R1D(column3, iRecord, 1), 0.0));
    assert(EQ(AUTORES_R1D(column3, iRecord, 2), 0.0));
  }

  AutoRes_TableInOut_AssociateInteger1DColumn (column2);
  AutoRes_TableInOut_AssociateReal1DColumn (column3);
  flag = AutoRes_TableInOut_Read ("test_table_io_column23.txt");
  assert(flag);

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int base = (iRecord + 1) * 10;
    
    assert(AUTORES_I(column0, iRecord) == base + 1);
    assert(EQ(AUTORES_R(column1, iRecord), base + 2.0));
    assert(AUTORES_I1D(column2, iRecord, 0) == base + 3);
    assert(AUTORES_I1D(column2, iRecord, 1) == base + 4);
    assert(EQ(AUTORES_R1D(column3, iRecord, 0), base + 5.0));
    assert(EQ(AUTORES_R1D(column3, iRecord, 1), base + 6.0));
    assert(EQ(AUTORES_R1D(column3, iRecord, 2), base + 7.0));
  }



  AutoRes_Table_delete (table);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);

  Test0 ();
  Test1 ();
  Test2 ();
  Test3 ();

  return 0;
}



