/* test_use_flag.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores_base.h"
#include "autores_memory.h"
#include "autores_column.h"
#include "autores_integer_column.h"
#include "autores_real_column.h"
#include "autores_use_flag.h"
#include "autores_initialize.h"



#define EQ(a,b)  fabs ((a) - (b)) < 0.00001

static void Test0 (void)
{
  AutoRes_Table *table;
  AutoRes_RealColumn *valueColumn;
  AutoRes_UseFlagCollection *collection;
  int nRecords;
  int nReservedRecords;
  int i;
  int flag;



  printf (" Test0 \n"); 



  table = AutoRes_Table_New ("table");

  valueColumn = AutoRes_RealColumn_New (table, "value");
  AutoRes_Table_addRealColumn (table, valueColumn);

  collection = AutoRes_UseFlagCollection_New (table);
  assert(AutoRes_UseFlagCollection_table (collection) == table);

  flag = AutoRes_UseFlagCollection_makeColumn (collection);
  assert(flag);
  assert(AutoRes_Table_nColumns (table) == 2); 
  


  flag = AutoRes_Table_reserveRecords (table, 3);
  assert(flag);

  for (i = 0; i < 5; i++) {
    int newRecordId = AutoRes_UseFlagCollection_addRecord (collection);

    assert(newRecordId != -1);
    AUTORES_R_SET(valueColumn, newRecordId, i * 10.0);
  }
  
  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 5);
  assert(nRecords <= nReservedRecords);

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 0));
  assert(EQ(AUTORES_R(valueColumn, 0), 0.0));
  
  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 1));
  assert(EQ(AUTORES_R(valueColumn, 1), 10.0));
  
  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 2));
  assert(EQ(AUTORES_R(valueColumn, 2), 20.0));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 3));
  assert(EQ(AUTORES_R(valueColumn, 3), 30.0));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 4));
  assert(EQ(AUTORES_R(valueColumn, 4), 40.0));



  AutoRes_UseFlagCollection_removeRecord (collection, 2);
  AutoRes_UseFlagCollection_removeRecord (collection, 3);

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 5);
  assert(nRecords <= nReservedRecords);

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 0));
  assert(EQ(AUTORES_R(valueColumn, 0), 0.0));
  
  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 1));
  assert(EQ(AUTORES_R(valueColumn, 1), 10.0));
  
  assert(!AutoRes_UseFlagCollection_recordIsUsed (collection, 2));

  assert(!AutoRes_UseFlagCollection_recordIsUsed (collection, 3));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 4));
  assert(EQ(AUTORES_R(valueColumn, 4), 40.0));



  for (i = 5; i < 7; i++) {
    int newRecordId = AutoRes_UseFlagCollection_addRecord (collection);

    assert(newRecordId != -1);
    AUTORES_R_SET(valueColumn, newRecordId, i * 10.0);
  }

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 7);
  assert(nRecords <= nReservedRecords);

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 0));
  assert(EQ(AUTORES_R(valueColumn, 0), 0.0));
  
  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 1));
  assert(EQ(AUTORES_R(valueColumn, 1), 10.0));
  
  assert(!AutoRes_UseFlagCollection_recordIsUsed (collection, 2));

  assert(!AutoRes_UseFlagCollection_recordIsUsed (collection, 3));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 4));
  assert(EQ(AUTORES_R(valueColumn, 4), 40.0));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 5));
  assert(EQ(AUTORES_R(valueColumn, 5), 50.0));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 6));
  assert(EQ(AUTORES_R(valueColumn, 6), 60.0));



  AutoRes_UseFlagCollection_pack (collection);

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 5);
  assert(nRecords <= nReservedRecords);

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 0));
  assert(EQ(AUTORES_R(valueColumn, 0), 0.0));
  
  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 1));
  assert(EQ(AUTORES_R(valueColumn, 1), 10.0));
  
  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 2));
  assert(EQ(AUTORES_R(valueColumn, 2), 40.0));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 3));
  assert(EQ(AUTORES_R(valueColumn, 3), 50.0));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 4));
  assert(EQ(AUTORES_R(valueColumn, 4), 60.0));



  AutoRes_UseFlagCollection_delete (collection);
  AutoRes_Table_delete (table);
}

static void Test1 (void)
{
  AutoRes_Table *table;
  AutoRes_RealColumn *valueColumn;
  AutoRes_Real1DColumn *valuesColumn;
  AutoRes_UseFlagCollection *collection;
  int nRecords;
  int nReservedRecords;
  int iRecord;
  int flag;



  printf (" Test1 \n"); 



  table = AutoRes_Table_New ("table");

  valueColumn = AutoRes_RealColumn_New (table, "value");
  AutoRes_Table_addRealColumn (table, valueColumn);

  valuesColumn = AutoRes_Real1DColumn_New (table, "values", 3);
  AutoRes_Table_addReal1DColumn (table, valuesColumn);

  collection = AutoRes_UseFlagCollection_New (table);
  assert(AutoRes_UseFlagCollection_table (collection) == table);

  flag = AutoRes_UseFlagCollection_makeColumn (collection);
  assert(flag);
  assert(AutoRes_Table_nColumns (table) == 3); 
  


  flag = AutoRes_Table_allocateRecords (table, 5);
  assert(flag);
  
  AutoRes_UseFlagCollection_initializeColumn (collection);

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 5);
  assert(nReservedRecords == nRecords);

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    AUTORES_R_SET(valueColumn, iRecord, iRecord * 10.0);
    AUTORES_R1D_SET(valuesColumn, iRecord, 0, iRecord * 10.0 + 1.0);
    AUTORES_R1D_SET(valuesColumn, iRecord, 1, iRecord * 10.0 + 2.0);
    AUTORES_R1D_SET(valuesColumn, iRecord, 2, iRecord * 10.0 + 3.0);
  }
  
  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 0));
  assert(EQ(AUTORES_R(valueColumn, 0), 0.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 0, 0), 1.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 0, 1), 2.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 0, 2), 3.0));
  
  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 1));
  assert(EQ(AUTORES_R(valueColumn, 1), 10.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 1, 0), 11.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 1, 1), 12.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 1, 2), 13.0));
  
  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 2));
  assert(EQ(AUTORES_R(valueColumn, 2), 20.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 2, 0), 21.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 2, 1), 22.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 2, 2), 23.0));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 3));
  assert(EQ(AUTORES_R(valueColumn, 3), 30.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 3, 0), 31.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 3, 1), 32.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 3, 2), 33.0));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 4));
  assert(EQ(AUTORES_R(valueColumn, 4), 40.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 4, 0), 41.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 4, 1), 42.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 4, 2), 43.0));



  for (iRecord = 5; iRecord < 7; iRecord++) {
    int newRecordId = AutoRes_UseFlagCollection_addRecord (collection);

    assert(newRecordId != -1);
    AUTORES_R_SET(valueColumn, newRecordId, iRecord * 10.0);
    AUTORES_R1D_SET(valuesColumn, newRecordId, 0, iRecord * 10.0 + 1.0);
    AUTORES_R1D_SET(valuesColumn, newRecordId, 1, iRecord * 10.0 + 2.0);
    AUTORES_R1D_SET(valuesColumn, newRecordId, 2, iRecord * 10.0 + 3.0);
  }

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 7);
  assert(nRecords <= nReservedRecords);

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 0));
  assert(EQ(AUTORES_R(valueColumn, 0), 0.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 0, 0), 1.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 0, 1), 2.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 0, 2), 3.0));
  
  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 1));
  assert(EQ(AUTORES_R(valueColumn, 1), 10.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 1, 0), 11.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 1, 1), 12.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 1, 2), 13.0));
  
  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 2));
  assert(EQ(AUTORES_R(valueColumn, 2), 20.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 2, 0), 21.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 2, 1), 22.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 2, 2), 23.0));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 3));
  assert(EQ(AUTORES_R(valueColumn, 3), 30.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 3, 0), 31.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 3, 1), 32.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 3, 2), 33.0));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 4));
  assert(EQ(AUTORES_R(valueColumn, 4), 40.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 4, 0), 41.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 4, 1), 42.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 4, 2), 43.0));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 5));
  assert(EQ(AUTORES_R(valueColumn, 5), 50.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 5, 0), 51.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 5, 1), 52.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 5, 2), 53.0));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 6));
  assert(EQ(AUTORES_R(valueColumn, 6), 60.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 6, 0), 61.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 6, 1), 62.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 6, 2), 63.0));



  AutoRes_UseFlagCollection_removeRecord (collection, 0);
  AutoRes_UseFlagCollection_removeRecord (collection, 1);
  AutoRes_UseFlagCollection_removeRecord (collection, 2);

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 7);
  assert(nRecords <= nReservedRecords);

  assert(!AutoRes_UseFlagCollection_recordIsUsed (collection, 0));
  
  assert(!AutoRes_UseFlagCollection_recordIsUsed (collection, 1));
  
  assert(!AutoRes_UseFlagCollection_recordIsUsed (collection, 2));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 3));
  assert(EQ(AUTORES_R(valueColumn, 3), 30.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 3, 0), 31.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 3, 1), 32.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 3, 2), 33.0));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 4));
  assert(EQ(AUTORES_R(valueColumn, 4), 40.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 4, 0), 41.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 4, 1), 42.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 4, 2), 43.0));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 5));
  assert(EQ(AUTORES_R(valueColumn, 5), 50.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 5, 0), 51.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 5, 1), 52.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 5, 2), 53.0));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 6));
  assert(EQ(AUTORES_R(valueColumn, 6), 60.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 6, 0), 61.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 6, 1), 62.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 6, 2), 63.0));



  AutoRes_UseFlagCollection_pack (collection);

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  assert(nRecords == 4);
  assert(nRecords <= nReservedRecords);

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 0));
  assert(EQ(AUTORES_R(valueColumn, 0), 30.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 0, 0), 31.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 0, 1), 32.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 0, 2), 33.0));
  
  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 1));
  assert(EQ(AUTORES_R(valueColumn, 1), 40.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 1, 0), 41.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 1, 1), 42.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 1, 2), 43.0));
  
  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 2));
  assert(EQ(AUTORES_R(valueColumn, 2), 50.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 2, 0), 51.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 2, 1), 52.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 2, 2), 53.0));

  assert(AutoRes_UseFlagCollection_recordIsUsed (collection, 3));
  assert(EQ(AUTORES_R(valueColumn, 3), 60.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 3, 0), 61.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 3, 1), 62.0));
  assert(EQ(AUTORES_R1D(valuesColumn, 3, 2), 63.0));



  AutoRes_UseFlagCollection_delete (collection);
  AutoRes_Table_delete (table);
}

int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);

  Test0 ();
  Test1 ();

  return 0;
}

