/* sample_bucket2d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores.h"



/* add / remove point-like objects into a bucket */
static void Test0 (void)
{
  AutoRes_Bucket2D *bucket;
  int nAverageLinksPerCell = 10;
  int cx, cy;
  int linkId;
  int flag;



  printf (" TestBucket2D 0 \n"); 

  bucket = AutoRes_Bucket2D_New (0.0, 0.0, 
				 10.0, 10.0);
  
  flag = AutoRes_Bucket2D_reserve 
    (bucket, 10, 10, nAverageLinksPerCell);
  assert(flag);



  printf (" <add> \n");

  AutoRes_Bucket2D_addObjectAt (bucket, 0.5, 1.0, 101);
  AutoRes_Bucket2D_addObjectAt (bucket, 0.5, 2.5, 102);
  AutoRes_Bucket2D_addObjectAt (bucket, 1.5, 1.5, 103);
  AutoRes_Bucket2D_addObjectAt (bucket, 0.5, 1.5, 104);
  AutoRes_Bucket2D_addObjectAt (bucket, 1.0, 1.0, 105);

  for (cx = 0; cx < 3; cx++) {
    for (cy = 0; cy < 3; cy++) {
      printf (" cell %d %d \t", cx, cy);

      for (linkId = AutoRes_Bucket2D_startLinkIdIn (bucket, cx, cy);
	   linkId != -1;
	   linkId = AutoRes_Bucket2D_nextLinkId (bucket, linkId)) {
	int objectId = AutoRes_Bucket2D_objectId (bucket, linkId);
	
	printf (" %d ", objectId);
      }

      printf ("\n");
    }
  }

  printf (" find 103 at (1.5, 1.5) from ");
  AutoRes_Bucket2D_getCellIdAt (bucket, 1.5, 1.5,
				&cx, &cy);
  
  for (linkId = AutoRes_Bucket2D_startLinkIdIn (bucket, cx, cy);
       linkId != -1;
       linkId = AutoRes_Bucket2D_nextLinkId (bucket, linkId)) {
    int objectId = AutoRes_Bucket2D_objectId (bucket, linkId);
    
    printf (" %d ", objectId);
  }
  printf ("\n");

  printf (" find 105 at (1.0, 1.0) from ");
  AutoRes_Bucket2D_getCellIdAt (bucket, 1.0, 1.0,
				&cx, &cy);

  for (linkId = AutoRes_Bucket2D_startLinkIdIn (bucket, cx, cy);
       linkId != -1;
       linkId = AutoRes_Bucket2D_nextLinkId (bucket, linkId)) {
    int objectId = AutoRes_Bucket2D_objectId (bucket, linkId);
    
    printf (" %d ", objectId);
  }
  printf ("\n");



  printf (" <remove> \n");

  AutoRes_Bucket2D_removeObjectAt (bucket, 0.5, 1.0, 101);
  AutoRes_Bucket2D_removeObjectAt (bucket, 0.5, 1.5, 104);
  AutoRes_Bucket2D_removeObjectAt (bucket, 1.0, 1.0, 105);

  for (cx = 0; cx < 3; cx++) {
    for (cy = 0; cy < 3; cy++) {
      printf (" cell %d %d \t", cx, cy);

      for (linkId = AutoRes_Bucket2D_startLinkIdIn (bucket, cx, cy);
	   linkId != -1;
	   linkId = AutoRes_Bucket2D_nextLinkId (bucket, linkId)) {
	int objectId = AutoRes_Bucket2D_objectId (bucket, linkId);
	
	printf (" %d ", objectId);
      }

      printf ("\n");
    }
  }



  AutoRes_Bucket2D_delete (bucket);
}

/* add / remove volumetric objects into a bucket */
static void Test1 (void)
{
  AutoRes_Bucket2D *bucket;
  int nAverageLinksPerCell = 10;
  int cx0, cy0;
  int cx1, cy1;
  int outOfRangeExists;
  int cx, cy;
  int linkId;
  int flag;
  


  printf (" TestBucket2D 1 \n"); 

  bucket = AutoRes_Bucket2D_New (0.0, 0.0, 
				 10.0, 10.0);
  
  flag = AutoRes_Bucket2D_reserve 
    (bucket, 10, 10, nAverageLinksPerCell);
  assert(flag);



  printf (" <add> \n");

  AutoRes_Bucket2D_addObjectWithin 
    (bucket, 
     0.5, 1.5, 
     1.5, 2.5, 
     101);
  AutoRes_Bucket2D_addObjectWithin 
    (bucket,
     0.5, 0.5, 
     1.5, 0.5, 
     102);
  AutoRes_Bucket2D_addObjectWithin 
    (bucket,
     -0.5, -0.5, 
     2.5, 2.5, 
     103);
  AutoRes_Bucket2D_addObjectWithin 
    (bucket,
     0.5, 1.5, 
     0.5, 1.5, 
     104);
  AutoRes_Bucket2D_addObjectWithin 
    (bucket,
     1.0, 1.0, 
     1.0, 1.0, 
     105);
  AutoRes_Bucket2D_addObjectWithin 
    (bucket,
     10.5, 10.5, 
     12.5, 12.5, 
     106);

  AutoRes_Bucket2D_getCellIdWithin 
    (bucket, 
     0.0, 0.0,
     2.0, 2.0,
     &cx0, &cy0, 
     &cx1, &cy1);

  outOfRangeExists = 0;
  for (cx = cx0; cx <= cx1; cx++) {
    for (cy = cy0; cy <= cy1; cy++) {
      if (AutoRes_Bucket2D_isOutOfRange (bucket, cx, cy)) {
	outOfRangeExists = 1;
      } else {
	printf (" cell %d %d \t", cx, cy);

	for (linkId = AutoRes_Bucket2D_startLinkIdIn (bucket, cx, cy);
	     linkId != -1;
	     linkId = AutoRes_Bucket2D_nextLinkId (bucket, linkId)) {
	  int objectId = AutoRes_Bucket2D_objectId (bucket, linkId);

	  printf (" %d ", objectId);
	}

	printf ("\n");
      }
    }
  }
  if (outOfRangeExists) {
    printf (" cell out of range \t");

    for (linkId = AutoRes_Bucket2D_startLinkIdOutOfRange (bucket);
	 linkId != -1;
	 linkId = AutoRes_Bucket2D_nextLinkId (bucket, linkId)) {
      int objectId = AutoRes_Bucket2D_objectId (bucket, linkId);
      
      printf (" %d ", objectId);
    }

    printf ("\n");
  }



  printf (" <remove> \n");

  AutoRes_Bucket2D_removeObjectWithin 
    (bucket,
     -0.5, -0.5, 
     2.5, 2.5, 
     103);
  AutoRes_Bucket2D_removeObjectWithin 
    (bucket,
     0.5, 1.5, 
     0.5, 1.5, 
     104);
  AutoRes_Bucket2D_removeObjectWithin 
    (bucket,
     10.5, 10.5, 
     12.5, 12.5, 
     106);

  AutoRes_Bucket2D_getCellIdWithin 
    (bucket, 
     0.0, 0.0,
     2.0, 2.0,
     &cx0, &cy0, 
     &cx1, &cy1);

  outOfRangeExists = 0;
  for (cx = cx0; cx <= cx1; cx++) {
    for (cy = cy0; cy <= cy1; cy++) {
      if (AutoRes_Bucket2D_isOutOfRange (bucket, cx, cy)) {
	outOfRangeExists = 1;
      } else {
	printf (" cell %d %d \t", cx, cy);

	for (linkId = AutoRes_Bucket2D_startLinkIdIn (bucket, cx, cy);
	     linkId != -1;
	     linkId = AutoRes_Bucket2D_nextLinkId (bucket, linkId)) {
	  int objectId = AutoRes_Bucket2D_objectId (bucket, linkId);

	  printf (" %d ", objectId);
	}

	printf ("\n");
      }
    }
  }
  if (outOfRangeExists) {
    printf (" cell out of range \t");

    for (linkId = AutoRes_Bucket2D_startLinkIdOutOfRange (bucket);
	 linkId != -1;
	 linkId = AutoRes_Bucket2D_nextLinkId (bucket, linkId)) {
      int objectId = AutoRes_Bucket2D_objectId (bucket, linkId);
      
      printf (" %d ", objectId);
    }

    printf ("\n");
  }



  AutoRes_Bucket2D_delete (bucket);
}

int main (int argc, char* argv[])
{
  AutoRes_Initialize (0);

  Test0 ();
  Test1 ();

  return 0;
}

