/* sample_bucket3d.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores.h"



/* add / remove point-like objects into a bucket */
static void Test0 (void)
{
  AutoRes_Bucket3D *bucket;
  int nAverageLinksPerCell = 10;
  int cx, cy, cz;
  int linkId;
  int flag;



  printf (" TestBucket3D 0 \n"); 

  bucket = AutoRes_Bucket3D_New (0.0, 0.0, 0.0, 
				 10.0, 10.0, 10.0);
  
  flag = AutoRes_Bucket3D_reserve 
    (bucket, 10, 10, 10, nAverageLinksPerCell);
  assert(flag);



  printf (" <add> \n");

  AutoRes_Bucket3D_addObjectAt (bucket, 0.5, 1.0, 0.5, 101);
  AutoRes_Bucket3D_addObjectAt (bucket, 0.5, 2.5, 1.5, 102);
  AutoRes_Bucket3D_addObjectAt (bucket, 1.5, 1.5, 1.5, 103);
  AutoRes_Bucket3D_addObjectAt (bucket, 0.5, 1.5, 0.5, 104);
  AutoRes_Bucket3D_addObjectAt (bucket, 1.0, 1.0, 1.0, 105);

  for (cx = 0; cx < 3; cx++) {
    for (cy = 0; cy < 3; cy++) {
      for (cz = 0; cz < 3; cz++) {
	printf (" cell %d %d %d \t", cx, cy, cz);
	
	for (linkId = AutoRes_Bucket3D_startLinkIdIn (bucket, cx, cy, cz);
	     linkId != -1;
	     linkId = AutoRes_Bucket3D_nextLinkId (bucket, linkId)) {
	  int objectId = AutoRes_Bucket3D_objectId (bucket, linkId);
	  
	  printf (" %d ", objectId);
	}
	
	printf ("\n");
      }
    }
  }

  printf (" find 103 at (1.5, 1.5, 1.5) from ");
  AutoRes_Bucket3D_getCellIdAt (bucket, 1.5, 1.5, 1.5, 
				&cx, &cy, &cz);

  for (linkId = AutoRes_Bucket3D_startLinkIdIn (bucket, cx, cy, cz);
       linkId != -1;
       linkId = AutoRes_Bucket3D_nextLinkId (bucket, linkId)) {
    int objectId = AutoRes_Bucket3D_objectId (bucket, linkId);
    
    printf (" %d ", objectId);
  }
  printf ("\n");

  printf (" find 105 at (1.0, 1.0, 1.0) from ");
  AutoRes_Bucket3D_getCellIdAt (bucket, 1.0, 1.0, 1.0,
				&cx, &cy, &cz);

  for (linkId = AutoRes_Bucket3D_startLinkIdIn (bucket, cx, cy, cz);
       linkId != -1;
       linkId = AutoRes_Bucket3D_nextLinkId (bucket, linkId)) {
    int objectId = AutoRes_Bucket3D_objectId (bucket, linkId);
    
    printf (" %d ", objectId);
  }
  printf ("\n");



  printf (" <remove> \n");

  AutoRes_Bucket3D_removeObjectAt (bucket, 0.5, 1.0, 0.5, 101);
  AutoRes_Bucket3D_removeObjectAt (bucket, 0.5, 1.5, 0.5, 104);
  AutoRes_Bucket3D_removeObjectAt (bucket, 1.0, 1.0, 1.0, 105);

  for (cx = 0; cx < 3; cx++) {
    for (cy = 0; cy < 3; cy++) {
      for (cz = 0; cz < 3; cz++) {
	printf (" cell %d %d %d \t", cx, cy, cz);

	for (linkId = AutoRes_Bucket3D_startLinkIdIn (bucket, cx, cy, cz);
	     linkId != -1;
	     linkId = AutoRes_Bucket3D_nextLinkId (bucket, linkId)) {
	  int objectId = AutoRes_Bucket3D_objectId (bucket, linkId);
	  
	  printf (" %d ", objectId);
	}
	
	printf ("\n");
      }
    }
  }



  AutoRes_Bucket3D_delete (bucket);
}

/* add / remove volumetric objects into a bucket */
static void Test1 (void)
{
  AutoRes_Bucket3D *bucket;
  int nAverageLinksPerCell = 10;
  int cx0, cy0, cz0;
  int cx1, cy1, cz1;
  int outOfRangeExists;
  int cx, cy, cz;
  int linkId;
  int flag;
  


  printf (" TestBucket3D 1 \n"); 

  bucket = AutoRes_Bucket3D_New (0.0, 0.0, 0.0,
				 10.0, 10.0, 10.0);
  
  flag = AutoRes_Bucket3D_reserve 
    (bucket, 10, 10, 10, nAverageLinksPerCell);
  assert(flag);



  printf (" <add> \n");

  AutoRes_Bucket3D_addObjectWithin 
    (bucket,
     0.5, 1.5, 0.5, 
     1.5, 2.5, 0.5, 
     101);
  AutoRes_Bucket3D_addObjectWithin 
    (bucket,
     0.5, 0.5, 1.5, 
     1.5, 0.5, 1.5, 
     102);
  AutoRes_Bucket3D_addObjectWithin 
    (bucket,
     -0.5, -0.5, -0.5, 
     2.5, 2.5, 2.5, 
     103);
  AutoRes_Bucket3D_addObjectWithin 
    (bucket,
     0.5, 1.5, 0.5,
     0.5, 1.5, 0.5, 
     104);
  AutoRes_Bucket3D_addObjectWithin 
    (bucket,
     1.0, 1.0, 1.0,
     1.0, 1.0, 1.0,
     105);
  AutoRes_Bucket3D_addObjectWithin 
    (bucket,
     10.5, 10.5, 10.5,
     12.5, 12.5, 12.5, 
     106);

  AutoRes_Bucket3D_getCellIdWithin 
    (bucket, 
     0.0, 0.0, 0.0,
     2.0, 2.0, 2.0, 
     &cx0, &cy0, &cz0, 
     &cx1, &cy1, &cz1);

  outOfRangeExists = 0;
  for (cx = cx0; cx <= cx1; cx++) {
    for (cy = cy0; cy <= cy1; cy++) {
      for (cz = cz0; cz <= cz1; cz++) {
	if (AutoRes_Bucket3D_isOutOfRange (bucket, cx, cy, cz)) {
	  outOfRangeExists = 1;
	} else {
	  printf (" cell %d %d %d \t", cx, cy, cz);
	  
	  for (linkId = AutoRes_Bucket3D_startLinkIdIn (bucket, cx, cy, cz);
	       linkId != -1;
	       linkId = AutoRes_Bucket3D_nextLinkId (bucket, linkId)) {
	    int objectId = AutoRes_Bucket3D_objectId (bucket, linkId);
	    
	    printf (" %d ", objectId);
	  }
	  
	  printf ("\n");
	}
      }
    }
  }
  if (outOfRangeExists) {
    printf (" cell out of range \t");

    for (linkId = AutoRes_Bucket3D_startLinkIdOutOfRange (bucket);
	 linkId != -1;
	 linkId = AutoRes_Bucket3D_nextLinkId (bucket, linkId)) {
      int objectId = AutoRes_Bucket3D_objectId (bucket, linkId);
      
      printf (" %d ", objectId);
    }

    printf ("\n");
  }



  printf (" <remove> \n");

  AutoRes_Bucket3D_removeObjectWithin 
    (bucket,
     -0.5, -0.5, -0.5, 
     2.5, 2.5, 2.5, 
     103);
  AutoRes_Bucket3D_removeObjectWithin 
    (bucket,
     0.5, 1.5, 0.5, 
     0.5, 1.5, 0.5, 
     104);
  AutoRes_Bucket3D_removeObjectWithin 
    (bucket,
     10.5, 10.5, 10.5,
     12.5, 12.5, 12.5, 
     106);

  AutoRes_Bucket3D_getCellIdWithin 
    (bucket, 
     0.0, 0.0, 0.0,
     2.0, 2.0, 2.0, 
     &cx0, &cy0, &cz0, 
     &cx1, &cy1, &cz1);

  outOfRangeExists = 0;
  for (cx = cx0; cx <= cx1; cx++) {
    for (cy = cy0; cy <= cy1; cy++) {
      for (cz = cz0; cz <= cz1; cz++) {
	if (AutoRes_Bucket3D_isOutOfRange (bucket, cx, cy, cz)) {
	  outOfRangeExists = 1;
	} else {
	  printf (" cell %d %d %d \t", cx, cy, cz);

	  for (linkId = AutoRes_Bucket3D_startLinkIdIn (bucket, cx, cy, cz);
	       linkId != -1;
	       linkId = AutoRes_Bucket3D_nextLinkId (bucket, linkId)) {
	    int objectId = AutoRes_Bucket3D_objectId (bucket, linkId);
	    
	    printf (" %d ", objectId);
	  }
	  
	  printf ("\n");
	}
      }
    }
  }
  if (outOfRangeExists) {
    printf (" cell out of range \t");

    for (linkId = AutoRes_Bucket3D_startLinkIdOutOfRange (bucket);
	 linkId != -1;
	 linkId = AutoRes_Bucket3D_nextLinkId (bucket, linkId)) {
      int objectId = AutoRes_Bucket3D_objectId (bucket, linkId);
      
      printf (" %d ", objectId);
    }

    printf ("\n");
  }



  AutoRes_Bucket3D_delete (bucket);
}

int main (int argc, char* argv[])
{
  AutoRes_Initialize (0);

  Test0 ();
  Test1 ();

  return 0;
}

