/* sample_file.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores.h"



#define MAX_NODES 10000
int NNodes;
double Node_coord[MAX_NODES][3];
int Node_boundaryFlag[MAX_NODES];
int IntegerValue;
double RealValue;

static void ClearNode (void)
{
  int iNode;
  int iDof;

  NNodes = 0;
  for (iNode = 0; iNode < MAX_NODES; iNode++) {
    for (iDof = 0; iDof < 3; iDof++) {
      Node_coord[iNode][iDof] = 0.0;
    }
  }
  for (iNode = 0; iNode < MAX_NODES; iNode++) {
    Node_boundaryFlag[iNode] = 0;
  }

  IntegerValue = 0;
  RealValue = 0.0;
}

static void InitializeNode (void)
{
  NNodes = 3;

  Node_coord[0][0] = 0.0;
  Node_coord[0][1] = 1.0;
  Node_coord[0][2] = 2.0;

  Node_coord[1][0] = 10.0;
  Node_coord[1][1] = 11.0;
  Node_coord[1][2] = 12.0;

  Node_coord[2][0] = 100.0;
  Node_coord[2][1] = 101.0;
  Node_coord[2][2] = 102.0;

  Node_boundaryFlag[0] = 0;
  Node_boundaryFlag[1] = 1;
  Node_boundaryFlag[2] = 1;

  IntegerValue = 10;
  RealValue = 0.3;
}

static void CheckNode (void)
{
  assert(NNodes == 3);

  assert(fabs (Node_coord[0][0] - 0.0) < 0.000001);
  assert(fabs (Node_coord[0][1] - 1.0) < 0.000001);
  assert(fabs (Node_coord[0][2] - 2.0) < 0.000001);

  assert(fabs (Node_coord[1][0] - 10.0) < 0.000001);
  assert(fabs (Node_coord[1][1] - 11.0) < 0.000001);
  assert(fabs (Node_coord[1][2] - 12.0) < 0.000001);

  assert(fabs (Node_coord[2][0] - 100.0) < 0.000001);
  assert(fabs (Node_coord[2][1] - 101.0) < 0.000001);
  assert(fabs (Node_coord[2][2] - 102.0) < 0.000001);

  assert(Node_boundaryFlag[0] == 0);
  assert(Node_boundaryFlag[1] == 1);
  assert(Node_boundaryFlag[2] == 1);

  assert(IntegerValue == 10);
  assert(fabs (RealValue - 0.3) < 0.000001);
}

static void PrintNode (void)
{
  int iNode;

  printf (" NNodes %d \n",
	  NNodes); 
  printf (" IntegerValue %d RealValue %f \n",
	  IntegerValue, RealValue); 
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    double z = Node_coord[iNode][2];
    int boundaryFlag = Node_boundaryFlag[iNode];
    
    printf (" node %d ",
	    iNode); 
    printf (" coord ( %f %f %f ) ",
	    x, y, z); 
    printf (" boundaryFlag %d ",
	    boundaryFlag); 
    printf ("\n"); 
  }
}

static void FileNode (const char *fileName, int writeFlag, int binaryFlag)
{
  AutoRes_File *file;
  int iNode;
  int iDof;

  file = AutoRes_File_New ();

#if 0
  AutoRes_File_setDefaultIntegerColumnWidth (file, 8);
  AutoRes_File_setDefaultRealColumnWidth (file, 16);
#endif

  AutoRes_File_open (file, fileName, writeFlag, binaryFlag);

  /* header section */

  AutoRes_File_integer (file, &NNodes);
  AutoRes_File_nextLine (file);

  AutoRes_File_integer (file, &IntegerValue);
  AutoRes_File_real (file, &RealValue);
  AutoRes_File_nextLine (file);

  /* record section */
  for (iNode = 0; iNode < NNodes; iNode++) {
    for (iDof = 0; iDof < 3; iDof++) {
      AutoRes_File_real (file, &Node_coord[iNode][iDof]);
    }
    AutoRes_File_integer (file, &Node_boundaryFlag[iNode]);
    AutoRes_File_nextLine (file);
  }

  AutoRes_File_close (file);

  AutoRes_File_delete (file);
}

static void TestFile (void)
{
  int writeFlag;
  int binaryFlag;

  printf (" TestFile \n"); 

  InitializeNode ();
  PrintNode ();



  /* text */

  binaryFlag = 0;

  writeFlag = 1;
  FileNode ("node.txt", writeFlag, binaryFlag);

  ClearNode ();
  writeFlag = 0;
  FileNode ("node.txt", writeFlag, binaryFlag);

  PrintNode ();
  CheckNode ();



  /* binary */

  binaryFlag = 1;

  writeFlag = 1;
  FileNode ("node.bin", writeFlag, binaryFlag);

  ClearNode ();
  writeFlag = 0;
  FileNode ("node.bin", writeFlag, binaryFlag);

  PrintNode ();
  CheckNode ();
}



int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);

  TestFile ();

  return 0;
}



