/* sample_free_list.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores.h"



static void TestFreeListCollection (void)
{
  AutoRes_Table *table;
  AutoRes_RealColumn *valueColumn;
  AutoRes_FreeListCollection *collection;
  int nRecords;
  int nReservedRecords;
  int iRecord;
  int i;

  printf (" TestFreeListCollection \n"); 

  table = AutoRes_Table_New ("table");

  valueColumn = AutoRes_RealColumn_New (table, "value");
  AutoRes_Table_addRealColumn (table, valueColumn);

  collection = AutoRes_FreeListCollection_New (table);
  AutoRes_FreeListCollection_makeColumn (collection);
  
  printf (" nColumns %d \n",
	  AutoRes_Table_nColumns (table)); 
  


  AutoRes_Table_reserveRecords (table, 3);

  for (i = 0; i < 5; i++) {
    int newRecordId = AutoRes_FreeListCollection_addRecord (collection);

    AUTORES_R_SET(valueColumn, newRecordId, i * 10.0);
  }
  
  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  printf (" nRecords %d nReservedRecords %d \n",
	  nRecords, nReservedRecords);

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    if (AutoRes_FreeListCollection_recordIsUsed (collection, iRecord)) {
      double value = AUTORES_R(valueColumn, iRecord);

      printf (" record %d value = %f \n",
	      iRecord, value); 
    } else {
      printf (" record %d is not used \n",
	      iRecord); 
    }
  }



  AutoRes_FreeListCollection_removeRecord (collection, 1);
  AutoRes_FreeListCollection_removeRecord (collection, 2);
  AutoRes_FreeListCollection_removeRecord (collection, 3);

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  printf (" nRecords %d nReservedRecords %d \n",
	  nRecords, nReservedRecords);

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    if (AutoRes_FreeListCollection_recordIsUsed (collection, iRecord)) {
      double value = AUTORES_R(valueColumn, iRecord);
      
      printf (" record %d value = %f \n",
	      iRecord, value); 
    } else {
      printf (" record %d is not used \n",
	      iRecord); 
    }
  }



  for (i = 5; i < 7; i++) {
    int newRecordId = AutoRes_FreeListCollection_addRecord (collection);

    AUTORES_R_SET(valueColumn, newRecordId, i * 10.0);
  }

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  printf (" nRecords %d nReservedRecords %d \n",
	  nRecords, nReservedRecords);

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    if (AutoRes_FreeListCollection_recordIsUsed (collection, iRecord)) {
      double value = AUTORES_R(valueColumn, iRecord);
      
      printf (" record %d value = %f \n",
	      iRecord, value); 
    } else {
      printf (" record %d is not used \n",
	      iRecord); 
    }
  }



  AutoRes_FreeListCollection_pack (collection);

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  printf (" nRecords %d nReservedRecords %d \n",
	  nRecords, nReservedRecords);

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    if (AutoRes_FreeListCollection_recordIsUsed (collection, iRecord)) {
      double value = AUTORES_R(valueColumn, iRecord);

      printf (" record %d value = %f \n",
	      iRecord, value); 
    } else {
      printf (" record %d is not used \n",
	      iRecord); 
    }
  }



  AutoRes_FreeListCollection_removeRecord (collection, 0);
  AutoRes_FreeListCollection_removeRecord (collection, 2);

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  printf (" nRecords %d nReservedRecords %d \n",
	  nRecords, nReservedRecords);

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    if (AutoRes_FreeListCollection_recordIsUsed (collection, iRecord)) {
      double value = AUTORES_R(valueColumn, iRecord);
      
      printf (" record %d value = %f \n",
	      iRecord, value); 
    } else {
      printf (" record %d is not used \n",
	      iRecord); 
    }
  }



  for (i = 7; i < 12; i++) {
    int newRecordId = AutoRes_FreeListCollection_addRecord (collection);

    AUTORES_R_SET(valueColumn, newRecordId, i * 10.0);
  }

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  printf (" nRecords %d nReservedRecords %d \n",
	  nRecords, nReservedRecords);

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    if (AutoRes_FreeListCollection_recordIsUsed (collection, iRecord)) {
      double value = AUTORES_R(valueColumn, iRecord);
      
      printf (" record %d value = %f \n",
	      iRecord, value); 
    } else {
      printf (" record %d is not used \n",
	      iRecord); 
    }
  }



  AutoRes_FreeListCollection_delete (collection);
  AutoRes_Table_delete (table);
}



int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);

  TestFreeListCollection ();

  return 0;
}

