/* sample_io.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores.h"



#define MAX_NODES 10000
int NNodes;
double Node_coord[MAX_NODES][3];
int Node_boundaryFlag[MAX_NODES];
int IntegerValue;
double RealValue;

static void ClearNode (void)
{
  int iNode;
  int iDof;

  NNodes = 0;
  for (iNode = 0; iNode < MAX_NODES; iNode++) {
    for (iDof = 0; iDof < 3; iDof++) {
      Node_coord[iNode][iDof] = 0.0;
    }
  }
  for (iNode = 0; iNode < MAX_NODES; iNode++) {
    Node_boundaryFlag[iNode] = 0;
  }

  IntegerValue = 0;
  RealValue = 0.0;
}

static void InitializeNode (void)
{
  NNodes = 3;

  Node_coord[0][0] = 0.0;
  Node_coord[0][1] = 1.0;
  Node_coord[0][2] = 2.0;

  Node_coord[1][0] = 10.0;
  Node_coord[1][1] = 11.0;
  Node_coord[1][2] = 12.0;

  Node_coord[2][0] = 100.0;
  Node_coord[2][1] = 101.0;
  Node_coord[2][2] = 102.0;

  Node_boundaryFlag[0] = 0;
  Node_boundaryFlag[1] = 1;
  Node_boundaryFlag[2] = 1;

  IntegerValue = 10;
  RealValue = 0.3;
}

static void CheckNode (void)
{
  assert(NNodes == 3);

  assert(fabs (Node_coord[0][0] - 0.0) < 0.000001);
  assert(fabs (Node_coord[0][1] - 1.0) < 0.000001);
  assert(fabs (Node_coord[0][2] - 2.0) < 0.000001);

  assert(fabs (Node_coord[1][0] - 10.0) < 0.000001);
  assert(fabs (Node_coord[1][1] - 11.0) < 0.000001);
  assert(fabs (Node_coord[1][2] - 12.0) < 0.000001);

  assert(fabs (Node_coord[2][0] - 100.0) < 0.000001);
  assert(fabs (Node_coord[2][1] - 101.0) < 0.000001);
  assert(fabs (Node_coord[2][2] - 102.0) < 0.000001);

  assert(Node_boundaryFlag[0] == 0);
  assert(Node_boundaryFlag[1] == 1);
  assert(Node_boundaryFlag[2] == 1);

  assert(IntegerValue == 10);
  assert(fabs (RealValue - 0.3) < 0.000001);
}

static void PrintNode (void)
{
  int iNode;

  printf (" NNodes %d \n",
	  NNodes); 
  printf (" IntegerValue %d RealValue %f \n",
	  IntegerValue, RealValue); 
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    double x = Node_coord[iNode][0];
    double y = Node_coord[iNode][1];
    double z = Node_coord[iNode][2];
    int boundaryFlag = Node_boundaryFlag[iNode];
    
    printf (" node %d ",
	    iNode); 
    printf (" coord ( %f %f %f ) ",
	    x, y, z); 
    printf (" boundaryFlag %d ",
	    boundaryFlag); 
    printf ("\n"); 
  }
}

static void FileNode (const char *fileName, int writeFlag, int binaryFlag)
{
  AutoRes_InOut *inOut;
  AutoRes_IntegerField *integerValueField;
  AutoRes_RealField *realValueField;
  AutoRes_Real1DField *coordField;
  AutoRes_IntegerField *boundaryFlagField;
  int iNode;
  
  inOut = AutoRes_InOut_New ();
  
  integerValueField = AutoRes_IntegerField_New ("IntegerValue");
  AutoRes_InOut_addIntegerFieldIntoHeader 
    (inOut, integerValueField);

  realValueField = AutoRes_RealField_New ("RealValue");
  AutoRes_InOut_addRealFieldIntoHeader 
    (inOut, realValueField);
  
  coordField = AutoRes_Real1DField_New ("coord", 3);
  AutoRes_InOut_addReal1DFieldIntoRecord 
    (inOut, coordField);

  boundaryFlagField = AutoRes_IntegerField_New ("boundaryFlag");
  AutoRes_InOut_addIntegerFieldIntoRecord 
    (inOut, boundaryFlagField);
  
  if (writeFlag) {
    AutoRes_InOut_writeMetaData (inOut, "node.cdm");
  }



  AutoRes_InOut_open (inOut, fileName, writeFlag, binaryFlag);

  AutoRes_InOut_associateNRecords (inOut, &NNodes);
  AutoRes_IntegerField_associate (integerValueField, &IntegerValue);
  AutoRes_RealField_associate (realValueField, &RealValue);
  AutoRes_InOut_doHeaderSection (inOut);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    AutoRes_Real1DField_associate 
      (coordField, Node_coord[iNode]);
    AutoRes_IntegerField_associate 
      (boundaryFlagField, &Node_boundaryFlag[iNode]);
    AutoRes_InOut_doEachOfRecordSection (inOut, iNode);
  }

  AutoRes_InOut_close (inOut);

  AutoRes_InOut_delete (inOut);
}

static void MetaFileNode 
(const char *fileName, int writeFlag, int binaryFlag)
{
  AutoRes_InOut *inOut;
  AutoRes_IntegerField *integerValueField;
  AutoRes_RealField *realValueField;
  AutoRes_Real1DField *coordField;
  AutoRes_IntegerField *boundaryFlagField;
  int iNode;
  
  inOut = AutoRes_InOut_New ();
  
  AutoRes_InOut_readMetaData (inOut, "node.cdm");

  integerValueField = AutoRes_InOut_headerFieldOfName (inOut, "IntegerValue");
  realValueField = AutoRes_InOut_headerFieldOfName (inOut, "RealValue");
  
  coordField = AutoRes_InOut_recordFieldOfName (inOut, "coord");
  boundaryFlagField = AutoRes_InOut_recordFieldOfName (inOut, "boundaryFlag");
  
  AutoRes_InOut_open (inOut, fileName, writeFlag, binaryFlag);

  AutoRes_InOut_associateNRecords (inOut, &NNodes);
  AutoRes_IntegerField_associate (integerValueField, &IntegerValue);
  AutoRes_RealField_associate (realValueField, &RealValue);
  AutoRes_InOut_doHeaderSection (inOut);
  
  for (iNode = 0; iNode < NNodes; iNode++) {
    AutoRes_Real1DField_associate 
      (coordField, Node_coord[iNode]);
    AutoRes_IntegerField_associate 
      (boundaryFlagField, &Node_boundaryFlag[iNode]);
    AutoRes_InOut_doEachOfRecordSection (inOut, iNode);
  }

  AutoRes_InOut_close (inOut);

  AutoRes_InOut_delete (inOut);
}

static void TestInOut (void)
{
  int writeFlag;
  int binaryFlag = 0;

  printf (" TestInOut \n"); 

  InitializeNode ();
  PrintNode ();



  writeFlag = 1;
  FileNode ("node2.txt", writeFlag, binaryFlag);

  ClearNode ();
  writeFlag = 0;
  FileNode ("node2.txt", writeFlag, binaryFlag);

  PrintNode ();
  CheckNode ();



  writeFlag = 1;
  MetaFileNode ("node3.txt", writeFlag, binaryFlag);

  ClearNode ();
  writeFlag = 0;
  MetaFileNode ("node3.txt", writeFlag, binaryFlag);

  PrintNode ();
  CheckNode ();
}



int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);

  TestInOut ();

  return 0;
}
