/* sample_ordered.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores.h"



static void TestOrderedCollection (void)
{
  AutoRes_Table *table;
  AutoRes_RealColumn *valueColumn;
  AutoRes_OrderedCollection *collection;
  int nRecords;
  int nReservedRecords;
  int iRecord;
  int i;

  printf (" TestOrderedCollection \n"); 

  table = AutoRes_Table_New ("table");

  valueColumn = AutoRes_RealColumn_New (table, "value");
  AutoRes_Table_addRealColumn (table, valueColumn);

  collection = AutoRes_OrderedCollection_New (table);
  AutoRes_OrderedCollection_makeColumn (collection);
  
  printf (" nColumns %d \n",
	  AutoRes_Table_nColumns (table)); 
  


  AutoRes_Table_reserveRecords (table, 3);

  for (i = 0; i < 5; i++) {
    int newRecordId = AutoRes_OrderedCollection_addRecord (collection);

    AUTORES_R_SET(valueColumn, newRecordId, i * 10.0);
  }
  
  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  printf (" nRecords %d nReservedRecords %d \n",
	  nRecords, nReservedRecords);

  printf (" forward \n"); 
  for (iRecord = AutoRes_OrderedCollection_startRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_nextRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    printf (" record %d value = %f \n",
	    iRecord, value); 
  }

  printf (" backward \n"); 
  for (iRecord = AutoRes_OrderedCollection_endRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_prevRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    printf (" record %d value = %f \n",
	    iRecord, value); 
  }



  AutoRes_OrderedCollection_removeRecord (collection, 2);
  AutoRes_OrderedCollection_removeRecord (collection, 3);

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  printf (" nRecords %d nReservedRecords %d \n",
	  nRecords, nReservedRecords);

  printf (" forward \n"); 
  for (iRecord = AutoRes_OrderedCollection_startRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_nextRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    printf (" record %d value = %f \n",
	    iRecord, value); 
  }

  printf (" backward \n"); 
  for (iRecord = AutoRes_OrderedCollection_endRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_prevRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    printf (" record %d value = %f \n",
	    iRecord, value); 
  }



  for (i = 5; i < 7; i++) {
    int newRecordId = AutoRes_OrderedCollection_addRecord (collection);

    AUTORES_R_SET(valueColumn, newRecordId, i * 10.0);
  }

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  printf (" nRecords %d nReservedRecords %d \n",
	  nRecords, nReservedRecords);

  printf (" forward \n"); 
  for (iRecord = AutoRes_OrderedCollection_startRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_nextRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    printf (" record %d value = %f \n",
	    iRecord, value); 
  }

  printf (" backward \n"); 
  for (iRecord = AutoRes_OrderedCollection_endRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_prevRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    printf (" record %d value = %f \n",
	    iRecord, value); 
  }



  AutoRes_OrderedCollection_removeRecord (collection, 0);
  AutoRes_OrderedCollection_removeRecord (collection, 1);

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  printf (" nRecords %d nReservedRecords %d \n",
	  nRecords, nReservedRecords);

  printf (" forward \n"); 
  for (iRecord = AutoRes_OrderedCollection_startRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_nextRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    printf (" record %d value = %f \n",
	    iRecord, value); 
  }

  printf (" backward \n"); 
  for (iRecord = AutoRes_OrderedCollection_endRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_prevRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    printf (" record %d value = %f \n",
	    iRecord, value); 
  }



  for (iRecord = 7; iRecord < 12; iRecord++) {
    int newRecordId = AutoRes_OrderedCollection_addRecord (collection);

    AUTORES_R_SET(valueColumn, newRecordId, iRecord * 10.0);
  }

  nRecords = AutoRes_Table_nRecords (table);
  nReservedRecords = AutoRes_Table_nReservedRecords (table);

  printf (" nRecords %d nReservedRecords %d \n",
	  nRecords, nReservedRecords);

  printf (" forward \n"); 
  for (iRecord = AutoRes_OrderedCollection_startRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_nextRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    printf (" record %d value = %f \n",
	    iRecord, value); 
  }

  printf (" backward \n"); 
  for (iRecord = AutoRes_OrderedCollection_endRecordId (collection); 
       iRecord != -1; 
       iRecord = AutoRes_OrderedCollection_prevRecordId 
	 (collection, iRecord)) {
    double value = AUTORES_R(valueColumn, iRecord);
    
    printf (" record %d value = %f \n",
	    iRecord, value); 
  }



  AutoRes_OrderedCollection_delete (collection);
  AutoRes_Table_delete (table);
}



int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);

  TestOrderedCollection ();

  return 0;
}

