/* sample_table.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores.h"



static void TestTable0 (void)
{
  AutoRes_Table *tableA;
  AutoRes_IntegerColumn *columnA;
  AutoRes_Integer1DColumn *columnB;
  int nRecords;
  int iRecord;
  int sum;
  int index;

  printf (" TestTable0 \n"); 

  tableA = AutoRes_Table_New ("tableA");

  columnA = AutoRes_IntegerColumn_New (tableA, "columnA");
  AutoRes_Table_addIntegerColumn (tableA, columnA);

  printf (" nColumns %d \n",
	  AutoRes_Table_nColumns (tableA)); 
  
  nRecords = 10;
  AutoRes_Table_allocateRecords (tableA, nRecords);

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    AUTORES_I_SET(columnA, iRecord, iRecord * 10);
  }
  
  sum = 0;
  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    int value = AUTORES_I(columnA, iRecord);

    printf (" columnA[%d] = %d \n",
	    iRecord, value); 
    sum += value;
  }

  printf (" sum %d \n",
	  sum); 

  columnB = AutoRes_Integer1DColumn_New (tableA, "columnB", 3);
  AutoRes_Table_addInteger1DColumn (tableA, columnB);

  printf (" nColumns %d \n",
	  AutoRes_Table_nColumns (tableA)); 

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    for (index = 0; index < 3; index++) {
      AUTORES_I1D_SET(columnB, iRecord, index, 
		      iRecord * 100 + index);
    }
  }
  
  sum = 0;
  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    for (index = 0; index < 3; index++) {
      int value = AUTORES_I1D(columnB, iRecord, index);
      
      printf (" columnB[%d][%d] = %d \n",
	      iRecord, index, value); 
      if (index == 0) {
	sum += value;
      }
    }
  }

  printf (" sum %d \n",
	  sum); 

  AutoRes_Table_delete (tableA);
}

static void TestTable1 (void)
{
  AutoRes_Table *tableA;
  AutoRes_RealColumn *columnA;
  AutoRes_Real1DColumn *columnB;
  int iRecord;
  int nRecords;
  double sum;
  int index;

  printf (" TestTable1 \n"); 

  tableA = AutoRes_Table_New ("tableA");

  columnA = AutoRes_RealColumn_New (tableA, "columnA");
  AutoRes_Table_addRealColumn (tableA, columnA);

  printf (" nColumns %d \n",
	  AutoRes_Table_nColumns (tableA)); 

  nRecords = AutoRes_Table_nRecords (tableA);
  printf (" nRecords %d \n",
	  nRecords); 

  AutoRes_Table_reserveRecords (tableA, 2);

  for (iRecord = 0; iRecord < 10; iRecord++) {
    AutoRes_Table_addRecord (tableA);
    AUTORES_R_SET(columnA, iRecord, iRecord * 10.0 + 0.5);
  }
  
  nRecords = AutoRes_Table_nRecords (tableA);
  printf (" nRecords %d \n",
	  nRecords); 

  sum = 0.0;
  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    double value = AUTORES_R(columnA, iRecord);

    printf (" columnA[%d] = %f \n",
	    iRecord, value); 
    sum += value;
  }

  printf (" sum %f \n",
	  sum); 

  columnB = AutoRes_Real1DColumn_New (tableA, "columnB", 3);
  AutoRes_Table_addReal1DColumn (tableA, columnB);

  printf (" nColumns %d \n",
	  AutoRes_Table_nColumns (tableA)); 

  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    for (index = 0; index < 3; index++) {
      AUTORES_R1D_SET(columnB, iRecord, index, 
		      iRecord * 100.0 + index + 0.5);
    }
  }

  sum = 0.0;
  for (iRecord = 0; iRecord < nRecords; iRecord++) {
    for (index = 0; index < 3; index++) {
      double value = AUTORES_R1D(columnB, iRecord, index);
      
      printf (" columnB[%d][%d] = %f \n",
	      iRecord, index, value); 
      if (index == 0) {
	sum += value;
      }
    }
  }

  printf (" sum %f \n",
	  sum); 

  AutoRes_Table_delete (tableA);
}



int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);

  TestTable0 ();
  TestTable1 ();

  return 0;
}

