/* sample_table_io.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include "autores.h"



AutoRes_Table *NodeTable;
AutoRes_Real1DColumn *Node_coord;
AutoRes_IntegerColumn *Node_boundaryFlag;
int IntegerValue;
double RealValue;

static void PrintNode (void)
{
  int nNodes;
  int iNode;

  nNodes = AutoRes_Table_nRecords (NodeTable);

  printf (" nNodes %d \n",
	  nNodes); 
  printf (" IntegerValue %d RealValue %f \n",
	  IntegerValue, RealValue); 

  for (iNode = 0; iNode < nNodes; iNode++) {
    double x = AUTORES_R1D(Node_coord, iNode, 0);
    double y = AUTORES_R1D(Node_coord, iNode, 1);
    double z = AUTORES_R1D(Node_coord, iNode, 2);
    int boundaryFlag = AUTORES_I(Node_boundaryFlag, iNode);
    
    printf (" node %d ",
	    iNode); 
    printf (" coord ( %f %f %f ) ",
	    x, y, z); 
    printf (" boundaryFlag %d ",
	    boundaryFlag); 
    printf ("\n"); 
  }
}

static void FileNode 
(const char *fileName,
 int writeFlag,
 int binaryFlag)
{
  AutoRes_TableInOut *tableInOut;
  AutoRes_InOut *inOut;
  AutoRes_IntegerField *integerValueField;
  AutoRes_RealField *realValueField;
  AutoRes_Real1DField *coordField;
  AutoRes_IntegerField *boundaryFlagField;
  
  tableInOut = AutoRes_TableInOut_New (NodeTable);

  inOut = AutoRes_TableInOut_inOut (tableInOut);

  integerValueField = AutoRes_IntegerField_New ("IntegerValue");
  AutoRes_InOut_addIntegerFieldIntoHeader 
    (inOut, integerValueField);

  realValueField = AutoRes_RealField_New ("RealValue");
  AutoRes_InOut_addRealFieldIntoHeader 
    (inOut, realValueField);

  coordField = AutoRes_Real1DField_New ("coord", 3);
  AutoRes_InOut_addReal1DFieldIntoRecord 
    (inOut, coordField);

  boundaryFlagField = AutoRes_IntegerField_New ("boundaryFlag");
  AutoRes_InOut_addIntegerFieldIntoRecord 
    (inOut, boundaryFlagField);

  if (writeFlag) {
    AutoRes_InOut_writeMetaData (inOut, "node2.cdm");
  }



  AutoRes_IntegerField_associate (integerValueField, &IntegerValue);
  AutoRes_RealField_associate (realValueField, &RealValue);

  AutoRes_TableInOut_associateColumnWithField 
    (tableInOut, 
     Node_coord, AUTORES_COLUMN_REAL1D, 
     coordField, AUTORES_FIELD_REAL1D);
  AutoRes_TableInOut_associateColumnWithField 
    (tableInOut, 
     Node_boundaryFlag, AUTORES_COLUMN_INTEGER,
     boundaryFlagField, AUTORES_FIELD_INTEGER);

  AutoRes_TableInOut_do (tableInOut, fileName, writeFlag, binaryFlag);

  AutoRes_TableInOut_delete (tableInOut);
}

static void MetaFileNode 
(const char *fileName,
 int writeFlag,
 int binaryFlag)
{
  AutoRes_TableInOut *tableInOut;
  AutoRes_InOut *inOut;
  
  tableInOut = AutoRes_TableInOut_New (NodeTable);

  inOut = AutoRes_TableInOut_inOut (tableInOut);

  AutoRes_InOut_readMetaData (inOut, "node2.cdm");

  AutoRes_IntegerField_associate 
    (AutoRes_InOut_headerFieldOfName (inOut, "IntegerValue"), 
     &IntegerValue);
  AutoRes_RealField_associate 
    (AutoRes_InOut_headerFieldOfName (inOut, "RealValue"), 
     &RealValue);

  AutoRes_TableInOut_associateColumnOfNameWithFieldOfName 
    (tableInOut, "coord", "coord");
  AutoRes_TableInOut_associateColumnOfNameWithFieldOfName 
    (tableInOut, "boundaryFlag", "boundaryFlag");

  AutoRes_TableInOut_do (tableInOut, fileName, writeFlag, binaryFlag);

  AutoRes_TableInOut_delete (tableInOut);
}

static void TestTableInOut (void)
{
  int writeFlag;
  int binaryFlag = 0;

  printf (" TestTableInOut \n"); 

  NodeTable = AutoRes_Table_New ("NodeTable");

  Node_coord = AutoRes_Real1DColumn_New (NodeTable, "coord", 3);
  AutoRes_Table_addReal1DColumn (NodeTable, Node_coord);

  Node_boundaryFlag = AutoRes_IntegerColumn_New (NodeTable, "boundaryFlag");
  AutoRes_Table_addIntegerColumn (NodeTable, Node_boundaryFlag);
  


  writeFlag = 0;
  FileNode ("node.txt", writeFlag, binaryFlag);

  PrintNode ();
  
  writeFlag = 1;
  FileNode ("node4.txt", writeFlag, binaryFlag);



  writeFlag = 0;
  MetaFileNode ("node4.txt", writeFlag, binaryFlag);

  PrintNode ();
  
  writeFlag = 1;
  MetaFileNode ("node5.txt", writeFlag, binaryFlag);



  AutoRes_Table_delete (NodeTable);
}



int main (int argc, char *argv[])
{
  AutoRes_Initialize (0);

  TestTableInOut ();

  return 0;
}

